@@ -14,7 +14,7 @@ __copyright__ = 'this file is in the public domain'
 import socket, sys, re
 
 # config
-host = 'r8.cg.nu'
+host = 'gozerbot.org'
 port = 5600
 passwd = 'mekker'
 printto = 'dunk_'
