@@ -31,19 +31,23 @@ def aliascheck(ievent):
         alias = aliases.data[cmnd]
         ievent.txt = ievent.txt.replace(cmnd, alias, 1)
         ievent.alias = alias
-    except:
+        ievent.aliased = cmnd
+    except (IndexError, KeyError):
         pass
 
 @locked
 def aliassave():
+    """ save the aliases data """
     aliases.save()
 
 @locked
 def aliasset(fromm, to):
+    """ set an alias """
     aliases.data[fromm] = to
 
 @locked
 def aliasdel(fromm):
+    """ delete an alias """
     try:
         del aliases.data[fromm]
         aliases.save()
@@ -52,5 +56,6 @@ def aliasdel(fromm):
         pass
 
 def aliasget(fromm):
+    """ retrieve an alias """
     if aliases.data.has_key(fromm): 
         return aliases.data[fromm]
