@@ -10,13 +10,14 @@ from gozerbot.generic import rlog, handle_exception, calledfrom
 from gozerbot.ircevent import makeargrest
 from gozerbot.dol import Dol
 from gozerbot.thr import start_new_thread
+from gozerbot.runner import runner
 import sys, copy
 
 class Callback(object):
 
     """ class representing a callback """
 
-    def __init__(self, func, prereq, plugname, kwargs, threaded=True):
+    def __init__(self, func, prereq, plugname, kwargs, threaded=False):
         # the callback function
         self.func = func
         # pre condition function
@@ -41,7 +42,7 @@ class Callbacks(object):
         """ return nr of callbacks """
         return len(self.cbs)
 
-    def add(self, what, func, prereq=None, kwargs=None, threaded=True, \
+    def add(self, what, func, prereq=None, kwargs=None, threaded=False, \
 nr=False):
         """ add a callback """
         what = what.upper()
@@ -130,7 +131,9 @@ str(cb.func))
             if cb.threaded:
                 start_new_thread(cb.func, (bot, ievent), cb.kwargs)
             else:
-                cb.func(bot, ievent, **cb.kwargs)
+                runner.put("cb-%s" % cb.plugname, cb.func, bot, ievent, \
+**cb.kwargs)
+                #cb.func(bot, ievent, **cb.kwargs)
         except Exception, ex:
             handle_exception()
 
@@ -172,7 +175,9 @@ str(cb.func))
             if cb.threaded:
                 start_new_thread(cb.func, (bot, ievent), cb.kwargs)
             else:
-                cb.func(bot, ievent, **cb.kwargs)
+                runner.put("cb-%s" % cb.plugname, cb.func, bot, ievent, \
+**cb.kwargs)
+                #cb.func(bot, ievent, **cb.kwargs)
         except Exception, ex:
             handle_exception()
 
