@@ -11,28 +11,7 @@ from gozerbot.datadir import datadir
 import os, pickle, subprocess
 
 # version string
-ver = 'GOZERBOT 0.7.1.1'
-
-def gethgrev():
-    """ get mercurial revision """
-    rev = None
-    try:
-        rev = subprocess.Popen(['hg', 'parents'], stderr=subprocess.PIPE, stdout=subprocess.PIPE).communicate()[0].split(':')[1].strip()
-    except Exception, ex:
-        pass
-    return rev
-
-def getsvnrev():
-    """ get subversion revision """
-    rev = None
-    try:
-        lines = subprocess.Popen(['svn', 'info'], stderr=subprocess.PIPE, stdout=subprocess.PIPE).communicate().splitlines()
-        for line in lines:
-            if line.startswith('Revision'):
-                rev = line.split()[1]
-    except Exception, ex:
-        pass
-    return rev
+ver = 'GOZERBOT 0.8.1.0 RELEASE'
 
 def diffdict(dictfrom, dictto):
     """ check for differences between two dicts """
@@ -86,20 +65,13 @@ class Config(dict):
         configmodtime = os.stat(self.dir + os.sep + 'config')[8]
         if picklemodtime and picklemodtime > configmodtime:
             # if so update the config dict with the pickled data
-            a = diffdict(self, frompickle)
-            self.update(a)
+            # a = diffdict(self, frompickle)
+            self.update(frompickle)
         # set version
         if self['dbenable']:
             self['version'] = ver + ' ' + self['dbtype'].upper()
         else:
             self['version'] = ver
-        rev = gethgrev()
-        if rev:
-            self['version'] += " HG revision %s" % rev
-            return
-        rev = getsvnrev()
-        if rev:
-            self['version'] += " SVN revision %s" % rev
     
     def save(self):
         """ save config data to pickled file """
@@ -120,20 +92,14 @@ class Config(dict):
             self['version'] = ver + ' ' + self['dbtype'].upper()
         else:
             self['version'] = ver
-        rev = gethgrev()
-        if rev:
-            self['version'] += " HG revision %s" % rev
-            return
-        rev = getsvnrev()
-        if rev:
-            self['version'] += " SVN revision %s" % rev
 
     def defaultconfig(self):
         """ init default config values if no config file is found """
         self['loglevel'] = 100
         self['jabberenable'] = 0
+        self['ircdisable'] = 0
         self['stripident'] = 1
-        self['owneruserhost'] = 'bart@127.0.0.1'
+        self['owneruserhost'] = ['bart@127.0.0.1', ]
         self['nick'] = 'gozerbot'
         self['server'] = 'localhost'
         self['port'] = 6667
@@ -169,16 +135,19 @@ jabberowner = 'bartholo@localhost'
 jabberhost = 'localhost'
 jabberuser = 'gozerbot@localhost'
 jabberpass = 'pass'
+jabberoutsleep = 0.1
 
 ## irc section:
 
+ircdisable = 0
+
 # stripident .. enable stripping of ident from userhost
 stripident = 1
 
 # userhost of owner .. make sure this matches your client's userhost
 # if it doesn't match you will get an userhost denied message when you
 # try to send commands to the bot
-owneruserhost = 'bart@127.0.0.1'
+owneruserhost = ['bart@127.0.0.1', ]
 
 # the nick the bot tries to use, only used if no nick is set
 # otherwise the bot will use the last nick used by the !nick command
@@ -217,10 +186,14 @@ nolimiter = 0
 # quit message
 quitmsg = 'http://gozerbot.org'
 
+# nickserv .. set pass to enable nickserv ident
+nickservpass = ""
+nickservtxt = ['set unfiltered on', ]
+
 ## if you want to use a database:
 
 dbenable = 0 # set to 1 to enable
-dbtype = 'mysql' # one of mysql, postgres or sqlite
+dbtype = 'mysql' # one of mysql or sqlite
 dbname = "gb_db"
 dbhost = "localhost"
 dbuser = "bart"
@@ -241,6 +214,22 @@ udpallowednicks = ['#dunkbots', 'dunker']
 udppassword = 'mekker'
 udpseed = "" # set this to 16 char wide string if you want to encrypt the data
 udpstrip = 1 # strip all chars < char(32)
+udpsleep = 0 # sleep in sendloop .. can be used to delay packet traffic
+
+# tcp
+tcp = 0 # set to 1 to enable
+partytcp = 0
+tcpipv6 = 0
+tcphost = 'localhost'
+tcpport = 5500 
+tcpmasks = ['192.168*', ]
+tcpallow = ['127.0.0.1', ]
+tcpallowednicks = ['#dunkbots', 'dunker', 'dunker@jabber.xs4all.nl']
+tcppassword = 'mekker'
+tcpseed = "bla1234567890bla"
+# set this to 16 char wide string if you want to encrypt the data
+tcpstrip = 1
+tcpsleep = 0   
 
 ## other stuff:
 
@@ -260,8 +249,14 @@ collboot = "gozerbot.org:8088"
 # name of the main bot
 mainbotname = 'main'
 
-# allowed characters for strippedtxt
+# allowed character for strippedtxt
 allowedchars = []
+
+# set to 1 to allow addons
+addonallow = 0
+
+# enable loadlist
+loadlist = 0
 """
 
 def writeconfig():
@@ -271,5 +266,45 @@ def writeconfig():
         cfgfile.write(configtxt)
         cfgfile.close()
 
+loadlist = """
+core
+misc
+irc
+not
+grep
+reverse
+count
+chanperm
+choice
+fleet
+ignore
+upgrade
+job
+reload
+rest
+tail
+user
+googletalk
+all
+at
+backup
+install
+reload
+tell
+reverse
+to
+underauth
+userstate
+alias
+nickserv
+"""
+
+def writeloadlist():
+    """ write loadlist to datadir """
+    if not os.path.isfile(datadir + os.sep + 'loadlist'):
+        cfgfile = open(datadir + os.sep + 'loadlist', 'w')
+        cfgfile.write(loadlist)
+        cfgfile.close()
+
 # create the config dict and load it from file
 config = Config(datadir)
