@@ -8,9 +8,43 @@ __copyright__ = 'this file is in the public domain'
 
 from gozerbot.datadir import datadir
 from gozerbot.config import config
-import time, sys, re, traceback, Queue, urllib, urllib2, urlparse, socket, random
-import os, sgmllib, thread, popen2, types, calendar, httplib, string, glob
-import htmlentitydefs
+from stat import ST_UID, ST_MODE, S_IMODE
+from subprocess import Popen, PIPE
+import time, sys, re, traceback, Queue, urllib, urllib2, urlparse, socket
+import random, os, sgmllib, thread, types, calendar, httplib, string, StringIO
+import glob, htmlentitydefs, tempfile, pickle, getopt
+
+def getversion():
+    version = config['version']
+    rev = gethgrev()
+    if rev:
+        version += " HG revision %s" % rev
+    else:
+        rev = getsvnrev()
+        if rev:
+            version += " SVN revision %s" % rev
+    return version
+
+def gethgrev():
+    """ get mercurial revision """
+    rev = None
+    try:
+        rev = int(gozerpopen(['hg', 'tip']).fromchild.readlines()[0].split(':')[1].strip())
+    except Exception, ex:
+        pass
+    return rev
+
+def getsvnrev():
+    """ get subversion revision """
+    rev = None
+    try:
+        lines = gozerpopen(['svn', 'info']).fromchild.readlines()
+        for line in lines:
+            if line.startswith('Revision'):
+                rev = int(line.split()[1])
+    except Exception, ex:
+        pass
+    return rev
 
 try:
     import chardet
@@ -26,24 +60,65 @@ hostnamere = re.compile('^[\w\.]+$')
 logging_enabled = False
 logfile = None
 
+def makeoptions(ievent, options):
+    try:
+        optargs = ""
+        optlist = []
+        for i,j in options.iteritems():
+            if '--' in i:
+                optlist.append("%s=" % i[2:])
+                if j:
+                    optlist.append(j)
+                continue
+            if '-' in i:
+                if j:
+                    optargs += ":%s" % i[1:]
+                else:
+                    optargs += i[1:]
+        args = ievent.txt.split()
+        (opts, rest) = getopt.getopt(args[1:], optargs, optlist)
+        o = dict(options)
+        o.update(dict(opts))
+        ievent.options = o
+        ievent.txt = args[0] + ' ' + ' '.join(rest)
+    except getopt.GetoptError, ex:
+        ievent.reply(str(ex))
+        return 0
+    except Exception, ex:
+        handle_exception()
+        return 0
+    return 1
+
 def enable_logging():
+    """ enable logging """
     global logging_enabled
     logging_enabled = True
 
 if config.has_key('logfile'):
-    def openlogfile(filename, mode="a+"):
+    def openlogfile(fname, mode="a+"):
+        """ open log file """
         try:
-            f = open(filename, mode)
+            f = open(fname, mode)
         except IOError, e:
             sys.stderr.write("[%s] (%s) %s" % \
                 (time.strftime("%H:%M:%S"), "logfile", str(e)))
             return None
         else:
             return f
-    filename = os.path.join(datadir, config['logfile'])
-    logfile = openlogfile(filename)
+    fname = os.path.join(datadir, config['logfile'])
+    logfile = openlogfile(fname)
+
+
+def setdefenc(encoding):
+    import sys
+    reload(sys)
+    sys.setdefaultencoding(encoding)
+
+def plugfile():
+    return datadir + os.sep + calledfrom(sys._getframe())
 
 def cchar(bot, ievent):
+    """ return control char of bot/ievent's channel """
     try:
         cchar = bot.channels[ievent.channel]['cc']
     except LookupError:
@@ -52,10 +127,11 @@ def cchar(bot, ievent):
         cchar = config['defaultcc'] or '!'
     return cchar
 
-def splittxt(what):
+def splittxt(what, l=375):
+    """ split text into <l> char portions """
     txtlist = []
     start = 0
-    end = 375
+    end = l
     length = len(what)
     for i in range(length/end+1):
         endword = what.find(' ', end)
@@ -65,7 +141,7 @@ def splittxt(what):
         if res:
             txtlist.append(res)
         start = endword
-        end = start + 375
+        end = start + l
     return txtlist
     
 class istr(str): 
@@ -97,7 +173,10 @@ popenlocked = lockdec(popenlock)
 
 class PopenWhitelistError(Exception):
 
+    """ exception to be raised if popen arguments contain options """
+
     def __init__(self, item):
+        Exception.__init__(self)
         self.item = item
         
     def __str__(self):
@@ -105,22 +184,51 @@ class PopenWhitelistError(Exception):
 
 class PopenListError(Exception):
 
+    """ exception to tbe raised if popen argument is not a list """
+
     def __init__(self, item):
+        Exception.__init__(self)
         self.item = item
         
     def __str__(self):
         return str(self.item)
 
-class Gozerpopen4(popen2.Popen4):
+class GozerStringIO(StringIO.StringIO):
+    def readlines(self):
+        return self.read().split('\n')
+
+class Gozerpopen4(Popen):
+
+    """ class providing a close method to subprocess """
+
+    def __init__(self, args):
+        Popen.__init__(self, args, shell=False, stdin=PIPE, stdout=PIPE, \
+stderr=PIPE, close_fds=True)
+        self.fromchild = self.stdout
+        self.tochild   = self.stdin
+        self.errors    = self.stderr
 
     def close(self):
-        result = self.wait()
-        self.fromchild.close()
-        self.tochild.close()
-        return result
+        self.wait()
+        try:
+            self.stdin.close()
+        except:
+            pass
+        try:
+            self.stdout.close()
+        except:
+            pass
+        try:
+            self.errors.close()
+        except:
+            pass
+        return self.returncode
 
 @popenlocked
 def gozerpopen(args, userargs=[]):
+    """ function to use to run a subprocess .. enforces list argument and checks 
+        for options passed 
+    """
     if type(args) != types.ListType:
         raise PopenListError(args)
     if type(userargs) != types.ListType:
@@ -140,11 +248,30 @@ def reboot():
     """ reboot the bot """
     os.execl(sys.argv[0], *sys.argv)
 
+def reboot_stateful(bot, ievent, fleet, partyline):
+    """ reboot the bot, but keep the connections """
+    config.reload()
+    session = {'bots': {}, 'name': bot.name, 'channel': ievent.channel, \
+'partyline': []}
+    for i in fleet.bots:
+        session['bots'].update(i._resumedata())
+    session['partyline'] = partyline._resumedata()
+    sessionfile = tempfile.mkstemp('-session', 'gozerbot-')[1]
+    pickle.dump(session, open(sessionfile, 'w'))
+    fleet.save()
+    os.execl(sys.argv[0], sys.argv[0], '-r', sessionfile)
+
 def calledfrom(frame):
     """ return plugin from which function is called """
     try:
         plugname = frame.f_back.f_code.co_filename
         name = plugname.split(os.sep)[-1][:-3]
+        if name == '__init__':
+            # directory plugin
+            name = plugname.split(os.sep)[-2]
+        if name == 'generic':
+            plugname = frame.f_back.f_back.f_code.co_filename
+            name = plugname.split(os.sep)[-1][:-3]
     except AttributeError:
         name = None
     del frame
@@ -153,7 +280,7 @@ def calledfrom(frame):
 def stripident(userhost):
     """ strip ident char from userhost """
     try:
-        jid = userhost.getNode()
+        userhost.getNode()
         return str(userhost)
     except AttributeError:
         pass
@@ -176,8 +303,9 @@ def stripidents(ulist):
     return result
 
 def rlogfile(level, descr, txt):
+    """ log to file """
     global logfile
-    if logfile==None or logfile.closed:
+    if logfile == None or logfile.closed:
         return
     if level >= config['loglevel']:
         try:
@@ -193,36 +321,60 @@ descr, txt))
 
 def rlog(level, descr, txt, nofile=False):
     """ log text if level >= loglevel """
-    global logging_enabled
     if not logging_enabled:
         return
     if level >= config['loglevel']:
         txt = str(txt)
-        sys.stdout.write("[%s] (%s) %s\n" % (uurminsec(time.time()), descr, \
+        t = time.time()
+        sys.stdout.write("[%s %s] (%s) %s\n" % (dmy(t), uurminsec(t), descr, \
 txt))
         sys.stdout.flush()
         if not nofile and logfile:
             rlogfile(level, descr, txt)
 
-def handle_exception(ievent=None, log=True):
+directlist = ['urlopen error timed out', '[Errno socket error] timed out', \
+'Name or service not known', 'No route to host', 'Non-recoverable failure in \
+name resolution']
+
+def handle_exception(ievent=None, log=True, short=False, txt=""):
     """ print exception trace to sys.stderr """
     result = ""
     exctype, excvalue, tb = sys.exc_info()
+    excvalue = str(excvalue)
+    if short:
+        if ievent:
+            ievent.reply(excvalue)
+        else:
+            rlog(10, 'generic', 'EXCEPTION: %s' % excvalue)
+        return
+    for i in directlist:
+        if i in excvalue:
+           if ievent:
+               ievent.reply(excvalue)
+           else:
+               if txt:
+                   rlog(10, 'generic', "%s: %s" % (txt, i))
+               else:
+                   rlog(10, 'generic', i)
+           return
     trace = traceback.extract_tb(tb)
     for i in trace:
-        filename = i[0].split(os.sep)[-1]
+        fname = i[0].split(os.sep)[-1]
         linenr = i[1]
         func = i[2]
-        result += "%s:%s %s | " % (filename, linenr, func)
-    errormsg = "EXCEPTION: %s%s: %s" % (result, exctype, excvalue)
+        result += "%s:%s %s | " % (fname, linenr, func)
+    if txt:
+        errormsg = "EXCEPTION: %s: %s%s: %s" % (txt, result, exctype, excvalue)
+    else:
+        errormsg = "EXCEPTION: %s%s: %s" % (result, exctype, excvalue)
     exceptionlist.append(errormsg)
     if ievent:
         ievent.reply(errormsg)
     if log:
         rlog(100, "ALERT", "EXCEPTION !!")
         traceback.print_exc(sys.stderr)
-	if logfile:
-	    traceback.print_exc(logfile)
+        if logfile:
+            traceback.print_exc(logfile)
     del trace
     
 def getlistensocket(listenip):
@@ -326,7 +478,7 @@ def hourmin(ttime):
     else:
         result += str(timeres[4])
     return result
-
+	
 timere = re.compile('(\S+)\s+(\S+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s+(\d+)')
 
 bdmonths = ['Bo', 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', \
@@ -341,6 +493,13 @@ def striptime(what):
     what = re.sub('\s+', ' ', what)
     return what
 
+def now():
+    if time.daylight:
+        ttime = time.ctime(time.time() + int(time.timezone) + 3600)
+    else:
+        ttime = time.ctime(time.time() + int(time.timezone))
+    return ttime
+
 def today():
     """ get time for today 00:00 """
     if time.daylight:
@@ -497,8 +656,10 @@ def waitforuser(bot, userhost, timeout=15):
     return result
 
 def useragent():
+    """ provide useragent string """
     (name, version) = config['version'].split()[0:2]
-    return 'Mozilla/5.0 (compatible; %s %s; http://www.gozerbot.org)' % (name, version)
+    return 'Mozilla/5.0 (compatible; %s %s; http://www.gozerbot.org)' % \
+(name, version)
 
 class CBURLopener(urllib.FancyURLopener):
 
@@ -520,23 +681,30 @@ def geturl(url, version=None):
     result.close()
     return tmp
 
-def geturl2(url):
+def geturl2(url, decode=False):
     """ use urllib2 to fetch an url """
-    rlog(10, 'generic', 'fetching %s' % url)
+    rlog(5, 'generic', 'fetching %s' % url)
     request = urllib2.Request(url)
     request.add_header('User-Agent', useragent())
     opener = urllib2.build_opener()
     result = opener.open(request)
-    tmp = istr(result.read())
-    tmp.info = result.info() # add header information to .info attribute
+    tmp = result.read()
+    info = result.info() # add header information to .info attribute
     result.close()
-    return tmp
+    if decode:
+        encoding = get_encoding(tmp)
+        rlog(0, 'geturl2', '%s encoding: %s' % (url, encoding))
+        res = istr(fromenc(tmp, encoding, url))
+    else:
+        res = istr(tmp)
+    res.info = info
+    return res
 
 def posturl(url, myheaders, postdata):
     """ very basic HTTP POST url retriever """
     # build headers
-    headers = {'Content-Type': 'application/x-www-form-urlencoded', 'Accept': 'text/plain; text/html',
-        'User-Agent': useragent()}
+    headers = {'Content-Type': 'application/x-www-form-urlencoded', \
+'Accept': 'text/plain; text/html', 'User-Agent': useragent()}
     headers.update(myheaders)
     # parse URL components
     urlparts = urlparse.urlparse(url)
@@ -558,19 +726,25 @@ def decode_html_entities(s):
     re_entn_match = re.compile(u'&#([^;]+);')
     s = s.decode('utf-8', 'replace')
     def to_entn(match):
+        """ convert to entities """
         if htmlentitydefs.entitydefs.has_key(match.group(1)):
-            return htmlentitydefs.entitydefs[match.group(1)].decode('utf-8', 'replace')
+            return htmlentitydefs.entitydefs[match.group(1)].decode('utf-8', \
+'replace')
         return match.group(0)
     def to_utf8(match):
+        """ convert to utf-8 """
         return unichr(long(match.group(1)))
     s = re_ent_match.sub(to_entn, s)
     s = re_entn_match.sub(to_utf8, s)
     return s
 
 def get_encoding(data):
+    """ get encoding from web data """
     # first we try if we have the .info attribute to determine the encoding from
-    if hasattr(data, 'info') and data.info.has_key('content-type') and 'charset' in data.info['content-type'].lower():
-        charset = data.info['content-type'].lower().split('charset', 1)[1].strip()
+    if hasattr(data, 'info') and data.info.has_key('content-type') and \
+'charset' in data.info['content-type'].lower():
+        charset = data.info['content-type'].lower().split('charset', 1)[1].\
+strip()
         if charset[0] == '=':
             charset = charset[1:].strip()
             if ';' in charset:
@@ -582,11 +756,15 @@ def get_encoding(data):
         metas = re.findall(u'<meta[^>]+>', data, re.I | re.M)
         if metas:
             for meta in metas:
-                test_http_equiv = re.search('http-equiv\s*=\s*[\'"]([^\'"]+)[\'"]', meta, re.I)
-                if test_http_equiv and test_http_equiv.group(1).lower() == 'content-type':
-                    test_content = re.search('content\s*=\s*[\'"]([^\'"]+)[\'"]', meta, re.I)
+                test_http_equiv = \
+re.search('http-equiv\s*=\s*[\'"]([^\'"]+)[\'"]', meta, re.I)
+                if test_http_equiv and \
+test_http_equiv.group(1).lower() == 'content-type':
+                    test_content = \
+re.search('content\s*=\s*[\'"]([^\'"]+)[\'"]', meta, re.I)
                     if test_content:
-                        test_charset = re.search('charset\s*=\s*([^\s\'"]+)', meta, re.I)
+                        test_charset = \
+re.search('charset\s*=\s*([^\s\'"]+)', meta, re.I)
                         if test_charset:
                             return test_charset.group(1)
     # everything else failed, let's see if we can use chardet
@@ -617,15 +795,15 @@ class Stripper(sgmllib.SGMLParser):
 
     def handle_data(self, data):
         """ data handler """
-        self.theString += toascii(data)
+        self.theString += fromenc(data)
 
 def striphtml(txt):
     """ strip html from txt """
     stripper = Stripper()
-    txt = stripper.strip(toascii(txt))
+    txt = stripper.strip(fromenc(txt))
     return txt
     
-def waitforqueue(queue, timeout=10):
+def waitforqueue(queue, timeout=10, maxitems=None):
     """ wait for queue output """
     result = []
     while 1:
@@ -636,27 +814,74 @@ def waitforqueue(queue, timeout=10):
         if not res:
             break
         result.append(res)
+        if maxitems and len(result) == maxitems:
+            break
     return result
 
-defenc = sys.getdefaultencoding()
+def decodeperchar(txt, encoding='utf-8', what=""):
+    res = []
+    nogo = []
+    for i in txt:
+        try:
+            res.append(i.decode(encoding))
+        except UnicodeDecodeError:
+            if i not in nogo:
+                nogo.append(i)
+            continue
+    if nogo:
+        if what:
+            rlog(5, 'decoding', "%s: can't decode %s characters to %s" % \
+(what, nogo, encoding))
+        else:
+            rlog(5, 'decoding', "can't decode %s characters to %s" % \
+(nogo, encoding))
+    return u"".join(res)
+
+def toenc(what, encoding='utf-8'):
+    if not what:
+        return u""
+    try:
+        w = unicode(what)
+        return w.encode(encoding)
+    except UnicodeEncodeError:
+        rlog(10, 'encoding', "can't encode %s to %s" % (what, encoding))
+        return u""
+
+def fromenc(txt, encoding='utf-8', what=""):
+    if not txt:
+        return u""
+    try:
+        if type(txt) == types.UnicodeType:
+            t = txt.encode(encoding)
+        t = unicode(txt)
+        return unicode(t.decode(encoding))
+    except UnicodeDecodeError:
+        return decodeperchar(txt, encoding, what)
+
 def toascii(what):
-    what = what.decode(defenc, 'replace')
+    """ convert to ascii """
+    what = what.encode('ascii', 'replace')
     return what
 
 def tolatin1(what):
-        what = what.decode('latin-1', 'replace')
-        return what
+    """ convert to latin-1 """
+    what = what.encode('latin-1', 'replace')
+    return what
 
 def strippedtxt(what):
-    strippedtxt = ""
-    printchars = string.letters + string.digits + string.punctuation + '' \
-+ ' '
+    """ strip txt to barebones """
+    txt = []
+    if config['allowedchars']:
+        allowed = config['allowedchars'] + ['\001', '\002', '\003', '\t']
+    else:
+        allowed = ['\001', '\002', '\003', '\t']
     for i in what:
-        if i in printchars:
-            strippedtxt += i
-    return strippedtxt
+        if ord(i) > 31 or (allowed and i in allowed):
+            txt.append(i)
+    return ''.join(txt)
 
 def uniqlist(l):
+    """ return unique values of list """
     result = []
     for i in l:
         j = i.strip()
@@ -665,6 +890,7 @@ def uniqlist(l):
     return result
 
 def fix_format(s):
+    """ make sure bold txt is completed """ 
     counters = {
         chr(2): 0, 
         chr(3): 0
@@ -678,11 +904,25 @@ def fix_format(s):
     return s
 
 def stripbold(s):
+    """ remove bold from string """
     s = s.replace(chr(2), '')
     s = s.replace(chr(3), '')
     return s
 
+def jabberstrip(what):
+    """ strip jabber txt to barebones """
+    txt = []
+    if not config['allowedchars']:
+        allowed = ['\n', '\t']
+    else:
+        allowed = config['allowedchars'] + ['\n', '\t']
+    for i in what:
+        if ord(i) > 31 or (allowed and i in allowed):
+            txt.append(i)
+    return ''.join(txt)
+
 def plugnames(dirname):
+    """ get all plugnames in dir """
     result = []
     for i in glob.glob(dirname + os.sep + '*.py'):
         result.append(i.split(os.sep)[-1][:-3])
@@ -692,14 +932,43 @@ def plugnames(dirname):
         pass
     return result
 
-def touch(filename):
-    fd = os.open(filename, os.O_WRONLY | os.O_CREAT)
+def filesize(path):
+    """ function to determine file size """
+    return os.stat(path)[6]
+
+def touch(fname):
+    """ touch filename """
+    fd = os.open(fname, os.O_WRONLY | os.O_CREAT)
     os.close(fd)  
 
 def stringinlist(s, l):
+    """ check if string is substring of the lists elements """
     for i in l:     
         if s in i:  
             return 1
 
 def stripped(userhost):
+    """ strip resource from userhost """ 
     return userhost.split('/')[0]
+
+def checkpermissions(ddir, umode):
+    """ set permissions of dir to umode """
+    uid = os.getuid()  
+    gid = os.getgid()
+    try:
+        stat = os.stat(ddir)
+    except OSError:
+        return
+    if stat[ST_UID] != uid:
+        try:
+            os.chown(ddir, uid, gid)
+            rlog(10, 'generic', 'chowned %s dir to %s:%s' % (ddir, uid, gid)) 
+        except:
+            pass
+    if S_IMODE(stat[ST_MODE]) != umode:
+        try:
+            os.chmod(ddir, umode)
+            rlog(10, 'generic', 'set umode of %s dir to %o' % (ddir, umode))
+        except:
+            handle_exception()
+            pass
