@@ -8,7 +8,8 @@ __copyright__ = 'this file is in the public domain'
 
 from gozerbot.persist import Persist
 from gozerbot.datadir import datadir
-import time, os
+from gozerbot.periodical import interval
+import time, os, thread
 
 ignore = Persist(datadir + os.sep + 'ignore')
 timeset = Persist(datadir + os.sep + 'ignoretimeset')
@@ -31,6 +32,8 @@ def delignore(userhost):
     try:
         del ignore.data[userhost]
         ignore.save()
+        del timeset.data[userhost]
+        timeset.save()
         return 1
     except KeyError:
         return 0
@@ -45,3 +48,11 @@ def shouldignore(userhost):
     if time.time() - ignoretime < ignoreset:
         return 1
     return 0
+
+@interval(1)
+def ignorecheck():
+    for userhost in ignore.data.keys():
+        if not shouldignore(userhost):
+            delignore(userhost)
+
+ignorecheck()
\ No newline at end of file
