@@ -7,8 +7,8 @@
 
 __copyright__ = 'this file is in the public domain'
 
-from gozerbot.generic import rlog, handle_exception, getrandomnick, todefenc, \
-fix_format, splittxt, waitforqueue
+from gozerbot.generic import rlog, handle_exception, getrandomnick, toenc, \
+fix_format, splittxt, waitforqueue, strippedtxt, fromenc, uniqlist
 from gozerbot.wait import Wait
 from gozerbot.config import config
 from gozerbot.ircevent import Ircevent
@@ -19,7 +19,7 @@ from gozerbot.pdod import Pdod
 from gozerbot.datadir import datadir
 from gozerbot.eventhandler import Outputhandler
 from gozerbot.fleet import fleet
-
+from gozerbot.botbase import BotBase
 import gozerbot.thr as thr
 import time, thread, socket, threading, os, Queue
 
@@ -35,73 +35,58 @@ class AlreadyConnecting(Exception):
 
     pass
 
-class Irc(object):
+class Irc(BotBase):
 
     """ the irc class, provides interface to irc related stuff """
 
-    def __init__(self, name='main'):
-        self.name = name 
+    def __init__(self, name='main', owner=[]):
+        BotBase.__init__(self, name, owner)
         self.type = 'irc'
         self.wait = Wait()
-        self.lastoutput = 0
         self.outputlock = thread.allocate_lock()
-        self.connectok = threading.Event()
-        self.outputhandler = Outputhandler(self)
         self.fsock = None
+        self.oldsock = None
         self.sock = None
-        self.nick = config['nick'] or 'gb1'
-        self.orignick = config['orignick'] or self.nick
-        self.server = 'localhost'
-        self.port = 6667
-        self.password = None
-        self.ipv6 = None
-        self.stopped = 0
         self.nolimiter = config['nolimiter']
         self.reconnectcount = 0
         self.pongcheck = 0
-        self.blocking = config['blocking'] or 1
         self.nickchanged = 0
         self.noauto433 = 0
-        self.connecting = False
-        self.less = Less(5)
-        self.state = Pdod(datadir + os.sep + '%s.state' % self.name)
         if not self.state.has_key('alternick'):
             self.state['alternick'] = config['alternick']
         if not self.state.has_key('no-op'):
             self.state['no-op'] = []
         if not self.state.has_key('nick'):
             self.state['nick'] = config['nick'] or 'gb1'
-        if not self.state.has_key('confignick'):
-            self.state['confignick'] = config['nick']
-        if config['nick'] != self.state['confignick']:
-            self.state['nick'] = config['nick']
-            self.state['confignick'] = config['nick']
-            self.state.save()
         self.nick = self.state['nick']
-        self.outputhandler.start()
+        self.nrevents = 0
+        self.gcevents = 0
+        self.outqueues = [Queue.Queue() for i in range(10)]
+        self.nicks401 = []
 
-    def connect(self, nick, server, port=6667, password=None, ipv6=None, \
-reconnect=True):
-        """ connect to server/port using nick .. connect can timeout so catch
-            exception .. reconnect if enabled """
-        if self.connecting:
-            rlog(10, self.name, 'already connecting')
-            raise AlreadyConnecting()
-        if self.connectok.isSet():
-            rlog(10, self.name, 'already connected')
-            raise AlreadyConnected()
+    def _raw(self, txt):
+        """ send raw text to the server """
+        if not txt:
+            return
+        rlog(2, self.name + '.sending', txt)
         try:
-            return self.doconnect(nick, server, port, password, ipv6)
+            if self.ssl:
+                self.sock.write(txt + '\n')
+            else:
+                self.sock.send(txt[:500] + '\n')
         except Exception, ex:
-            if self.stopped:
-                return 0
-            rlog(10, self.name, str(ex))
-            if reconnect:
-                return self.reconnect()
-        if not fleet.byname(self.name):
-            fleet.addbot(self)
+            # check for broken pipe error .. if so ignore 
+            # used for nonblocking sockets
+            try:
+                (errno, errstr) = ex
+                if errno != 32:
+                    raise
+                else:
+                    rlog(10, self.name, 'broken pipe error .. ignoring')
+            except:
+                rlog(10, self.name, "ERROR: can't send %s" % str(ex))
 
-    def doconnect(self, nick, server, port, password, ipv6):
+    def _connect(self, nick, server, port, password, ipv6, ssl=0):
         """ connect to server/port using nick """
         self.connecting = True
         self.connectok.clear()
@@ -109,34 +94,35 @@ reconnect=True):
         self.orignick = self.nick
         self.server = str(server)
         self.port = int(port)
-        self.password = password
-        self.ipv6 = ipv6
+        self.password = str(password)
+        self.ipv6 = int(ipv6)
+        self.ssl = int(ssl)
         # create socket
-        if ipv6:
-            rlog(1, self.name, 'creating ipv6 socket')
-            self.sock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
+        if self.ipv6:
+            rlog(10, self.name, 'creating ipv6 socket')
+            self.oldsock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
             self.ipv6 = 1
         else:
-            rlog(0, self.name, 'creating ipv4 socket')
-            self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            rlog(10, self.name, 'creating ipv4 socket')
+            self.oldsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         # optional bind
         elite = config['bindhost']
         if elite:
             try:
-                self.sock.bind((elite, 0))
+                self.oldsock.bind((elite, 0))
             except socket.gaierror:
                 rlog(10, self.name, "can't bind to %s" % elite)
         # do the connect .. set timeout to 30 sec upon connecting
         rlog(10, self.name, 'connecting to ' + self.server)
-        self.sock.settimeout(30)
-        self.sock.connect((self.server, int(self.port)))
+        self.oldsock.settimeout(30)
+        self.oldsock.connect((self.server, int(self.port)))
         # we are connected
         rlog(10, self.name, 'connection ok')
         self.stopped = 0
         # make file socket
-        self.fsock = self.sock.makefile("r")
+        self.fsock = self.oldsock.makefile("r")
         # set blocking
-        self.sock.setblocking(self.blocking)
+        self.oldsock.setblocking(self.blocking)
         self.fsock._sock.setblocking(self.blocking)
         # set socket time out
         if self.blocking:
@@ -145,45 +131,180 @@ reconnect=True):
                 socktimeout = 301.0
             else:
                 socktimeout = float(socktimeout)
-            self.sock.settimeout(socktimeout)
+            self.oldsock.settimeout(socktimeout)
             self.fsock._sock.settimeout(socktimeout)
         # start readloop
-        self.outputhandler.start()
-        rlog(0, self.name, 'starting readloop')
-        thr.start_new_thread(self.readloop, ())
+        if self.ssl:
+            rlog(10, self.name, 'ssl enabled')
+            self.sock = socket.ssl(self.oldsock) 
+        else:
+            self.sock = self.oldsock
+        thr.start_new_thread(self._readloop, ())
+        thr.start_new_thread(self._outloop, ())
         # execute onconnect file .. code that can be executed after login
-        self.logon()
-        self.onconnect()
+        self._logon()
+        self._onconnect()
         # init 
         self.reconnectcount = 0
         self.nickchanged = 0
-        self.outputhandler.nooutput = False
         self.connecting = False
+        saymonitor.start()
         return 1
 
-    def logon(self):
+    def _readloop(self):
+        """ loop on the socketfile """
+        self.stopped = 0
+        doreconnect = 0
+        timeout = 1
+        rlog(5, self.name, 'starting readloop')
+        prevtxt = ""
+        while not self.stopped:
+            try:
+                time.sleep(0.001)
+                if self.ssl:
+                    intxt = self.sock.read().split('\n')
+                else:
+                    intxt = self.fsock.readline().split('\n')
+                # if intxt == "" the other side has disconnected
+                if not intxt and not self.stopped:
+                    doreconnect = 1
+                    break
+                if self.stopped:
+                    break 
+                if prevtxt:
+                    intxt[0] = prevtxt + intxt[0]
+                    prevtxt = ""
+                if intxt[-1] != '':
+                    prevtxt = intxt[-1]
+                    intxt = intxt[:-1]
+                for r in intxt:
+                    r = r.rstrip()
+                    rr = fromenc(r)
+                    if not rr:
+                        continue
+                    res = strippedtxt(rr)
+                    res = rr
+                    rlog(2, self.name, res)
+                    # parse txt read into an ircevent
+                    try:
+                        ievent = Ircevent().parse(self, res)
+                    except Exception, ex:
+                        handle_exception()
+                        continue
+                    # call handle_ievent 
+                    if ievent:
+                        self.handle_ievent(ievent)
+                    timeout = 1
+            except UnicodeError:
+                handle_exception()
+                continue
+            except socket.timeout:
+                # timeout occured .. first time send ping .. reconnect if
+                # second timeout follows
+                if self.stopped:
+                    break
+                timeout += 1
+                if timeout > 2:
+                    doreconnect = 1
+                    rlog(10, self.name, 'no pong received')
+                    break
+                rlog(1, self.name, "socket timeout")
+                pingsend = self.ping()
+                if not pingsend:
+                    doreconnect = 1
+                    break
+                continue
+            except socket.sslerror, ex:
+                # timeout occured .. first time send ping .. reconnect if
+                # second timeout follows
+                if self.stopped:
+                    break
+                if not 'timed out' in str(ex):
+                    handle_exception()
+                    doreconnect = 1
+                    break
+                timeout += 1
+                if timeout > 2:
+                    doreconnect = 1
+                    rlog(10, self.name, 'no pong received')
+                    break
+                rlog(1, self.name, "socket timeout")
+                pingsend = self.ping()
+                if not pingsend:
+                    doreconnect = 1
+                    break
+                continue
+            except IOError, ex:
+                if 'temporarily' in str(ex):
+                    continue
+            except Exception, ex:
+                if self.stopped:
+                    break
+                err = ex
+                try:
+                    (errno, msg) = ex
+                except:
+                    errno = -1
+                    msg = err
+                # check for temp. unavailable error .. raised when using
+                # nonblocking socket .. 35 is FreeBSD 11 is Linux
+                if errno == 35 or errno == 11:
+                    time.sleep(0.5)
+                    continue
+                rlog(10, self.name, "error in readloop: %s" % msg)
+                doreconnect = 1
+                break
+        rlog(5, self.name, 'readloop stopped')
+        # see if we need to reconnect
+        self.connectok.clear()
+        self.connecting = False
+        self.connected = False
+        if doreconnect:
+            self.reconnect()
+
+    def _getqueues(self):
+        res = []
+        time.sleep(0.1)
+        for i in range(10):
+            q = self.outqueues[i]
+            if q.qsize():
+                res.append(q)
+        return res
+
+    def _outloop(self):
+        rlog(5, self.name, 'starting output loop')
+        while not self.stopped:
+            for queue in self._getqueues():
+                res = queue.get()
+                (printto, what, who, how, fromm, speed) = res
+                if not self.stopped and printto not in self.nicks401:
+                    self.out(printto, what, who, how, fromm, speed)
+        rlog(5, self.name, 'stopping output loop')
+
+    def _logon(self):
         """ log on to the network """
         # if password is provided send it
         if self.password:
             rlog(10, self.name ,'sending password')
-            self.send("PASS %s" % self.password)
+            self._raw("PASS %s" % self.password)
         # register with irc server
-        rlog(10, self.name, 'registering with server %s using nick: %s' % \
+        rlog(10, self.name, 'registering with %s using nick %s' % \
 (self.server, self.nick))
         rlog(10, self.name, 'this may take a while')
         # check for username and realname
-        username = config['username'] or self.nick
+        username = self.nick or config['username']
         realname = config['realname'] or username
         # first send nick
-        self.send("NICK %s" % self.nick)
+        self._raw("NICK %s" % self.nick)
         # send USER
-        self.send("USER %s localhost localhost :%s" % (username, realname))
+        self._raw("USER %s localhost localhost :%s" % (username, \
+realname))
         # wait on login
         self.connectok.wait()
         # registration ok
         rlog(10, self.name, 'logged on !')
 
-    def onconnect(self):
+    def _onconnect(self):
         """ run the onconnect-botname file after connect """
         onconnectfile = 'onconnect-%s' % self.name
         try:
@@ -201,105 +322,149 @@ reconnect=True):
                 rlog(5, self.name, 'error excecuting %s: %s' % \
 (onconnectfile, str(ex)))
 
-    def readloop(self):
-        """ loop on the socketfile """
+    def _resume(self, data, reto=None):
+        """ resume to server/port using nick """
+        try:
+            if data['ssl']:
+                return 0
+        except KeyError:
+            pass
+        self.connecting = False # we're already connected
+        self.nick = data['nick']
+        self.orignick = self.nick
+        self.server = str(data['server'])
+        self.port = int(data['port'])
+        self.password = data['password']
+        self.ipv6 = data['ipv6']
+        self.ssl = data['ssl']
+        # create socket
+        if self.ipv6:
+            rlog(1, self.name, 'resuming ipv6 socket')
+            self.sock = socket.fromfd(data['fd'], socket.AF_INET6, socket.SOCK_STREAM)
+            self.ipv6 = 1
+        else:
+            rlog(1, self.name, 'resuming ipv4 socket')
+            self.sock = socket.fromfd(data['fd'], socket.AF_INET, socket.SOCK_STREAM)
+        # do the connect .. set timeout to 30 sec upon connecting
+        rlog(10, self.name, 'resuming to ' + self.server)
+        self.sock.settimeout(30)
+        # we are connected
+        rlog(10, self.name, 'connection ok')
         self.stopped = 0
-        doreconnect = 0
-        timeout = 1
-        while not self.stopped:
-            try:
-                time.sleep(0.001)
-                res = todefenc(self.fsock.readline())
-                # if res == "" the other side has disconnected
-                if not res and not self.stopped:
-                    doreconnect = 1
-                    break
-                if self.stopped:
-                    break 
-                res = res.rstrip()
-                rlog(2, self.name, res)
-                # parse txt read into an ircevent
-                try:
-                    ievent = Ircevent().parse(self, res)
-                except Exception, ex:
-                    handle_exception()
-                    continue
-                # call handle_ievent 
-                if ievent:
-                    self.handle_ievent(ievent)
-                timeout = 1
-            except socket.timeout:
-                # timeout occured .. first time send ping .. reconnect if
-                # second timeout follows
-                if self.stopped:
-                    break
-                timeout += 1
-                if timeout > 2:
-                    doreconnect = 1
-                    rlog(10, self.name, 'no pong received')
-                    break
-                rlog(1, self.name, "socket timeout")
-                pingsend = self.ping()
-                if not pingsend:
-                    doreconnect = 1
-                    break
-                continue
-            except Exception, ex:
-                if self.stopped:
-                    break
-                err = ex
-                try:
-                    (errno, msg) = ex
-                except:
-                    errno = -1
-                    msg = err
-                # check for temp. unavailable error .. raised when using
-                # nonblocking socket .. 35 is FreeBSD 11 is Linux
-                if errno == 35 or errno == 11:
-                    time.sleep(0.5)
-                    continue
-                rlog(10, self.name, "error in readloop: %s" % msg)
-                self.outputhandler.nooutput = True
-                doreconnect = 1
-                break
-        rlog(10, self.name, 'readloop stopped')
-        # see if we need to reconnect
-        self.connectok.clear()
+        # make file socket
+        self.fsock = self.sock.makefile("r")
+        # set blocking
+        self.sock.setblocking(self.blocking)
+        # set socket time out
+        if self.blocking:
+            socktimeout = config['socktimeout']
+            if not socktimeout:
+                socktimeout = 301.0
+            else:
+                socktimeout = float(socktimeout)
+            self.sock.settimeout(socktimeout)
+        # start readloop
+        rlog(0, self.name, 'resuming readloop')
+        thr.start_new_thread(self._readloop, ())
+        thr.start_new_thread(self._outloop, ())
+        # init 
+        self.reconnectcount = 0
+        self.nickchanged = 0
         self.connecting = False
-        if doreconnect:
-            self.reconnect()
+        # still there server?
+        self._raw('PING :RESUME %s' % str(time.time()))
+        self.connectok.set()
+        self.connected = True
+        if reto:
+            self.say(reto, 'rebooting done')
+        return 1
+
+    def _resumedata(self):
+        try:
+            fd = self.sock.fileno()
+        except:
+            fd = None
+            self.exit()
+        return {self.name: {
+            'nick': self.nick,
+            'server': self.server,
+            'port': self.port,
+            'password': self.password,
+            'ipv6': self.ipv6,
+            'ssl': self.ssl,
+            'fd': fd
+            }}
+
+    def broadcast(self, txt):
+        for i in self.state['joinedchannels']:
+            self.say(i, txt, speed=1)
+
+    def save(self):
+        self.state.save()
+
+    def connect(self, nick, server, port=6667, password="", ipv6=0, \
+ssl=0, reconnect=True):
+        """ connect to server/port using nick .. connect can timeout so catch
+            exception .. reconnect if enabled """
+        if self.connecting:
+            rlog(10, self.name, 'already connecting')
+            raise AlreadyConnecting()
+        if self.connectok.isSet():
+            rlog(10, self.name, 'already connected')
+            raise AlreadyConnected()
+        try:
+            return self._connect(nick, server, port, password, ipv6, ssl)
+        except Exception, ex:
+            if self.stopped:
+                return 0
+            rlog(10, self.name, 'connecting: %s' % str(ex))
+            if reconnect:
+                return self.reconnect()
+            raise
+        if not fleet.byname(self.name):
+            fleet.addbot(self)
 
     def shutdown(self):
         """ shutdown sockets """
         rlog(10, self.name, 'shutdown')
         self.stopped = 1
+        time.sleep(1)
+        self.outqueues[0].put_nowait(('mekker', 'mekker', 'mekker', 'mekker', \
+'mekker', 6))
         try:
-            self.sock.shutdown(2)
+            if self.ssl:
+                self.oldsock.shutdown(2)
+            else:
+                self.sock.shutdown(2)
         except:
             pass
         try:
-            time.sleep(1)
-            self.sock.close()
+            if self.ssl:
+                self.oldsock.close()
+            else:
+                self.sock.close()
             self.fsock.close()
         except:
             pass
         self.connecting = False
+        self.connected = False
         self.connectok.clear()
 
     def exit(self):
         """ exit the bot """
         self.stopped = 1
-        self.outputhandler.stop()
+        self.connected = False
         self.shutdown()
 
     def reconnect(self):
         """ reconnect to the irc server """
         if self.stopped:
             return 0
+        time.sleep(2)
         self.shutdown()
         rlog(10, self.name, 'reconnecting')
-        self.stopped = 0
         time.sleep(5)
+        self.stopped = 0
         # determine how many seconds to sleep
         if self.reconnectcount > 0:
             reconsleep = self.reconnectcount*60
@@ -309,7 +474,7 @@ reconnect=True):
             return 0
         self.reconnectcount += 1
         return self.connect(self.nick, self.server, self.port, \
-self.password, self.ipv6)
+self.password, self.ipv6, self.ssl)
 
     def handle_pong(self, ievent):
         """ set pongcheck on received pong """
@@ -318,42 +483,55 @@ self.password, self.ipv6)
 
     def sendraw(self, txt):
         """ send raw text to the server """
-        if not txt:
+        if not txt or self.stopped or not self.connected:
             return
         rlog(2, self.name + '.sending', txt)
         try:
-            self.sock.send(txt + '\n')
+            if self.ssl:
+                self.sock.write(txt + '\n')
+            else:
+                self.sock.send(txt[:500] + '\n')
         except Exception, ex:
-            rlog(10, self.name, "ERROR: can't send %s" % str(ex))
+            # check for broken pipe error .. if so ignore 
+            # used for nonblocking sockets
+            if not self.blocking and 'broken pipe' in str(ex):
+                rlog(10, self.name, 'broken pipe error .. ignoring')
+            else:
+                rlog(10, self.name, 'send error: %s' % str(ex))
+                self.reconnect()
 
     def fakein(self, txt):
         """ do a fake ircevent """
         if not txt:
             return
-        rlog(1, self.name + '.fakein', txt)
+        rlog(10, self.name + '.fakein', txt)
         self.handle_ievent(Ircevent().parse(self, txt))
 
     def say(self, printto, what, who=None, how='msg', fromm=None, speed=5):
         """ say what to printto """
-        if not printto or not what:
+        if not printto or not what or printto in self.nicks401:
             return
         # check if printto is a socket
-        if type(printto) == socket.SocketType:
+        if 'socket' in repr(printto):
             try:
                 printto.send(what + '\n')
             except Exception, ex :
-                time.sleep(0.01)
+                if "Broken pipe" in str(ex):
+                    return
+                handle_exception()
             return
         # if who is set add "who: " to txt
         if who:
             what = "%s: %s" % (who, what)
-        self.out(printto, what, who, how, fromm, speed) 
+        if speed > 10:
+            speed = 10
+        self.outqueues[10-speed].put((printto, what, who, how, fromm, speed))
 
     def out(self, printto, what, who=None, how='msg', fromm=None, speed=5):
         """ output the first 375 chars .. put the rest into cache """
         # first do mombo to get txt converted to ascii
         try:
-            what = todefenc(what.rstrip())
+            what = toenc(what.rstrip())
         except Exception, ex:
             rlog(10, self.name, "can't output: %s" % str(ex))
             return
@@ -374,12 +552,12 @@ self.password, self.ipv6)
                 result += " (+%s)" % size
         else:
             result = txtlist[0]
-        self.outputhandler.put(speed, self, printto, result, how, who, fromm)
+        self.output(printto, result, how, who, fromm)
 
     def output(self, printto, what, how='msg' , who=None, fromm=None):
         """ first output .. then call saymonitor """
         self.outputnolog(printto, what, how, who, fromm)
-        saymonitor.callcb(self.name, printto, what, who, how, fromm)
+        saymonitor.put(self.name, printto, what, who, how, fromm)
         
     def outputnolog(self, printto, what, how, who=None, fromm=None):
         """ do output to irc server .. rate limit to 3 sec """
@@ -411,20 +589,23 @@ self.password, self.ipv6)
             return
         self.noauto433 = 1
         queue = Queue.Queue()
-        self.wait.register('NICK', self.nick, queue, 5)
-        self.send('NICK %s' % nick)
+        nick = nick[:16]
+        self.wait.register('NICK', self.nick[:16], queue, 12)
+        self._raw('NICK %s\n' % nick)
         result = waitforqueue(queue)
         self.noauto433 = 0
         if not result:
             return 0
         self.nick = nick
+        self.whois(nick)
         if setorig:
             self.orignick = nick
         if save:
             self.state['nick'] = nick
             self.state.save()
-            if fleet.getmainbot() == self:
+            if self.name == 'jabbermain' or self.name == 'main':
                 config.set('nick', nick)
+                config.save()
         return 1
 
     def join(self, channel, password=None):
@@ -432,7 +613,7 @@ self.password, self.ipv6)
         if not channel:
             return
         q = Queue.Queue()
-        self.wait.register('353', channel, q, 5)
+        self.wait.register('JOIN', channel, q, 5)
         if password:
             self.send('JOIN %s %s' % (channel, password))
         else:
@@ -484,21 +665,24 @@ self.password, self.ipv6)
         """ send text to irc server """
         if not txt:
             return
-        if self.stopped:
+        if self.stopped or not self.connected:
             return
+        txt = toenc(strippedtxt(txt))
+        txt = txt.rstrip()
         rlog(2, self.name + '.send', txt)
         try:
-            self.sock.send(txt[:510]+'\n')
+            if self.ssl:
+                self.sock.write(txt + '\n')
+            else:
+                self.sock.send(txt[:500] + '\n')
         except Exception, ex:
-            # check for broken pipe error .. if so ignore
-            try:
-                (errno, errstr) = ex
-                if errno != 32:
-                    raise
-                else:
-                    rlog(10, self.name, 'broken pipe error .. ignoring')
-            except:
-                rlog(10, self.name, "ERROR: can't send %s" % str(ex))
+            # check for broken pipe error .. if so ignore 
+            # used for nonblocking sockets
+            if not self.blocking and 'broken pipe' in str(ex):
+                rlog(10, self.name, 'broken pipe error .. ignoring')
+            else:
+                rlog(10, self.name, 'send error: %s' % str(ex))
+                self.reconnect()
 
     def voice(self, channel, who):
         """ give voice """
@@ -518,11 +702,11 @@ self.password, self.ipv6)
             return
         self.send('MODE %s -o %s' % (channel, who))
 
-    def quit(self, reason='http://r8.cg.nu'):
+    def quit(self, reason='http://gozerbot.org'):
         """ send quit message """
         rlog(10, self.name, 'sending quit')
         try:
-            self.sock.send('QUIT :%s\n' % reason)
+            self._raw('QUIT :%s' % reason)
         except IOError:
             pass
 
@@ -553,6 +737,10 @@ self.password, self.ipv6)
     def handle_ievent(self, ievent):
         """ handle ircevent .. dispatch to 'handle_command' method """ 
         try:
+            if ievent.cmnd == 'JOIN' or ievent.msg:
+                if ievent.nick.lower() in self.nicks401:
+                    self.nicks401.remove(ievent.nick)
+                    rlog(10, self.name, '%s joined .. unignoring')
             # see if the irc object has a method to handle the ievent
             method = getattr(self,'handle_' + ievent.cmnd.lower())
             # try to call method
@@ -569,6 +757,10 @@ self.password, self.ipv6)
         except:
             handle_exception()
 
+    def handle_432(self, ievent):
+        """ erroneous nick """
+        self.handle_433(ievent)
+
     def handle_433(self, ievent):
         """ handle nick already taken """
         if self.noauto433:
@@ -593,11 +785,13 @@ using randomnick %s' % (nick, randomnick))
         """ send pong response """
         if not ievent.txt:
             return
-        self.send('PONG :%s' % ievent.txt)
+        self._raw('PONG :%s' % ievent.txt)
 
     def handle_001(self, ievent):
         """ we are connected  """
         self.connectok.set()
+        self.connected = True
+        self.whois(self.nick)
 
     def handle_privmsg(self, ievent):
         """ check if msg is ctcp or not .. return 1 on handling """
@@ -636,8 +830,17 @@ pingtime2)
         """ ping the irc server """
         rlog(1, self.name, 'sending ping')
         try:
-            self.sock.send('PING :%s\n' % self.server)
+            self._raw('PING :%s' % self.server)
             return 1
         except Exception, ex:
             rlog(10, self.name, "can't send ping: %s" % str(ex))
             return 0
+
+    def handle_401(self, ievent):
+        try:
+            nick = ievent.arguments[1]
+            if nick not in self.nicks401:
+                rlog(10, self.name, '401 on %s .. ignoring' % nick)
+                self.nicks401.append(nick)
+        except:
+            pass
