@@ -7,12 +7,14 @@
 
 __copyright__ = 'this file is in the public domain'
 
-from gozerbot.generic import rlog, stripident, fix_format, todefenc
-import time, re, types
+from gozerbot.generic import rlog, stripident, fix_format, fromenc
+import time, re, types, copy
+
+cpy = copy.deepcopy
 
 def makeargrest(ievent):
     """ create ievent.args and ievent.rest .. this is needed because \
-        ircevents might be created outside the parse() function """
+         ircevents might be created outside the parse() function """ 
     try:
         ievent.args = ievent.txt.split()[1:]
     except ValueError:
@@ -20,7 +22,7 @@ def makeargrest(ievent):
     try:
         cmnd, ievent.rest = ievent.txt.split(' ', 1)
     except ValueError:
-        ievent.rest = ""
+        ievent.rest = ""   
     ievent.command = ievent.txt.split(' ')[0]
 
 class Ircevent(object):
@@ -31,26 +33,26 @@ class Ircevent(object):
         self.jabber = False
         self.groupchat = False
         self.cmnd = None
-        self.prefix = ""
-        self.postfix = ""
-        self.target = ""
+        self.prefix = u""
+        self.postfix = u""
+        self.target = u""
         self.arguments = []
-        self.nick = ""
-        self.user = ""
-        self.ruserhost = ""
-        self.userhost = ""
-        self.stripped = ""
-        self.resource = ""
-        self.channel = ""
-        self.origtxt = ""
-        self.txt = ""
-        self.command = ""
-        self.alias = ""
-        self.aliased = ""
+        self.nick = u""
+        self.user = u""
+        self.ruserhost = u""
+        self.userhost = u""
+        self.stripped = u""
+        self.resource = u""
+        self.channel = u""
+        self.origtxt = u""
+        self.txt = u""
+        self.command = u""
+        self.alias = u""
+        self.aliased = u""
         self.time = time.time()
         self.msg = False
         self.args = []
-        self.rest = ""
+        self.rest = u""
         self.usercmnd = 0
         self.bot = None
         self.sock = None
@@ -58,100 +60,87 @@ class Ircevent(object):
         self.queues = []
         self.printto = None
         self.speed = 5
-        self.options = []
         self.groups = None
-        self.cc = ""
+        self.cc = u""
         self.jid = None
         self.jidchange = None
         self.conn = None
         self.to = None
         self.denied = False
+        self.options = {}
         if ievent:
             self.copyin(ievent)
 
     def copyin(self, ievent):
         """ copy ievent """
-        self.cmnd = str(ievent.cmnd)
-        self.prefix = str(ievent.prefix)
-        self.postfix = str(ievent.postfix)
-        self.target = str(ievent.target)
+        self.cmnd = cpy(ievent.cmnd)
+        self.prefix = cpy(ievent.prefix)
+        self.postfix = cpy(ievent.postfix)
+        self.target = cpy(ievent.target)
         self.arguments = list(ievent.arguments)
-        self.nick = str(ievent.nick)
-        self.user = str(ievent.user)
-        self.ruserhost = str(ievent.ruserhost)
-        self.userhost = str(ievent.userhost)
-        self.stripped = str(ievent.stripped)
-        self.resource = str(ievent.resource)
-        self.channel = str(ievent.channel)
-        self.origtxt = str(ievent.origtxt)
-        self.txt = str(ievent.txt)
-        self.command = str(ievent.command)
-        self.alias = str(ievent.alias)
-        self.aliased = str(ievent.aliased)
+        self.nick = cpy(ievent.nick)
+        self.user = cpy(ievent.user)
+        self.ruserhost = cpy(ievent.ruserhost)
+        self.userhost = cpy(ievent.userhost)
+        self.stripped = cpy(ievent.stripped)
+        self.resource = cpy(ievent.resource)
+        self.channel = cpy(ievent.channel)
+        self.origtxt = cpy(ievent.origtxt)
+        self.txt = cpy(ievent.txt)
+        self.command = cpy(ievent.command)
+        self.alias = cpy(ievent.alias)
+        self.aliased = cpy(ievent.aliased)
         self.time = ievent.time
-        self.msg = ievent.msg
+        self.msg = cpy(ievent.msg)
         self.args = list(ievent.args)
-        self.rest = str(ievent.rest)
-        self.usercmnd = ievent.usercmnd
+        self.rest = cpy(ievent.rest)
+        self.usercmnd = cpy(ievent.usercmnd)
         self.bot = ievent.bot
         self.sock = ievent.sock
         self.printto = ievent.printto
         self.speed = int(ievent.speed)
         self.inqueue = ievent.inqueue
         self.queues = list(ievent.queues)
-        self.options = list(ievent.options)
-        self.cc = str(ievent.cc)
+        self.cc = cpy(ievent.cc)
         self.jid = ievent.jid
         self.jidchange = ievent.jidchange
         self.conn = ievent.conn
         self.to = ievent.to
         self.denied = ievent.denied
-
+        self.options = dict(ievent.options)
+        
     def __str__(self):
         return "cmnd=%s prefix=%s postfix=%s arguments=%s nick=%s user=%s \
-userhost=%s channel=%s txt=%s command=%s args=%s rest=%s speed=%s" % \
+userhost=%s channel=%s txt='%s' command=%s args=%s rest=%s speed=%s options=%s" % \
 (self.cmnd, self.prefix, self.postfix, self.arguments, self.nick, self.user, \
 self.userhost, self.channel, self.txt, self.command, self.args, self.rest, \
-self.speed)
+self.speed, self.options)
+
+    def __del__(self):
+        if self.bot:
+            self.bot.gcevents += 1
 
     def parse(self, bot, rawstr):
         """ parse raw string into ircevent """
-        rawstr = todefenc(rawstr.rstrip())
+        bot.nrevents += 1 
+        rawstr = rawstr.rstrip()
         splitted = re.split('\s+', rawstr)
         # check if there is a prefix (: in front)
-        if rawstr[0] == ':':
-            self.prefix = splitted[0][1:]
-            self.cmnd = splitted[1]
-            # get nick/userhost
-            nickuser = self.prefix.split('!')
-            if len(nickuser) == 2:
-                self.nick = nickuser[0]
-                self.userhost = nickuser[1]
-        else:
+        if not rawstr[0] == ':':
             # no prefix .. 1st word is command
-            self.cmnd = splitted[0]
-            # rest is arguments
-            self.arguments = splitted[1:]
-            go = False
-            for i in self.arguments:
-                if i.startswith(':'):
-                    go = True
-                    self.txt += "%s " % i[1:]
-                    continue
-                if go:
-                    self.txt += "%s " % i
-            # check if argument is txt
-            #if self.arguments and self.arguments[0].startswith(':'):
-            #    self.txt = ' '.join(self.arguments)[1:]
-            rlog(0, 'ircevent',"%s %s %s" % (self.cmnd, self.arguments, \
-self.txt))
-            return self
-        # get argument count so we can determine arguments and possible txt
+            splitted.insert(0, ":none!none@none")
+            rawstr = ":none!none@none " + rawstr
+        self.prefix = splitted[0][1:]
+        # get nick/userhost
+        nickuser = self.prefix.split('!')
+        if len(nickuser) == 2:
+            self.nick = nickuser[0]
+            self.userhost = nickuser[1]
+        self.cmnd = splitted[1]
         if pfc.has_key(self.cmnd):
             self.arguments = splitted[2:pfc[self.cmnd]+2]
             txtsplit = re.split('\s+', rawstr, pfc[self.cmnd]+2)
             self.txt = txtsplit[-1]
-            #self.txt = ' '.join(splitted[pfc[self.cmnd]+2:])
         else:
             self.arguments = splitted[2:]
         # 1st argument is target
@@ -170,10 +159,10 @@ self.txt))
         if self.cmnd == 'PRIVMSG':
             self.channel = self.arguments[0]
         elif self.cmnd == 'JOIN' or self.cmnd == 'PART':
-            if self.txt:
-                self.channel = self.txt
-            else:
+            if self.arguments:
                 self.channel = self.arguments[0]
+            else:
+                self.channel = self.txt
         elif self.cmnd == 'MODE':
             self.channel = self.arguments[0]
         elif self.cmnd == 'TOPIC':
@@ -193,9 +182,15 @@ self.txt))
             self.stripped = self.userhost
             # determine user
             self.user = self.userhost.split('@')[0]
-        self.origtxt = str(self.txt)
+        self.origtxt = self.txt
         self.channel = self.channel.strip()
-        rlog(-1, 'ircevent', str(self))
+        rlog(-1, 'ircevent', self)
+        try:
+            nr = int(self.cmnd)
+            if nr > 399:
+                rlog(10, bot.name + '.error', '%s: %s' % (self.cmnd, self.txt))
+        except ValueError:
+            pass
         return self
 
     def reply(self, txt, result=None, nick=None, dot=False, nritems=False, \
@@ -205,10 +200,20 @@ nr=False):
         """
         if result == []:
             return
+        restxt = ""
+        if type(result) == types.DictType:
+            for i, j in result.iteritems():
+                if type(j) == types.ListType:
+                    z = ' .. '.join(j)
+                else:
+                    z = j                 
+                restxt += "[%s] %s %s " % (i, z, dot)
+            if restxt:
+                restxt = restxt[:-6]
         lt = False
         if type(txt) == types.ListType and not result:
             result = txt
-            origtxt = ""
+            origtxt = u""
             lt = True
         else:
             origtxt = txt
@@ -216,7 +221,9 @@ nr=False):
             lt = True
         if self.queues:
             for i in self.queues:
-                if lt:
+                if restxt:
+                    i.put_nowait(restxt)
+                elif lt:
                     for j in result:
                         i.put_nowait(j)
                 else:
@@ -226,12 +233,18 @@ nr=False):
             rlog(10, 'irc', 'no bot defined in ircevent')
             return
         pretxt = origtxt
-        restxt = ""
-        if lt:
+        if lt and not restxt:
+            res = []
+            for i in result:
+                if type(i) == types.ListType or type(i) == types.TupleType:
+                    res.extend(i)
+                else:
+                    res.append(i)
+            result = res
             if nritems:
                 if len(result) > 1:
                     pretxt += "(%s items) .. " % len(result)
-            txtlist = [fix_format(str(i)) for i in result]
+            txtlist = result
             if not nr is False:
                 try:
                     start = int(nr)
@@ -240,7 +253,7 @@ nr=False):
                 txtlist2 = []
                 teller = start
                 for i in txtlist:
-                    txtlist2.append("%s) %s" % (teller, i))
+                    txtlist2.append(u"%s) %s" % (teller, i))
                     teller += 1
                 txtlist = txtlist2
             if dot == True:
@@ -250,7 +263,7 @@ nr=False):
             else:
                 restxt = ' '.join(txtlist)
         if pretxt:
-            restxt = todefenc(pretxt) + todefenc(restxt)
+            restxt = pretxt + restxt
         if self.cmnd == 'DCC' and self.sock:
             self.bot.say(self.sock, restxt, speed=self.speed)
             return
@@ -260,20 +273,22 @@ nr=False):
         if self.msg:
             self.bot.say(self.nick, restxt, fromm=self.nick, speed=self.speed)
             return
+        # check if bot is in silent mode .. if so use /msg 
+        silent = False
+        channel = self.printto or self.channel
+        try:
+            silent = self.bot.channels[channel]['silent']
+        except (KeyError, TypeError):
+            pass
+        if silent:
+                self.bot.say(self.nick, restxt, fromm=self.nick, \
+speed=self.speed)
+                return
         if self.printto:
             self.bot.say(self.printto, restxt, fromm=self.nick, \
 speed=self.speed)
             return
         else:
-            # check if bot is in silent mode .. if so use /msg 
-            try:
-                silent = self.bot.channels[self.channel]['silent']
-                if silent:
-                    self.bot.say(self.nick, restxt, fromm=self.nick, speed=\
-self.speed)
-                    return
-            except KeyError:
-                pass
             self.bot.say(self.channel, restxt, fromm=self.nick, \
 speed=self.speed)
 
