@@ -6,7 +6,7 @@
 
 __copyright__ = 'this file is in the public domain'
 
-from gozerbot.generic import rlog, todefenc, jabberstrip
+from gozerbot.generic import rlog, toenc, fromenc, jabberstrip
 from gozerbot.ircevent import makeargrest
 import xmpp, types, time
 
@@ -21,8 +21,14 @@ class Jabbermsg(xmpp.Message):
             xmpp.Message.__init__(self)
         self.jabber = True
         self.id = self.getID()
+        self.bot = None
+        self.botoutput = False
         rlog(2, 'jabbermsg', str(self))
 
+    def __del__(self):
+        if self.bot:
+            self.bot.gcevents += 1
+
     def reply(self, txt, result=None, nick=None, dot=False, nritems=False, \
 nr=False):
         """ reply txt .. first check if queue then if DCC then /msg
@@ -73,7 +79,7 @@ nr=False):
                 restxt = ' '.join(txtlist)
         if pretxt:
             restxt = pretxt + restxt
-        txt = todefenc(jabberstrip(restxt))
+        txt = jabberstrip(restxt)
         if not txt:
             return
         what = txt
@@ -113,8 +119,9 @@ nr=False):
             else:
                 repl.setTo(self.channel)
             repl.setType('groupchat')
+        if self.bot.google:
+            repl.setFrom(self.bot.me)
         repl.setID(self.id)
-        repl.setFrom(self.bot.jid)
         self.bot.send(repl)
 
     def toirc(self, bot):
@@ -126,8 +133,8 @@ nr=False):
         self.postfix = ""
         self.target = ""
         self.arguments = []
-        self.resource = str(self.getFrom().getResource())
-        self.channel = str(self.getFrom().getStripped())
+        self.resource = fromenc(self.getFrom().getResource())
+        self.channel = fromenc(self.getFrom().getStripped())
         self.nick = self.resource
         self.user = ""
         try:
@@ -136,12 +143,12 @@ nr=False):
         except KeyError:
             self.jid = self.getFrom()
         self.ruserhost = self.jid
-        self.userhost = str(self.jid)
-        self.stripped = self.jid.getStripped()
-        self.printto = str(self.channel)
-        self.txt = self.getBody()
+        self.userhost = fromenc(str(self.jid))
+        self.stripped = fromenc(self.jid.getStripped())
+        self.printto = self.channel
+        self.txt = jabberstrip(fromenc(self.getBody()))
         self.origtxt = self.txt
-        self.to = str(self.getTo())
+        self.to = fromenc(str(self.getTo()))
         self.time = time.time()
         self.msg = None
         self.args = []
@@ -152,7 +159,7 @@ nr=False):
         self.inqueue = None
         self.queues = []
         self.speed = 5
-        self.options = []
+        self.options = {}
         self.groups = None
         self.type = self.getType()
         self.cc = ""
@@ -163,37 +170,37 @@ nr=False):
             self.groupchat = True
         else:
             self.groupchat = False
-        if self.txt:
-            makeargrest(self)
+            self.userhost = self.stripped
+        self.bot.nrevents += 1
         
     def copyin(self, jmsg):
         """ copy in another jabber message object """
-        self.jidchange = jmsg.jidchange
-        self.cmnd = str(jmsg.cmnd)
+        self.jidchange = bool(jmsg.jidchange)
+        self.cmnd = unicode(jmsg.cmnd)
         self.conn = jmsg.conn
-        self.prefix = str(jmsg.prefix)
-        self.postfix = str(jmsg.postfix)
-        self.target = str(jmsg.target)
-        self.arguments = str(jmsg.arguments)
-        self.nick = str(jmsg.nick)
-        self.user = str(jmsg.user)
+        self.prefix = unicode(jmsg.prefix)
+        self.postfix = unicode(jmsg.postfix)
+        self.target = unicode(jmsg.target)
+        self.arguments = unicode(jmsg.arguments)
+        self.nick = unicode(jmsg.nick)
+        self.user = unicode(jmsg.user)
         if not jmsg.jid:
             self.jid = xmpp.JID(jmsg.userhost)
         else:
             self.jid = jmsg.jid
-        self.ruserhost = str(jmsg.ruserhost)
-        self.userhost = str(jmsg.userhost)
-        self.resource = str(jmsg.resource)
+        self.ruserhost = unicode(jmsg.ruserhost)
+        self.userhost = unicode(jmsg.userhost)
+        self.resource = unicode(jmsg.resource)
         self.stripped = jmsg.stripped
-        self.channel = str(jmsg.channel)
-        self.printto = str(jmsg.printto)
-        self.txt = str(jmsg.txt)
-        self.origtxt = str(jmsg.origtxt)
-        self.to = str(jmsg.to)
+        self.channel = unicode(jmsg.channel)
+        self.printto = unicode(jmsg.printto)
+        self.txt = unicode(jmsg.txt)
+        self.origtxt = unicode(jmsg.origtxt)
+        self.to = unicode(jmsg.to)
         self.time = jmsg.time
         self.msg = jmsg.msg
         self.args = list(jmsg.args)
-        self.rest = str(jmsg.rest)
+        self.rest = unicode(jmsg.rest)
         self.usercmnd = jmsg.usercmnd
         self.bot = jmsg.bot
         self.sock = jmsg.sock
@@ -201,12 +208,12 @@ nr=False):
         self.queues = list(jmsg.queues)
         self.printto = jmsg.printto
         self.speed = int(jmsg.speed)
-        self.options = jmsg.options
+        self.options = dict(jmsg.options)
         self.groups = jmsg.groups
         self.groupchat = jmsg.groupchat
-        self.cc = str(jmsg.cc)
-        self.alias = str(jmsg.alias)
-        self.aliased = str(jmsg.aliased)
+        self.cc = unicode(jmsg.cc)
+        self.alias = unicode(jmsg.alias)
+        self.aliased = unicode(jmsg.aliased)
         self.denied = jmsg.denied
 
     def ircstr(self):
