@@ -8,8 +8,10 @@
 __copyright__ = 'this file is in the public domain'
 __credits__ = 'Aim'
 
-from gozerbot.generic import rlog
-import thread
+from gozerbot.generic import rlog, handle_exception
+from gozerbot.fleet import fleet
+from gozerbot.thr import start_new_thread
+import thread, pickle, socket
 
 class Partyline(object):
 
@@ -20,6 +22,36 @@ class Partyline(object):
         self.socks = []
         self.lock = thread.allocate_lock()
 
+    def _doresume(self, data, reto=None):
+        for i in data['partyline']:
+            bot = fleet.byname(i['botname'])
+            sock = socket.fromfd(i['fileno'], socket.AF_INET, socket.SOCK_STREAM)
+            sock.setblocking(1)
+            nick = i['nick']
+            userhost = i['userhost']
+            channel = i['channel']
+            if not bot:
+                rlog(10, 'partyline', "can't find %s bot in fleet" % i['botname'])
+                continue
+            self.socks.append({'bot': bot, 'sock': sock, 'nick': nick, \
+'userhost': userhost, 'channel': channel, 'silent': i['silent']})
+            bot._dccresume(sock, nick, userhost, channel)        
+
+    def _resumedata(self):
+        result = []
+        for i in self.socks:
+            result.append({'botname': i['bot'].name, 'fileno': \
+i['sock'].fileno(), 'nick': i['nick'], 'userhost': i['userhost'], \
+'channel': i['channel'], 'silent': i['silent']})
+        return result
+
+    def resume(self, sessionfile):
+        session = pickle.load(open(sessionfile))
+        try:
+            self._doresume(session)
+        except Exception, ex:
+            handle_exception()
+
     def stop(self, bot):
         """ stop all user on bot """
         for i in self.socks:
@@ -27,7 +59,7 @@ class Partyline(object):
                 try:
                     i['sock'].shutdown(2)
                     i['sock'].close()
-                except IOError:
+                except:
                     pass
 
     def stop_all(self):
@@ -36,7 +68,7 @@ class Partyline(object):
             try:
                 i['sock'].shutdown(2)
                 i['sock'].close()
-            except IOError:
+            except:
                 pass
 
     def loud(self, nick): 
@@ -51,10 +83,13 @@ class Partyline(object):
             if i['nick'] == nick:
                 i['silent'] = True
 
-    def add_party(self, bot, sock, nick):
+    def add_party(self, bot, sock, nick, userhost, channel):
         ''' add a socket with nick to the list '''
+        for i in self.socks:
+            if i['sock'] == sock:
+                return            
         self.socks.append({'bot': bot, 'sock': sock, 'nick': nick, \
-'silent': False})
+'userhost': userhost, 'channel': channel, 'silent': False})
         rlog(1, 'partyline', 'added user %s on the partyline' % nick)
 
     def del_party(self, nick):
