@@ -16,6 +16,9 @@ class Pdol(Persist):
         Persist.__init__(self, fname)
         if not self.data:
             self.data = {}
+
+    def __iter__(self, name):
+        return self.data[name].__iter__()
  
     def __getitem__(self, item):
         if self.data.has_key(item):
@@ -37,6 +40,13 @@ class Pdol(Persist):
         """ add what to items list """
         return self.__setitem__(item, what)
 
+    def adduniq(self, item, what):
+        """ add what to items list if item not yet added """
+        if not self.data.has_key(item):
+            self.new(item)
+        if what not in self.data[item]:
+            return self.__setitem__(item, what)
+
     def get(self, item):
         """ get items list """
         return self.__getitem__(item)
@@ -48,3 +58,15 @@ class Pdol(Persist):
     def delete(self, item, what):
         """ remove what from item's list """
         del self.data[item][what]
+
+    def extend(self, item, what):
+        if not self.data.has_key(item):
+            self.new(item)
+        self.data[item].extend(what)
+
+    def remove(self, item, what):
+        try:
+            self.data[item].remove(what)
+            return 1
+        except (ValueError, KeyError):
+            return 0
