@@ -11,7 +11,7 @@ from gozerbot.eventhandler import commandhandler
 from gozerbot.users import users
 from gozerbot.monitor import outmonitor, saymonitor, jabbermonitor
 from gozerbot.generic import rlog, handle_exception, checkchan, lockdec, \
-plugnames, waitforqueue
+plugnames, waitforqueue, uniqlist, makeoptions
 from gozerbot.myimport import my_import
 from gozerbot.persist import Persist
 from gozerbot.persistconfig import PersistConfig
@@ -21,17 +21,19 @@ from gozerbot.callbacks import callbacks, jcallbacks
 from gozerbot.redispatcher import rebefore, reafter
 from gozerbot.ircevent import makeargrest, Ircevent
 from gozerbot.aliases import aliascheck
-from gozerbot.limiter import getlastalertmin, setlastalertmin, \
-getlastalerthour, setlastalerthour, shouldblockmin, shouldblockhour, addievent
 from gozerbot.ignore import shouldignore
 from gozerbot.thr import start_new_thread
+from gozerbot.persiststate import PersistState
 if config['jabberenable']:
     from gozerbot.jabbermsg import Jabbermsg
-import os.path, thread, time, Queue, thread
+import os, os.path, thread, time, Queue
 
 loadlock = thread.allocate_lock()
 locked = lockdec(loadlock)
 
+plugstate = PersistState(datadir + os.sep + 'pluginsstate')
+plugstate.define('plugallow', [])
+
 class Plugins(object):
 
     """ hold all the plugins """
@@ -42,13 +44,29 @@ class Plugins(object):
         self.plugdeny = Persist(datadir + os.sep + 'plugdeny')
         if not self.plugdeny.data:
             self.plugdeny.data = []
+        self.loadlist = []
+        self.avail = []
+        self.ondisk = []
+        
+    def enableloadlist(self):
+        if config['loadlist']:
+            try:
+                self.loadlist = [i.strip() for i in open(datadir + os.sep + \
+'loadlist').readlines()]
+            except:
+                self.loadlist = []
+        else:
+            self.loadlist = []
 
     def __getitem__(self, item):
         """ return plugin """
         if self.plugs.has_key(item):
             return self.plugs[item]
+        else:
+            return None
 
     def whatperms(self):
+        """ return what permissions are possible """
         result = []
         for i in rebefore.whatperms():
             if not i in result:
@@ -80,8 +98,9 @@ class Plugins(object):
             self.plugdeny.save()
 
     def plugsizes(self):
+        """ call the size() function in all plugins """
         reslist = []
-        for i,j in self.plugs.iteritems():
+        for i, j in self.plugs.iteritems():
             try:
                 reslist.append("%s: %s" % (i, j.size()))
             except AttributeError:
@@ -90,27 +109,14 @@ class Plugins(object):
 
     def list(self):
         """ list of registered plugins """
-        return self.plugs.keys()
+        self.avail.sort()
+        return self.avail
 
-    def regplugin(self, mod, name):
-        """ register plugin """
-        name = name.lower()
-        mod = mod.lower()
+    def plugimport(self, mod, name):
         modname = mod + '.' + name
-        # see if plugin is in deny
-        if name in self.plugdeny.data:
-            rlog(10, 'plugins', '%s (%s) in deny .. not loading' % \
-(name, mod))
-            return 0
-        # if plugin is already registered unload it
-        if self.plugs.has_key(name):
-            rlog(1, 'plugins', 'overloading %s plugin with %s version' \
-% (name, mod))
-            self.unloadnosave(name)
-        # import the plugin
         try:
             plug = my_import(modname)
-            self.plugs[name] = plug
+            self.plugs[name] = plug  
         except Exception, ex:
             self.unload(name)
             handle_exception()
@@ -120,14 +126,45 @@ class Plugins(object):
             rlog(0, 'plugins', 'calling %s init()' % name)
             result = plug.init()
             if not result:
-                return 
+                return    
         except AttributeError:
             pass
-        except:
+        except: 
             self.unload(name)
             raise
-        rlog(0, 'plugins', "%s (%s) registered" % (name, mod))
-        return 1
+        self.avail.append(name)
+
+    def regplugin(self, mod, name):
+        """ register plugin """
+        name = name.lower()
+        mod = mod.lower()
+        modname = mod + '.' + name
+        # see if plugin is in deny
+        if name in self.avail:
+            rlog(10, 'plugins', '%s already registered' % name)
+            return
+        if name in self.plugdeny.data:
+            rlog(10, 'plugins', '%s.%s in deny .. not loading' % \
+(mod, name))
+            return 0
+        if self.loadlist and name not in self.loadlist and name not in \
+plugstate['plugallow']:
+            rlog(8, 'plugins', 'not loading %s.%s' % (mod, name))
+            return 0
+        # if plugin is already registered unload it
+        if self.plugs.has_key(name):
+            rlog(10, 'plugins', 'overloading %s plugin with %s version' \
+% (name, mod))
+            self.unloadnosave(name)
+        # import the plugin
+        self.plugimport(mod, name)
+        rlog(0, 'plugins', "%s.%s registered" % (mod, name))
+        return name
+
+    def showregistered(self):
+        time.sleep(7)
+        self.avail.sort()
+        rlog(10, 'plugins', 'registered %s' % ' .. '.join(self.avail))
 
     def regdir(self, dirname, exclude=[]):
         threads = []
@@ -141,35 +178,37 @@ class Plugins(object):
                 plugs.append(plug)
             except:
                 handle_exception()
-        for i in threads:
-            i.join(5)
+        self.ondisk.extend(plugs)
         return plugs
 
-    def regcore(self):  
-        plugs = my_import('gozerplugs.plugs')
-        for i in plugs.__all__:
-            self.regplugin('gozerplugs.plugs', i)
+    def regcore(self): 
+        avail = [] 
         if config['dbenable']:
             dbplugs = my_import('gozerplugs.dbplugs')
             for i in dbplugs.__all__:
-                self.regplugin('gozerplugs.dbplugs', i)
+                start_new_thread(self.regplugin, ('gozerplugs.dbplugs', i))
+                avail.append(i)
+        plugs = my_import('gozerplugs.plugs')
+        for i in plugs.__all__:
+            if i not in avail:
+                start_new_thread(self.regplugin, ('gozerplugs.plugs', i))
+                avail.append(i)
+        self.ondisk.extend(avail)
 
     def regplugins(self):
         """ register plugins """
+        self.enableloadlist()
         self.regcore()
         if os.path.isdir('myplugs'):
             self.regdir('myplugs')
-        # log loaded plugs
-        pluglist = self.list()
-        pluglist.sort()
-        rlog(10, 'plugins', 'loaded %s' % ' .. '.join(pluglist))
+        start_new_thread(self.showregistered, ())
         self.overload()
 
     def overload(self):
         # see if there is a permoverload file and if so use it to overload
         # permissions based on function name
         try:
-            overloadfile = open('permoverload','r')
+            overloadfile = open(datadir + '/permoverload','r')
         except IOError:
             return
         try:
@@ -201,10 +240,9 @@ provided for %s" % funcname)
             handle_exception()
         
     def available(self):
-        """ available pluginsregister """
-        resultlist = self.plugs.keys()
-        resultlist.sort()
-        return resultlist
+        """ available plugins not yet registered """
+        self.ondisk.sort()
+        return self.ondisk
 
     def saveplug(self, plugname):
         """ call save on items in plugins savelist """
@@ -244,7 +282,7 @@ provided for %s" % funcname)
                         cfg.save()
                     except:
                         handle_exception()
-	    except AttributeError:
+            except AttributeError:
                 continue
 
     def save_cfgname(self, name):
@@ -256,7 +294,7 @@ provided for %s" % funcname)
                     cfg.save()
                 except:
                     handle_exception()
-	except (AttributeError, KeyError):
+        except (AttributeError, KeyError):
             pass
 	
     def exit(self):
@@ -267,16 +305,24 @@ provided for %s" % funcname)
         for name, plug in self.plugs.iteritems():
             try:
                 shutdown = getattr(plug, 'shutdown')
-                thread = start_new_thread(shutdown,())
+                thread = start_new_thread(shutdown, ())
                 threadlist.append((name, thread))
             except AttributeError:
                 continue
-	for name, thread in threadlist:
-	    thread.join()
-            rlog(1, 'plugins', '%s shutdown' % name)
+            except Exception, ex:
+                rlog(10, 'plugins', 'error shutting down %s: %s' % (name, \
+str(ex)))
+        try:
+            for name, thread in threadlist:
+                thread.join()
+                rlog(10, 'plugins', '%s shutdown finished' % name)
+        except:
+            handle_exception()
+            return    
 
     def reload(self, mod, name):
         """ reload plugin """
+        reloaded = []
         ok = 0
         # save data in plugin's savelist before we try to reload
         modname = mod + '.' + name
@@ -286,6 +332,14 @@ modname)
             return 
         # unload plugin
         self.unload(name)
+        try:
+            depend = self.plugs[name].__depend__
+            rlog(10, 'plugins', 'depends of %s: %s' % (modname, depend))
+        except (AttributeError, KeyError):
+            depend = []
+            pass
+        for plug in depend:
+            reloaded.extend(self.reload('gozerplugs.plugs' , plug))
         # see if module is already loaded if not load it
         if not self.plugs.has_key(name):
             self.plugs[name] = my_import(modname)
@@ -304,17 +358,33 @@ modname)
         if not ok:
             rlog(10, 'plugins', '%s module init failed' % name)
             return 0
-        rlog(10, 'plugins', 'reloaded plugin %s' % modname)
-        return 1
+        rlog(0, 'plugins', 'reloaded plugin %s' % modname)
+        reloaded.append(name)
+        plugstate['plugallow'].append(name)
+        plugstate.save()
+        #if name not in self.avail:
+        self.avail.append(name)
+        return reloaded
 
     def unload(self, plugname):
         """ unload plugin """
         # call plugins shutdown function if available
         try:
             self.plugs[plugname].shutdown()
-            rlog(0, 'plugins', '%s shutdown called' % plugname)
+            rlog(10, 'plugins', '%s shutdown called' % plugname)
         except (AttributeError, KeyError):
             pass
+        except Exception, ex:
+            handle_exception()
+        try:
+            plugstate['plugallow'].remove(plugname)
+            plugstate.save()
+        except (KeyError, ValueError):
+            pass
+        try:
+            self.avail.remove(plugname)
+        except ValueError:
+            pass
         self.saveplug(plugname)
         return self.unloadnosave(plugname)
 
@@ -354,9 +424,10 @@ modname)
         if what and command:
             return 1
 
-    @locked
     def dispatchtest(self, bot, ievent, direct=False):
         """ see if ievent would dispatch """
+        if shouldignore(ievent.userhost):
+            return (None, None)
         chan = checkchan(bot, ievent.txt)
         if chan != None:
             (channel, txt) = chan
@@ -384,6 +455,7 @@ modname)
             if com:
                 com.re = False
                 result = [cmnds, com]
+                ievent.txt = ievent.txt.strip()
             else:
                 # try RE after commands
                 com = reafter.getcallback(target)
@@ -392,6 +464,8 @@ modname)
                     result = [reafter, com]
         time.sleep(0.001)
         if result:
+            if com.name in bot.state['allowed']:
+                return result
             try:
                 chanperms = bot.channels[ievent.channel]['perms']
                 for i in result[1].perms:
@@ -417,25 +491,11 @@ modname)
         self.trydispatch(bot, ii)
         return waitforqueue(q, timeout)
 
+    @locked
     def trydispatch(self, bot, ievent, direct=False):
         """ try to dispatch ievent """
         # test for ignore
         if shouldignore(ievent.userhost):
-            return
-        # limiter test
-        lastalertmin = getlastalertmin(ievent.userhost)
-        lastalerthour = getlastalerthour(ievent.userhost)
-	nr = shouldblockhour(ievent.userhost)
-        if nr:
-            if time.time() - 3600 > lastalerthour:
-                ievent.reply('max %s commands per hour' % nr)
-                setlastalerthour(ievent.userhost, time.time())
-            return 0
-        nr = shouldblockmin(ievent.userhost)
-        if nr:
-            if time.time() - 60 > lastalertmin:
-                ievent.reply('max %s commands per minute' % nr)
-                setlastalertmin(ievent.userhost, time.time())
             return 0
         # check if channel is provided
         chan = checkchan(bot, ievent.txt)
@@ -471,10 +531,12 @@ modname)
         if bot.stopped:
             return 0
         # check for user provided channel
+        if com.options:
+            if not makeoptions(ievent, com.options):
+                return
         makeargrest(ievent)
         ievent.usercmnd = 1
         commandhandler.put(ievent.speed, what, com, bot, ievent)
-        addievent(ievent.userhost, ievent)
         return 1
 
     def multiple(self, bot, ievent):
@@ -522,4 +584,22 @@ modname)
             if what:
                 self.dispatch(what, com, bot, i)
 
+    def listreload(self, pluglist):
+        """ reload list of plugins """
+        failed = []
+        for what in pluglist:
+            splitted = what[:-3].split(os.sep)
+            mod = '.'.join(splitted[:-1])
+            plug  = splitted[-1]
+            # test for plugdeny
+            if plug in self.plugdeny.data:
+                rlog(10, 'upgrade', '%s in plugdeny .. not reloading' % plug) 
+                continue
+            # reload the plugin
+            try:
+                self.reload(mod, plug)
+            except Exception, ex:
+                failed.append(what)
+        return failed
+
 plugins = Plugins()
