@@ -18,8 +18,9 @@ class Recallback(object):
     """ a regular expression callback """
 
     def __init__(self, index, regex, func, perm, plugname, speed=5, \
-threaded=True, allowqueue=True):
+threaded=True, allowqueue=True, options={}):
         # index of place of RE callback in list
+        self.name = "noname"
         self.index = index
         # the RE to match
         self.regex = regex
@@ -39,6 +40,7 @@ threaded=True, allowqueue=True):
         # loop
         self.threaded = copy.deepcopy(threaded)
         self.allowqueue = copy.deepcopy(allowqueue)
+        self.options = dict(options)
 
 class Redispatcher(object):
 
@@ -95,14 +97,14 @@ funcname)
             return 1
 
     def add(self, index, regex, func, perm, speed=5, threaded=True, \
-allowqueue=True):
+allowqueue=True, options={}):
         """ add a command """
         try:
             # get plugin name from where callback is added
             plugname = calledfrom(sys._getframe())
             # add Recallback
             self.relist.append(Recallback(index, regex, func, perm, plugname, \
-speed, threaded, allowqueue))
+speed, threaded, allowqueue, options))
             # sort of index number
             self.relist.sort(lambda a, b: cmp(a.index, b.index))
             rlog(0, 'redispatcher', 'added %s (%s) ' % (regex, plugname))
@@ -154,7 +156,7 @@ class Botredispatcher(Redispatcher):
     def dispatch(self, callback, bot, ievent):
         """ dispatch callback on ircevent """
         try:
-            result = re.search(callback.regex, ievent.txt)
+            result = re.search(callback.compiled, ievent.txt)
             if result:
                 ievent.groups = list(result.groups())                
                 if callback.threaded:
