@@ -15,6 +15,14 @@ from gozerbot.plughelp import plughelp
 
 plughelp.add('user', 'manage users')
 
+def handle_whoami(bot, ievent):
+    """ user-whoami .. get your username """
+    ievent.reply('%s' % users.getname(ievent.userhost))
+
+cmnds.add('user-whoami', handle_whoami, 'USER')
+examples.add('user-whoami', 'get your username', 'user-whoami')
+aliases.data['whoami'] = 'user-whoami'
+
 def handle_meet(bot, ievent):
     """ user-meet <nick> .. introduce a new user to the bot """
     try:
@@ -89,8 +97,8 @@ def handle_merge(bot, ievent):
 'OPER'):
         ievent.reply("only OPER perm can merge with OPER user")
         return
-    if name == 'owner' and ievent.userhost != bot.owner:     
-         ievent.reply('only owneruserhost can merge with owner user') 
+    if name == 'owner' and not bot.ownercheck(ievent, "can merge with owner \
+user"):
          return 
     if not users.exist(name):
         ievent.reply("we have no user %s" % name)
@@ -273,6 +281,9 @@ def handle_perms(bot, ievent):
         ievent.reply("use getperms to get the permissions of somebody else")
         return
     name = users.getname(ievent.userhost)
+    if not name:
+         ievent.reply("can't find username for %s" % ievent.userhost)
+         return
     perms = users.db.execute(""" SELECT perm FROM perms WHERE \
 name = %s """, name)
     if perms:
@@ -425,8 +436,8 @@ def handle_adduserhost(bot, ievent):
     name, userhost = ievent.args
     userhost = stripident(userhost)
     name = name.lower()
-    if name == 'owner' and bot.owner != ievent.stripped:
-        ievent.reply('only owneruserhost can adduserhost to owner')
+    if name == 'owner' and not bot.ownercheck(ievent, 'can adduserhost to \
+owner'):
         return
     if not users.exist(name):
         ievent.reply("can't find user %s" % name)
@@ -460,8 +471,8 @@ def handle_deluserhost(bot, ievent):
     name, userhost = ievent.args
     userhost = stripident(userhost)
     name = name.lower()
-    if name == 'owner'  and ievent.stripped != bot.owner:
-        ievent.reply('only owneruserhost can delete userhosts from owner')
+    if name == 'owner'  and not bot.ownercheck(ievent, 'can delete userhosts \
+from owner'):
         return 
     if not users.exist(name):
         ievent.reply("can't find user %s" % name)
@@ -563,7 +574,7 @@ def handle_setemail(bot, ievent):
 
 cmnds.add('user-setemail', handle_setemail, 'OPER')
 examples.add('user-setemail', 'user-setemail <name> <email>.. set email of \
-user <name>', 'user-setemail dunker bart@r8.cg.nu')
+user <name>', 'user-setemail dunker bart@gozerbot.org')
 aliases.data['setemail'] = 'user-setemail'
 
 def handle_email(bot, ievent):
@@ -612,7 +623,7 @@ def handle_addpermit(bot, ievent):
     try:
         who, what = ievent.args
     except ValueError:
-        ievent.missing("<name> <what>")
+        ievent.missing("<name> <permit>")
         return
     if not users.exist(who):
         ievent.reply("can't find username of %s" % who)
@@ -642,9 +653,9 @@ aliasdel('allow')
 def handle_permit(bot, ievent):
     """ user-permit .. get permit list of user giving the command """
     if ievent.rest:
-        ievent.reply("use the allow command to allow somebody something .. \
-use getname <nick> to get the username of somebody .. this command shows \
-what permits you have")
+        ievent.reply("use the user-addpermit command to allow somebody \
+something .. use getname <nick> to get the username of somebody .. this \
+command shows what permits you have")
         return
     name = users.getname(ievent.userhost)
     if not name:
@@ -860,7 +871,7 @@ WHERE userhost LIKE %s """, '%%%s%%' % what)
         ievent.reply('no userhost matching %s found' % what)
 
 cmnds.add('user-search', handle_usersearch, 'OPER')
-examples.add('user-search', 'search users userhosts', 'user-search r8')
+examples.add('user-search', 'search users userhosts', 'user-search gozerbot')
 
 def handle_addpermall(bot, ievent):
     """ user-addpermall <perm> .. add permission to all users """
