@@ -2,7 +2,9 @@
 #
 #
 
-""" handle txt alarms """
+""" the alarm plugin allows for alarms that message the user giving the
+ command at a certain time or number of seconds from now
+"""
 
 __copyright__ = 'this file is in the public domain'
 
@@ -15,11 +17,10 @@ from gozerbot.fleet import fleet
 from gozerbot.datadir import datadir
 from gozerbot.plughelp import plughelp
 from gozerbot.aliases import aliases
-import gozerbot.thr as thr
+from gozerbot.periodical import periodical
 import time, os
 
-plughelp.add('alarm', 'the alarm plugin allows for alarms which /msg the user \
-giving the command at a certain time or at number of seconds from now')
+plughelp.add('alarm', 'remind the user with given txt at a certain time')
 
 class Alarmitem(object):
 
@@ -51,7 +52,7 @@ class Alarms(Persist):
                 getattr(i, 'printto')
             except AttributeError:
                 setattr(i, 'printto', "")
-        self.run = 1
+            periodical.addjob(i.time - time.time(), 1, self.alarmsay, i.nick, i)
 
     def size(self):
         """ return number of alarms """
@@ -71,59 +72,42 @@ class Alarms(Persist):
         bot = fleet.byname(item.botname)
         if bot:
             if item.printto:
-                bot.say(item.printto, "[%s] %s" % (item.nick, item.txt))
+                bot.say(item.printto, "[%s] %s" % (item.nick, item.txt), \
+speed=1)
             else:
-                bot.say(item.nick, item.txt)
-            return 1
+                bot.say(item.nick, item.txt, speed=1)
+        self.delete(item.idnr)
 
     def add(self, botname, nick, ttime, txt, printto=None):
         """ add alarm """
         nick = nick.lower()
         nrid = nextid.next('alarms')
-        self.data.append(Alarmitem(botname, nrid, nick, ttime, txt, \
-printto=printto))
+        item = Alarmitem(botname, nrid, nick, ttime, txt, printto=printto)
+        pid = periodical.addjob(ttime - time.time(), 1, self.alarmsay, nick, \
+item)
+        item.idnr = pid
+        self.data.append(item)
         self.save()
-        return nrid
+        return pid
 
     def delete(self, idnr):
         """ delete alarmnr """
         for i in range(len(self.data)-1, -1, -1):
             if self.data[i].idnr == idnr:
                 del self.data[i]
-                self.save()
-                return 1
-
-    def check(self):
-        """ check if alarms should ring """
-        rlog(0, 'alarms', 'starting alarms checker')
-        # poll data every second
-        while self.run:
-            time.sleep(1)
-            if not self.run:
-                break
-            ttime = time.time()
-            for i in self.data:
-                # see if its time to say txt
-                if ttime > i.time:
-                    self.alarmsay(i)
-                    self.delete(i.idnr)
+                if periodical.killjob(idnr):
+                    self.save()
+                    return 1
 
 alarms = Alarms(datadir + os.sep + 'alarms')
 
+def shutdown():
+    periodical.kill()
+
 def size():
     """ return number of alarms """
     return alarms.size()
 
-def init():
-    """ run after reload """
-    thr.start_new_thread(alarms.check, ())
-    return 1
-    
-def shutdown():
-    """ run before reload """
-    alarms.run = 0
-    return 1
-
 def handle_alarmadd(bot, ievent):
     """ alarm <txt-with-time> | <+delta> <txt> .. add an alarm """
     if ievent.cmnd == 'DCC':
@@ -177,7 +161,7 @@ def handle_alarmadd(bot, ievent):
 
 cmnds.add('alarm', handle_alarmadd, 'USER', allowqueue=False)
 examples.add('alarm', 'say txt at a specific time or time diff', \
-'1) alarm 12:00 lunchtime 2) alarm 3-11-2007 0:01 birthday ! 3) alarm +180 \
+'1) alarm 12:00 lunchtime 2) alarm 3-11-2008 0:01 birthday ! 3) alarm +180 \
 egg ready')
 
 def handle_alarmlist(bot, ievent):
@@ -227,4 +211,4 @@ def handle_alarmdel(bot, ievent):
         ievent.reply('failed to delete alarm with id %s' % alarmnr)
 
 cmnds.add('alarm-del', handle_alarmdel, 'OPER')
-examples.add('alarm-del', 'delete alarm with id <nr>', 'alarm-del 1')
+examples.add('alarm-del', 'delete alarm with id <nr>', 'alarm-del 7')
