@@ -2,7 +2,9 @@
 #
 #
 
-""" implement aliases commands """
+""" this alias plugin allows aliases for commands to be added. aliases are in
+ the form of <alias> -> <command> .. aliases to aliases are not allowed
+"""
 
 __copyright__ = 'this file is in the public domain'
 
@@ -46,10 +48,7 @@ def handle_aliasset(bot, ievent):
         ievent.missing('<from> <to>')
         return
     if aliases.data.has_key(aliasto):
-        ievent.reply('cannot alias an alias')
-        return
-    if not cmnds.has_key(aliasto):
-        ievent.reply('command does not exist')
+        ievent.reply("can't alias an alias")
         return
     if cmnds.has_key(aliasfrom):
         ievent.reply('command with same name already exists.')
@@ -60,7 +59,7 @@ def handle_aliasset(bot, ievent):
     ievent.reply('alias added')
 
 cmnds.add('alias-set', handle_aliasset, 'OPER', allowqueue=False)
-examples.add('alias-set', 'alias-set <from> <to> .. define alias', \
+examples.add('alias-set', 'alias-set <alias> <command> .. define alias', \
 'alias-set ll list')
 aliases.data['alias'] = 'alias-set'
 
