@@ -73,4 +73,4 @@ def handle_delbotsnack(bot, ievent):
 
 cmnds.add('botsnack-del', handle_delbotsnack, 'OPER')
 examples.add('botsnack-del', 'delete entry from botsnack list', 'botsnack-del \
-mekker')
+thnx <nick> ;]')
