@@ -2,7 +2,7 @@
 #
 #
 
-""" make a random choice """
+""" the choice command can be used with a string or in a pipeline """
 
 __copyright__ = 'this file is in the public domain'
 
@@ -12,7 +12,7 @@ from gozerbot.examples import examples
 from gozerbot.plughelp import plughelp
 import random
 
-plughelp.add('choice', 'make a random choice of string or pipeline')
+plughelp.add('choice', 'make a random choice')
 
 def handle_choice(bot, ievent):
     """ make a random choice out of different words or list elements """ 
