@@ -2,11 +2,12 @@
 #
 #
 
-""" core bot commands """
-
 __copyright__ = 'this file is in the public domain'
+__gendocskip__ = ['userhostcache', ]
 
-from gozerbot.generic import getwho, die, reboot, elapsedstring
+from gozerbot.generic import getwho, die, reboot, elapsedstring, \
+reboot_stateful, getversion
+from gozerbot.exit import globalshutdown
 from gozerbot.commands import cmnds
 from gozerbot.examples import examples
 from gozerbot.callbacks import callbacks
@@ -18,27 +19,31 @@ from gozerbot.fleet import fleet
 from gozerbot.config import config
 from gozerbot.statdict import Statdict
 from gozerbot.aliases import aliases, aliasreverse, aliasset
+from gozerbot.partyline import partyline
 from gozerbot.plughelp import plughelp
 from gozerbot.eventhandler import mainhandler
-from gozerbot.backup import backupstop
-from gozerbot.dol import Dol
+from sets import Set
 import time, threading, sys, re
 
 plughelp.add('core', 'core commands for the bot')
 
 def handle_whatperms(bot, ievent):
+    """ show possible permissions """
     ievent.reply("the following permission are available: ", \
 plugins.whatperms())
 
 cmnds.add('whatperms', handle_whatperms, ['USER', 'OPER', 'ANON'])
+examples.add('whatperms', 'show what permissions are available', 'whatperms')
 
 def handle_encoding(bot, ievent):
+    """ show default encoding """
     ievent.reply('default encoding is %s' % sys.getdefaultencoding())
 
 cmnds.add('encoding', handle_encoding, ['USER', 'OPER', 'ANON'])
 examples.add('encoding', 'show default encoding', 'encoding')
 
 def handle_uptime(bot, ievent):
+    """ show uptime """
     ievent.reply("uptime is %s" % elapsedstring(time.time()-bot.starttime))
 
 cmnds.add('uptime', handle_uptime, ['USER', 'WEB', 'JCOLL', 'ANON'])
@@ -47,23 +52,18 @@ aliasset('up', 'uptime')
 
 def handle_userhostcache(bot, ievent):
     """ show bots userhost cache """
-    ievent.reply(str(bot.userhosts))
+    ievent.reply(str(bot.userhosts.data))
 
 cmnds.add('userhostcache', handle_userhostcache, 'OPER')
 
 def handle_list(bot, ievent):
     """ list [<plugin>] .. list loaded plugins or list commands provided by \
         plugin """
-    plugslist = plugins.list()
     try:
         what = ievent.args[0]
     except:
         # no arguments given .. show plugins
-        result = []
-        for i in plugslist:
-            result.append(i.split('.')[-1])
-        result.sort()
-        ievent.reply('we have the following plugins: ', result)
+        ievent.reply('\002loaded plugins:\002 ', plugins.list())
         return
     # show commands of <what> plugin
     result = []
@@ -89,68 +89,82 @@ plugin', '1) list 2) list rss')
 
 def handle_available(bot, ievent):
     """ available .. show what plugins are available """
-    ievent.reply('available plugins: ', plugins.available())
+    l = Set(plugins.list())
+    ondisk = Set(plugins.available())
+    diff = list(ondisk - l)    
+    diff.sort()
+    ievent.reply('\002available plugins:\002 (reload to enable): ', diff)
 
 cmnds.add('available', handle_available, ['USER', 'WEB', 'ANON'])
-examples.add('available', 'show what plugins are available', 'available')
+examples.add('available', 'show what plugins are available but not loaded \
+(see the list command for loaded plugins)', 'available')
 aliasset('avail', ' available')
+aliasset('plugins', 'available')
 
 def doquit(bot, ievent):
     """ quit .. quit the bot """
-    try:
-        backupstop = 1
-        fleet.exitall()
-        plugins.exit()
-    finally:
-        die()
+    globalshutdown()
 
 cmnds.add('quit', doquit, 'OPER')
 examples.add('quit', 'quit the bot', 'quit')
 aliasset('shutdown', 'quit')
+aliasset('exit', 'quit')
+aliasset('halt', 'quit')
 
 def handle_reboot(bot, ievent):
     """ reboot .. reboot the bot """
-    try:
+    ievent.reply('rebooting')
+    global backupstop
+    stateful = True
+    if ievent.rest == 'cold':
+        stateful = False
+    time.sleep(2)
+    try: 
         backupstop = 1
-        fleet.exitall()
+        if not stateful:
+            fleet.exitall()
         plugins.exit()
     finally:
-        mainhandler.put(0, reboot)
+        if stateful:
+            mainhandler.put(0, reboot_stateful, bot, ievent, fleet, partyline)
+        else:
+            mainhandler.put(0, reboot)
 
 cmnds.add('reboot', handle_reboot, 'OPER')
 examples.add('reboot', 'restart the bot', 'reboot')
 
 def handle_commands(bot, ievent):
-    """ commands [<permission>] .. show commands allowed by permission .. \
-        default permission is 'USER' """
+    """ commands <plugin> .. show commands of <plugin> """
     try:
-        what = ievent.args[0].upper()
+        plugin = ievent.args[0].lower()
     except IndexError:
-        what = users.getperms(ievent.userhost)
-    cmndlist = cmnds.list(what)
-    result = Dol()
-    result2 = []
-    tmp = []
-    for i in cmndlist:
-        try:
-            a, b = i.split('-')
-            result.adduniq(a, b)
-        except:
-            if i not in result2:
-                result2.append(i)
-    for i, j in result.iteritems():
-        tmp = j
-        tmp.sort()
-        result2.append("%s-(%s)" % (i, '|'.join(tmp)))
-    result2.sort()
-    if result2:  
-        ievent.reply('commands for %s permission: ' % what, result2, dot=True)
+        ievent.missing('<plugin> .. see the list command for available \
+plugins')
+        return
+    if not plugins.plugs.has_key(plugin):
+        ievent.reply('no %s plugin is loaded .. see the available command for \
+available plugins (reload to enable)' % plugin)
+        return
+    result = []
+    for i, j in cmnds.iteritems():
+        if plugin == j.plugname:
+            txt = ""
+            alias = aliasreverse(i)
+            if alias:
+                txt += "%s (%s)" % (i, alias)
+            else:
+                txt = i
+            if txt:
+                result.append(txt)
+    if result:
+        result.sort()
+        ievent.reply('%s has the following commands: ' % plugin, result, \
+dot=True)
     else:
-        ievent.reply("can't find any command for permission %s" % what)
+        ievent.reply('no commands found for plugin %s' % plugin)
 
 cmnds.add('commands', handle_commands, ['USER', 'WEB', 'ANON'])
-examples.add('commands', 'show commands allowed for [<permission>]', '1) \
-commands 2) commands OPER')
+examples.add('commands', 'show commands of <plugin>', '1) commands core')
 
 def handle_perm(bot, ievent):
     """ perm <command> .. get permission of command """
@@ -187,10 +201,7 @@ def handle_cc(bot, ievent):
             return
         try:
             bot.channels[chan]['cc'] = what
-        except KeyError:
-            ievent.reply("no channel %s in database")
-            return
-        except TypeError:
+        except (KeyError, TypeError):
             ievent.reply("no channel %s in database" % chan)
             return
         bot.channels.save()
@@ -201,16 +212,15 @@ def handle_cc(bot, ievent):
             cchar = bot.channels[chan]['cc']
             ievent.reply('control character(s) for channel %s are/is %s' % \
 (chan, cchar))
-        except KeyError:
-            ievent.reply("default cc is !")
-        except TypeError:
+        except (KeyError, TypeError):
             ievent.reply("default cc is !")
 
 cmnds.add('cc', handle_cc, 'USER', allowqueue=False)
 examples.add('cc', 'set control char of channel or show control char of \
-channel','1) cc # 2) cc')
+channel','1) cc ! 2) cc')
 
 def handle_ccadd(bot, ievent):
+    """ add a control char to the channels cc list """
     try:
         chan = ievent.args[1]
     except IndexError:
@@ -224,10 +234,7 @@ def handle_ccadd(bot, ievent):
             return
         try:
             bot.channels[chan]['cc'] += what
-        except KeyError:
-            ievent.reply("no channel %s in database")
-            return
-        except TypeError:
+        except (KeyError, TypeError):
             ievent.reply("no channel %s in database" % chan)
             return
         bot.channels.save()
@@ -236,8 +243,10 @@ def handle_ccadd(bot, ievent):
         ievent.missing('<cc> [<channel>]')
 
 cmnds.add('cc-add', handle_ccadd, 'OPER', allowqueue=False)
+examples.add('cc-add', 'cc-add <control char> .. add control character', 'cc-add #')
 
 def handle_ccdel(bot, ievent):
+    """ remove a control char from the channels cc list """
     try:
         chan = ievent.args[1]
     except IndexError:
@@ -264,6 +273,7 @@ bot.channels[chan]['cc'].replace(what, '')
         ievent.missing('<cc> [<channel>]')
 
 cmnds.add('cc-del', handle_ccdel, 'OPER')
+examples.add('cc-del', 'cc-del <control character> .. remove cc', 'cc-del #')
 
 def handle_intro(bot, ievent):
     """ intro <nick> .. do whois on nick to sync userhosts cache """
@@ -276,7 +286,8 @@ def handle_intro(bot, ievent):
     ievent.reply('whois command send')
 
 cmnds.add('intro', handle_intro, 'OPER')
-examples.add('intro', 'intro <nick>', 'intro dunker')
+examples.add('intro', 'do a whois of <nick> to sync userhost into the \
+userhost cache', 'intro dunker')
 
 def handle_loglevel(bot, ievent):
     """ loglevel <level> .. get or set loglevel .. the lower the more the \
@@ -293,8 +304,9 @@ def handle_loglevel(bot, ievent):
     ievent.reply('loglevel is now %s' % config['loglevel'])
 
 cmnds.add('loglevel', handle_loglevel,'OPER')
-examples.add('loglevel', 'get/set current loglevel', '1) loglevel 2) \
-loglevel 0')
+examples.add('loglevel', 'get/set current loglevel .. the lower the loglevel \
+the more the bot logs', '1) loglevel 2) \
+loglevel 100')
 
 def handle_partylist(bot, ievent):
     """ partylist .. show users on partylist """
@@ -335,11 +347,11 @@ def handle_running(bot, ievent):
     ievent.reply("threads running: ", result)
 
 cmnds.add('running', handle_running, ['USER', 'OPER', 'ANON'])
-examples.add('running', 'show from which files threads are running', 'running')
+examples.add('running', 'show running threads', 'running')
 
 def handle_save(bot, ievent):
     """ save .. save data to disk """
-    ievent.reply('saving ..')
+    ievent.reply('saving')
     plugins.save()
     fleet.save()
     config.save()
@@ -350,7 +362,7 @@ examples.add('save', 'save bot data', 'save')
 
 def handle_version(bot, ievent):
     """ version .. show bot's version """
-    ievent.reply(config['version'])
+    ievent.reply(getversion())
 
 cmnds.add('version', handle_version, ['USER', 'WEB', 'JCOLL', 'ANON'])
 examples.add('version', 'show version of the bot', 'version')
@@ -379,8 +391,9 @@ def handle_help(bot, ievent):
     try:
         what = ievent.args[0]
     except IndexError:
-        ievent.reply('help <cmnd> or help <plugin> .. see the !available \
-command for a list of available plugins')
+        ievent.reply('help <cmnd> or help <plugin> .. see the !list \
+command for a list of available plugins or see !available command for a list \
+of plugins to be reloaded')
         return
     phelp = plughelp.get(what)
     cmndresult = []
@@ -390,8 +403,9 @@ command for a list of available plugins')
         for i, j in cmnds.iteritems():
             if what == j.plugname:
                 for perm in j.perms:
-                    if perm in perms:                
-                        cmndresult.append(i)
+                    if perm in perms:
+                        if i not in cmndresult:
+                            cmndresult.append(i)
         if cmndresult:
             cmndresult.sort()
             resultstr = ""
@@ -443,8 +457,8 @@ str(perms))
         example = examples[what]
     except KeyError:
         return
-    ievent.reply('command description of %s: %s .. alias: %s .. examples:\
- %s' % (what, example.descr, aliasreverse(what), example.example))
+    ievent.reply('%s .. alias: %s .. examples: %s' % (example.descr, \
+aliasreverse(what), example.example))
 
 cmnds.add('help', handle_help, ['USER', 'WEB', 'ANON'])
 examples.add('help', 'get help on <cmnd> or <plugin>', '1) help test 2) \
@@ -454,7 +468,7 @@ def handle_apro(bot, ievent):
     """ apro <cmnd> .. apropos for command """
     try:
         what = ievent.args[0]
-    except:
+    except IndexError:
         ievent.missing('<what>')
         return
     result = []
@@ -481,7 +495,7 @@ def handle_u(bot, ievent):
     """ u <nick> .. show userhost entry in cache """
     try:
         nick = ievent.args[0]
-    except:
+    except IndexError:
         ievent.missing('<nick>')
         return
     result = getwho(bot, nick)
@@ -495,6 +509,7 @@ examples.add('u', 'u <nick> .. get userhost cache entry for <nick>', \
 'u dunker')
 
 def handle_less(bot, ievent):
+    """ get entry from the output cache """
     try:
         if len(ievent.args) == 3:
             (who, index1, index2) = ievent.args
@@ -534,6 +549,7 @@ examples.add('less', "less [<who>] [<index1>] <index2> .. get txt from bots \
 output cache", '1) less 0 2) less 0 2 3) less bart 1 0')
 
 def handle_lesssize(bot, ievent):
+    """ show size of output cache """
     try:
         who = ievent.args[0]
     except IndexError:
@@ -545,6 +561,7 @@ examples.add('less-size', "show sizes of data in bot's ouput cache", \
 'less-size')
 
 def handle_more(bot, ievent):
+    """ pop message from the output cache """
     try:
         who = ievent.args[0]
     except IndexError:
@@ -557,15 +574,15 @@ def handle_more(bot, ievent):
         ievent.reply('no more data available for %s' % who)
         return
     if bot.jabber:
-         if size:
-             bot.say(ievent.userhost, "%s (+%s)" % (what, size))
-         else:
-             ievent.reply(what)
-         return      
+        if size:
+            bot.say(ievent.userhost, "%s (+%s)" % (what, size))
+        else:
+            ievent.reply(what)
+        return      
     if size:
-        bot.output(ievent.printto, "%s (+%s)" % (what, size))
+        bot.output(ievent.printto, "%s \002(+%s)\002" % (what, size))
     else:
         ievent.reply(what)
 
 cmnds.add('more', handle_more, ['USER', 'ANON'])
-examples.add('more', 'return txt from output cache', 'more')
+examples.add('more', 'return txt from output cache', '1) more 2) more test')
