@@ -2,15 +2,18 @@
 #
 #
 
-""" use in a pipeline .. shows nr of elements """
-
 __copyright__ = 'this file is in the public domain'
 
 from gozerbot.commands import cmnds
 from gozerbot.generic import waitforqueue
 from gozerbot.examples import examples
+from gozerbot.plughelp import plughelp
+
+plughelp.add('count', 'the count command counts the number of results in a \
+pipeline')
 
 def handle_count(bot, ievent):
+    """ show nr of elements in result list """
     if not ievent.inqueue:
         ievent.reply("use count in a pipeline")
         return
