@@ -2,9 +2,12 @@
 #
 #
 
-""" fleet related commands """
+""" the fleet makes it possible to run multiple bots in one gozerbot.
+ this can both be irc and jabber bots
+"""
 
 __copyright__ = 'this file is in the public domain'
+__gendoclast__ = ['fleet-disconnect', 'fleet-del']
 
 from gozerbot.fleet import fleet
 from gozerbot.commands import cmnds
@@ -22,14 +25,17 @@ def handle_fleetconnect(bot, ievent):
     except IndexError:
         ievent.missing('<botname>')
         return
-    if fleet.connect(botname):
-        ievent.reply('%s connected' % botname)
-    else:
-        ievent.reply("can't connect %s" % botname)
+    try:
+        if fleet.connect(botname):
+            ievent.reply('%s connected' % botname)
+        else:
+            ievent.reply("can't connect %s" % botname)
+    except Exception, ex:
+        ievent.reply(str(ex))
 
 cmnds.add('fleet-connect', handle_fleetconnect, 'OPER')
 examples.add('fleet-connect', 'connect bot with <name> to irc \
-server', 'fleet-connect undernet')
+server', 'fleet-connect test')
 
 def handle_fleetdisconnect(bot, ievent):
     """ fleet-disconnect <botname> .. disconnect a fleet bot from server """
@@ -39,14 +45,17 @@ def handle_fleetdisconnect(bot, ievent):
         ievent.missing('<botname>')
         return
     ievent.reply('exiting %s' % botname)
-    if fleet.exit(botname):
-        ievent.reply("%s bot stopped" % botname)
-    else:
-        ievent.reply("can't stop %s bot" % botname)
+    try:
+        if fleet.exit(botname):
+            ievent.reply("%s bot stopped" % botname)
+        else:
+            ievent.reply("can't stop %s bot" % botname)
+    except Exception, ex:
+        ievent.reply(str(ex))
 
 cmnds.add('fleet-disconnect', handle_fleetdisconnect, 'OPER')
 examples.add('fleet-disconnect', 'fleet-disconnect <name> .. disconnect \
-bot with <name> from irc server', 'fleet-disconnect undernet')
+bot with <name> from irc server', 'fleet-disconnect test')
 
 def handle_fleetlist(bot, ievent):
     """ fleet-list .. list bot names in fleet """
@@ -60,41 +69,56 @@ def handle_fleetaddirc(bot, ievent):
         bot to fleet """
     from gozerbot.bot import Bot
     length  = len(ievent.args)
-    if length == 6:
-        (name, nick, server, ipv6, port, password) = ievent.args
+    if length == 7:
+        (name, nick, server, ipv6, ssl, port, password) = ievent.args
+    elif length == 6:
+        (name, nick, server, ipv6, ssl, port) = ievent.args
+        password = ""
     elif length == 5:
-        (name, nick, server, ipv6,  port) = ievent.args
-        passwd = None
+        (name, nick, server, ipv6, ssl) = ievent.args
+        password = ""
+        if ssl:
+           port = 6697
+        else:
+           port = 6667
     elif length == 4:
         (name, nick, server, ipv6) = ievent.args
-        password = None
+        password = ""
+        ssl = 0
         port = 6667
     elif length == 3:
         (name, nick, server) = ievent.args
         port = 6667
-        password = None
+        password = ""
+        ssl = 0
         ipv6 = 0
     else:
-        ievent.missing('<name> <nick> <server> [<ipv6>] [<port>] [<password>]')
+        ievent.missing('<name> <nick> <server> [<ipv6>] [<ssl>] [<port>] \
+[<password>]')
         return
     if fleet.byname(name):
         ievent.reply('we already have a bot with %s name in fleet' % \
 name)
         return
-    b = Bot(config['owneruserhost'], name)
+    b = Bot(name, config['owneruserhost'])
     b.nick = nick
     b.server = server
     b.port = port
     b.password = password
     b.ipv6 = ipv6
-    fleet.addbot(b)
-    fleet.connect(name)
-    ievent.reply('%s started' % name)
+    b.ssl = ssl
+    try:
+        fleet.addbot(b)
+        fleet.connect(name)
+        ievent.reply('%s started' % name)
+    except Exception, ex:
+        ievent.reply(str(ex))
+        fleet.delete(b)
 
 cmnds.add('fleet-addirc', handle_fleetaddirc, 'OPER')
-examples.add('fleet-addirc', 'fleet-addirc <name> <nick> <server> [port] \
-[passwd] [ipv6] .. add new server to fleet', 'fleet-addirc test gozertest \
-irc.xs4all.nl')
+examples.add('fleet-addirc', 'fleet-addirc <name> <nick> <server> [ipv6] \
+[port] [passwd] .. add new server to fleet', 'fleet-addirc test3 gozertest \
+localhost')
 aliasset('fleet-add', 'fleet-addirc')
 
 def handle_fleetaddjabber(bot, ievent):
@@ -132,13 +156,17 @@ name)
     b.user = user
     b.password = password
     b.port = port
-    fleet.addbot(b)
-    fleet.connect(name)
-    ievent.reply('%s started' % name)
+    try:
+        fleet.addbot(b)
+        fleet.connect(name)
+        ievent.reply('%s started' % name)
+    except Exception, ex:
+        ievent.reply(str(ex))
+        fleet.delete(b)
 
 cmnds.add('fleet-addjabber', handle_fleetaddjabber, 'OPER')
 examples.add('fleet-addjabber', 'fleet-addjabber <name> <host> <user> \
-<passwd> [<port>] .. add new jabber server to fleet', 'fleet-add test \
+<passwd> [<port>] .. add new jabber server to fleet', 'fleet-addjabber test2 \
 jabber.xs4all.nl jtest@jabber.xs4all.nl xwe23')
 
 def handle_fleetdel(bot, ievent):
@@ -148,10 +176,13 @@ def handle_fleetdel(bot, ievent):
     except IndexError:
         ievent.missing('<name>')
         return
-    if fleet.delete(name):
-        ievent.reply('%s deleted' % name)
-    else:
-        ievent.reply('%s delete failed' % name)
+    try:
+        if fleet.delete(name):
+            ievent.reply('%s deleted' % name)
+        else:
+            ievent.reply('%s delete failed' % name)
+    except Exception, ex:
+        ievent.reply(str(ex))
 
 cmnds.add('fleet-del', handle_fleetdel, 'OPER')
 examples.add('fleet-del', 'fleet-del <botname> .. delete bot from fleet \
@@ -171,11 +202,14 @@ def docmnd(bot, ievent):
     if cmnd.find('cmnd') != -1:
         ievent.reply("no looping please ;]")
         return
-    if name == 'all':
-        fleet.cmndall(ievent, cmnd)
-    else:
-        fleet.cmnd(ievent, name, cmnd)
+    try:
+        if name == 'all':
+            fleet.cmndall(ievent, cmnd)
+        else:
+            fleet.cmnd(ievent, name, cmnd)
+    except Exception, ex:
+        ievent.reply(str(ex))
 
 cmnds.add('cmnd', docmnd, ['USER', 'WEB'])
-examples.add('cmnd', "cmnd 'all'|<botname> <cmnd> .. excecute command on \
+examples.add('cmnd', "cmnd all|<botname> <cmnd> .. excecute command on \
 bot with <name> or on all fleet bots", '1) cmnd main st 2) cmnd all st')
