@@ -2,10 +2,6 @@
 #
 #
 
-""" hack to make jabber work with google talk .. this needs the bot to be
-    in a googletalk user roster since it uses
-"""
-
 __copyright__ = 'this file is in the public domain'
 
 from gozerbot.callbacks import jcallbacks
@@ -36,6 +32,7 @@ def googlejidchange(bot, msg):
     newjid = msg.getAttrs()['to']
     bot.jid = xmpp.JID(newjid)
     bot.me = str(bot.jid)
+    bot.google = True
     rlog(10, 'googletalk', "changed ident to %s" % bot.me)
 
 jcallbacks.add('Presence', googlejidchange, googletalktest)
