@@ -2,13 +2,12 @@
 #
 #
 
-""" used in a pipeline .. grep elements """
-
 __copyright__ = 'this file is in the public domain'
 
 from gozerbot.commands import cmnds
 from gozerbot.generic import waitforqueue
 from gozerbot.plughelp import plughelp
+from gozerbot.examples import examples
 import getopt, re
 
 plughelp.add('grep', ' grep elements of a pipeline')
@@ -81,3 +80,4 @@ def handle_grep(bot, ievent):
         ievent.reply(res, dot=True)
 
 cmnds.add('grep', handle_grep, ['USER', 'WEB', 'ANON'])
+examples.add('grep', 'grep the output of a command', 'list | grep todo')
