@@ -3,7 +3,7 @@
 # (c) Wijnand 'tehmaze' Modderman - http://tehmaze.com
 # BSD License
 
-""" hg related information, only works on hg version of the bot """
+""" only works on hg version of the bot """
 
 from gozerbot.generic import gozerpopen
 from gozerbot.aliases import aliases
@@ -144,7 +144,7 @@ str(returncode))
 
 cmnds.add('hg-changes', handle_hg_changes, 'USER')
 examples.add('hg-changes', 'show changes in the current release (tip) or \
-between two releases', '1) hg-changes tip 2) hg-changes 666 tip')
+between two releases', '1) hg-changes tip 2) hg-changes 1900 tip')
 
 def handle_hg_committers(bot, ievent):
     """ show hg committers and the number of commits they made """
@@ -187,7 +187,7 @@ str(returncode))
         ievent.reply('no result')
 
 cmnds.add('hg-committers', handle_hg_committers, 'USER')
-examples.add('hg-committers', 'shows hg committer statistics', 'hg-committers [short]')
+examples.add('hg-committers', 'shows hg committer statistics', 'hg-committers')
 aliases.data['committers'] = 'hg-committers'
 
 def handle_hglog(bot, ievent):
@@ -237,4 +237,4 @@ str(returncode))
 
 cmnds.add('hg-log', handle_hglog, 'USER')
 examples.add('hg-log', 'show the changelog for the repo, or for a file', \
-'hg-log gozerbot/plugs/hg.py')
+'hg-log gozerplugs/plugs/hg.py')
