@@ -9,6 +9,7 @@ __copyright__ = 'this file is in the public domain'
 from gozerbot.commands import cmnds
 from gozerbot.examples import examples
 from gozerbot.redispatcher import rebefore, reafter
+from gozerbot.ignore import shouldignore
 from gozerbot.datadir import datadir
 from gozerbot.persist import Persist
 from gozerbot.generic import lockdec, cchar
@@ -16,7 +17,10 @@ from gozerbot.aliases import aliases
 from gozerbot.plughelp import plughelp
 from gozerbot.callbacks import callbacks
 from gozerbot.users import users
-import thread, os
+from gozerbot.config import config
+if config['dbenable']:
+    from gozerbot.db import db
+import thread, os, time
 
 plughelp.add('infoitem', 'also known as factoids .. info can be retrieved \
 by keyword or searched')
@@ -31,17 +35,18 @@ class Infoitems(Persist):
     """ information items """
 
     @locked
-    def add(self, item, issue):
+    def add(self, item, issue, userhost=None, ttime=None):
         """ add an item """
         item = item.lower()
         if self.data.has_key(item):
-            self.data[item].append(issue)
+            if not issue in self.data[item]:
+                self.data[item].append(issue)
         else:
             self.data[item] = [issue]
         self.save()
 
     @locked
-    def addnosave(self, item, issue):
+    def addnosave(self, item, issue, userhost=None, ttime=None):
         """ add item but don't save """
         item = item.lower()
         if self.data.has_key(item):
@@ -87,10 +92,80 @@ class Infoitems(Persist):
         """ return number of items """
         return len(self.data)
 
-info = Infoitems(datadir + os.sep + 'infoitems')
+    def searchdescr(self, txt):
+        result = []
+        for i, j in self.data.iteritems():
+            for z in j:
+                if txt in z:
+                    result.append((i, z))
+        return result
 
-if not info.data:
-    info.data = {}
+    def searchitem(self, txt):
+        result = []
+        for i, j in self.data.iteritems():
+            if txt in i:
+                result.append((i, j))
+        return result
+
+class InfoitemsDb(object):
+
+    """ information items """
+ 
+    def add(self, item, description, userhost, ttime):
+        """ add an item """
+        item = item.lower()
+        result = db.execute(""" INSERT INTO infoitems(item, description, \
+userhost, time) VALUES(%s, %s, %s, %s) """, (item, description, \
+userhost, ttime))
+        return result
+
+    def get(self, item):
+        """ get infoitems """
+        item = item.lower()
+        result = db.execute(""" SELECT description FROM \
+infoitems WHERE item = %s """, item)
+        res = []
+        if result:
+            for i in result:
+                res.append(i[0])
+        return res
+
+    def delete(self, indexnr):
+        """ delete item with indexnr  """
+        result = db.execute(""" DELETE FROM infoitems WHERE indx = %s """, \
+indexnr)
+        return result
+
+    def deltxt(self, item, txt):
+        """ delete item with matching txt """
+        result = db.execute(""" DELETE FROM infoitems WHERE item = %s AND \
+description LIKE %s """, (item, '%%%s%%' % txt))
+        return result
+
+    def size(self):
+        """ return number of items """
+        result = db.execute(""" SELECT COUNT(*) FROM infoitems """)
+        return result[0][0]
+
+    def searchitem(self, search):
+        """ search items """
+        result = db.execute(""" SELECT item, description \
+FROM infoitems WHERE item LIKE %s """, '%%%s%%' % search)
+        return result
+
+    def searchdescr(self, search):
+        """ search descriptions """
+        result = db.execute(""" SELECT item, description \
+FROM infoitems WHERE description LIKE %s """, '%%%s%%' % search)
+        return result
+
+if config['dbenable']:
+    info = InfoitemsDb()
+else:
+    info = Infoitems(datadir + os.sep + 'infoitems')
+    if not info.data:
+        info.data = {}
+assert(info)
 
 def size():
     """ return number of infoitems """
@@ -105,10 +180,11 @@ and ievent.txt:
 
 def infocb(bot, ievent):
     """ implement a !infoitem callback """
-    if users.allowed(ievent.userhost, 'USER'):
+    if not shouldignore(ievent.userhost) and \
+users.allowed(ievent.userhost, 'USER'):
         data = info.get(ievent.txt)
         if data:
-            ievent.reply('infoitems for %s: ' % ievent.txt, data , dot=True)
+            ievent.reply('%s is ' % ievent.txt, data , dot=True)
 
 callbacks.add('PRIVMSG', infocb, infopre)
 
@@ -130,7 +206,7 @@ def handle_addinfoitem(bot, ievent):
         ievent.reply('i need at least 3 chars for the description')
         return
     what = what.strip()
-    info.add(what, description)
+    info.add(what, description, ievent.userhost, time.time())
     ievent.reply('item added')
 
 rebefore.add(10, '^(.+?)\s+=\s+(.+)$', handle_addinfoitem, ['USER', \
@@ -145,21 +221,17 @@ def handle_question(bot, ievent):
         ievent.reply('i need a argument')
         return
     what = what.strip().lower()
-    try:
-        infoitems = info.data[what]
-    except KeyError:
-        ievent.reply('no infoitems known for %s' % what)
-        return
+    infoitems = info.get(what)
     if infoitems:
-        ievent.reply("infoitems for %s: " % what, infoitems, dot=True)
+        ievent.reply("%s is " % what, infoitems, dot=True)
     else:
-        ievent.reply('no infoitems known for %s' % what)
+        ievent.reply('nothing known about %s' % what)
         return
 
 reafter.add(10, '^(.+)\?$', handle_question, ['USER', 'WEB', 'JCOLL', \
-'ANON'], allowqueue=False)
+'ANON'], allowqueue=True)
 reafter.add(10, '^\?(.+)$', handle_question, ['USER', 'WEB', 'JCOLL', \
-'ANON'], allowqueue=False)
+'ANON'], allowqueue=True)
 examples.add('?', 'show infoitems of <what>', '1) test? 2) ?test')
 
 def handle_forget(bot, ievent):
@@ -198,13 +270,12 @@ def handle_searchdescr(bot, ievent):
     else:
         what = ievent.rest
     what = what.strip().lower()
-    result = []
-    for item, value in info.data.iteritems():
-        for item2 in value:
-            if item2.find(what) != -1:
-                result.append("[%s] %s" % (item, item2))
+    result = info.searchdescr(what)
     if result: 
-        ievent.reply("the following matches %s: " % what, result, dot=True)
+        res = []
+        for i in result:
+            res.append("[%s] %s" % (i[0], i[1]))
+        ievent.reply("the following matches %s: " % what, res, dot=True)
     else:
         ievent.reply('none found')
 
@@ -214,7 +285,7 @@ infoitems', 'info-sd http')
 aliases.data['sd'] = 'info-sd'
 aliases.data['sl'] = 'info-sd'
 
-def handle_searchinfo(bot, ievent):
+def handle_searchitem(bot, ievent):
     """ info-si <txt> .. search information keywords """
     if not ievent.rest:
         ievent.missing('<txt>')
@@ -222,16 +293,16 @@ def handle_searchinfo(bot, ievent):
     else:
         what = ievent.rest
     what = what.strip().lower()
-    result = []
-    for item in info.data.keys():
-        if item.find(what) != -1:
-            result.append(item)
+    result = info.searchitem(what)
     if result:
-        ievent.reply("the following matches %s: " % what, result, dot=True)
+        res = []
+        for i in result:
+            res.append("[%s] %s" % (i[0], i[1]))
+        ievent.reply("the following matches %s: " % what, res, dot=True)
     else:
         ievent.reply('none found')
 
-cmnds.add('info-si', handle_searchinfo, ['USER', 'WEB', 'ANON'])
+cmnds.add('info-si', handle_searchitem, ['USER', 'WEB', 'ANON'])
 examples.add('info-si', 'info-si <txt> ..  search the infoitems keys', \
 'info-si test')
 aliases.data['si'] = 'info-si'
