@@ -8,14 +8,16 @@ __copyright__ = 'this file is in the public domain'
 
 from gozerbot.commands import cmnds
 from gozerbot.examples import examples
-from gozerbot.redispatcher import reafter, rebefore
+from gozerbot.redispatcher import rebefore
 from gozerbot.datadir import datadir
 from gozerbot.generic import handle_exception, rlog, lockdec
 from gozerbot.statdict import Statdict
-from gozerbot.users import users
 from gozerbot.aliases import aliases
 from gozerbot.plughelp import plughelp
+from gozerbot.config import config
 from gozerplugs.plugs.links import links
+if config['dbenable']:
+    from gozerbot.db import db
 import thread, pickle, time, os
 
 plughelp.add('karma', 'maintain karma of items .. use ++ to raise karma by 1 \
@@ -239,14 +241,16 @@ class Karma:
         return statdict.down(limit=limit)
 
     def search(self, item):
+        """ search karma items """
         result = []
         item = item.lower()
-        for i,j in self.karma.iteritems():
+        for i, j in self.karma.iteritems():
             if item in i:
                 result.append((i, j))
         return result
 
     def whatup(self, nick):
+        """ show what items where upped by nick """
         nick = nick.lower()
         statdict = Statdict()
         for i, j in self.whoup.iteritems():
@@ -256,6 +260,7 @@ class Karma:
         return statdict.top()
 
     def whatdown(self, nick):
+        """ show what items where lowered by nick """
         nick = nick.lower()
         statdict = Statdict()
         for i, j in self.whodown.iteritems():
@@ -264,7 +269,232 @@ class Karma:
                     statdict.upitem(i)
         return statdict.top()
 
-karma = Karma(datadir)
+class KarmaDb(object):
+
+    """ karma object """
+
+    from gozerbot.db import db
+
+    def save(self):
+        pass
+
+    def size(self):
+        """ return number of karma items """
+        nritems = db.execute(""" SELECT COUNT(*) FROM karma """)
+        return nritems[0][0]
+
+    def get(self, item):
+        """ get karma of item """
+        item = item.lower()
+        value = db.execute(""" SELECT value FROM karma WHERE item = %s """, \
+item)
+        if value:
+            return value[0][0]
+        else:
+            return 0
+
+    def delete(self, item):
+        """ delete karma item """
+        item = item.lower()
+        value = db.execute(""" DELETE FROM karma WHERE item = %s """, item)
+        return value
+
+    def addwhy(self, item, updown, reason):
+        """ add why of karma up/down """
+        item = item.lower()
+        result = db.execute(""" INSERT INTO whykarma(item, updown, why) \
+VALUES (%s, %s, %s) """, (item, updown, reason))
+        return result
+            
+    def upitem(self, item, reason=None):
+        """ up a karma item with/without reason """
+        item = item.lower()
+        value = db.execute(""" SELECT value FROM karma WHERE item = %s """, \
+item)
+        try:
+            val = value[0][0]
+        except (TypeError, IndexError):
+            result = db.execute(""" INSERT INTO karma(item, value) VALUES \
+(%s, 1) """, item)
+            if reason:
+                self.addwhy(item, 'up', reason.strip())
+            return result
+        val += 1
+        result = db.execute(""" UPDATE karma SET value = %s WHERE item = %s \
+""", (val, item))
+        if reason:
+            self.addwhy(item, 'up', reason.strip())
+        return result
+
+    def down(self, item, reason=None):
+        """ lower a karma item with/without reason """
+        item = item.lower()
+        value = db.execute(""" SELECT value FROM karma WHERE item = %s """, \
+item)
+        try:
+            val = value[0][0]
+        except (TypeError, IndexError):
+            result = db.execute(""" INSERT INTO karma(item, value) VALUES \
+(%s, -1) """, item)
+            if reason:
+                self.addwhy(item, 'down', reason.strip())
+            return result
+        val -= 1
+        result = db.execute(""" UPDATE karma SET value = %s WHERE item = %s \
+""", (val, item))
+        if reason:
+            self.addwhy(item, 'down', reason.strip())
+        return result
+
+    def whykarmaup(self, item):
+        """ get why of karma ups """
+        item = item.lower()
+        result = db.execute(""" SELECT why FROM whykarma WHERE \
+item = %s AND updown = 'up' """, item)
+        res = []
+        if result:
+             for i in result:
+                 res.append(i[0])
+        return res
+
+    def whykarmadown(self, item):
+        """ get why of karma downs """
+        item = item.lower()
+        result = db.execute(""" SELECT why FROM whykarma WHERE \
+item = %s AND updown = 'down' """, item)
+        res = []
+        if result:
+             for i in result:
+                 res.append(i[0])
+        return res
+
+    def setwhoup(self, item, nick):
+        """ set who upped a karma item """
+        item = item.lower()
+        nick = nick.lower()
+        result = db.execute(""" INSERT INTO whokarma(item, nick, updown) \
+VALUES(%s, %s, %s) """, (item, nick, 'up'))
+        return result
+ 
+    def setwhodown(self, item, nick):
+        """ set who lowered a karma item """
+        item = item.lower()
+        nick = nick.lower()
+        result = db.execute(""" INSERT INTO whokarma(item, nick, updown) \
+VALUES(%s, %s, %s) """, (item, nick, 'down'))
+        return result
+
+    def getwhoup(self, item):
+        """ get list of who upped a karma item """
+        item = item.lower()
+        result = db.execute(""" SELECT nick FROM whokarma WHERE item = %s \
+AND updown = 'up' """, item)
+        res = []
+        if result:
+             for i in result:
+                 res.append(i[0])
+        return res
+
+    def getwhodown(self, item):
+        """ get list of who downed a karma item """
+        item = item.lower()
+        result = db.execute(""" SELECT nick FROM whokarma WHERE item = %s \
+AND updown = 'down' """, item)
+        res = []
+        if result:
+             for i in result:
+                 res.append(i[0])
+        return res
+
+    def search(self, item):
+        """ search for matching karma item """
+        item = item.lower()
+        result = db.execute(""" SELECT item,value FROM karma WHERE item LIKE \
+%s """, '%%%s%%' % item)
+        res = []
+        if result:
+             for i in result:
+                 res.append(i)
+        return res
+
+    def good(self, limit=10):
+        """ show top 10 of karma items """
+        statdict = Statdict()
+        result = db.execute(""" SELECT item, value FROM karma """)
+        if not result:
+            return []
+        for i in result:
+            if i[0].startswith('quote '):
+                continue
+            statdict.upitem(i[0], value=i[1])
+        return statdict.top(limit=limit)
+
+    def bad(self, limit=10):
+        """ show lowest 10 of negative karma items """
+        statdict = Statdict()
+        result = db.execute(""" SELECT item, value FROM karma """)
+        if not result:
+            return []
+        for i in result:
+            if i[0].startswith('quote '):
+                continue
+            statdict.upitem(i[0], value=i[1])
+        return statdict.down(limit=limit)
+
+    def quotegood(self, limit=10):
+        """ show top 10 of karma items """
+        statdict = Statdict()
+        result = db.execute(""" SELECT item, value FROM karma """)
+        if not result:
+            return []
+        for i in result:
+            if not i[0].startswith('quote '):
+                continue
+            statdict.upitem(i[0], value=i[1])
+        return statdict.top(limit=limit)
+
+    def quotebad(self, limit=10):
+        """ show lowest 10 of negative karma items """
+        statdict = Statdict()
+        result = db.execute(""" SELECT item, value FROM karma """)
+        if not result:
+            return []
+        for i in result:
+            if not i[0].startswith('quote '):
+                continue
+            statdict.upitem(i[0], value=i[1])
+        return statdict.down(limit=limit)
+
+    def whatup(self, nick):
+        """ show what items are upped by nick """
+        nick = nick.lower()
+        statdict = Statdict()
+        result = db.execute(""" SELECT item FROM whokarma WHERE nick = %s \
+AND updown = 'up' """, nick)
+        if not result:
+            return []
+        for i in result:
+            statdict.upitem(i[0])
+        return statdict.top()
+
+    def whatdown(self, nick):
+        """ show what items are lowered by nick """
+        nick = nick.lower()
+        statdict = Statdict()
+        result = db.execute(""" SELECT item FROM whokarma WHERE nick = %s \
+AND updown = 'down' """, nick)
+        if not result:
+            return []
+        for i in result:
+            statdict.upitem(i[0])
+        return statdict.top()
+
+if config['dbenable']:
+    karma = KarmaDb()
+else:
+    karma = Karma(datadir)
+assert(karma)
+
 savelist.append(karma)
 
 ratelimited = []
@@ -272,6 +502,7 @@ limiterlock = thread.allocate_lock()
 limlock = lockdec(limiterlock)
 
 def size():
+    """ return number of kamra items """
     return karma.size()
 
 @limlock
@@ -304,7 +535,7 @@ seconds" % int((bot.state[name]['karma']['time'] + waittime) - time.time()))
         # Ratelimiting passed :)
         try:
             ratelimited.remove(name)
-        except:
+        except ValueError:
             pass
         return 1
     except Exception, ex:
@@ -365,12 +596,14 @@ def handle_karmaup(bot, ievent):
         return
     (item, reason) = ievent.groups
     item = item.strip().lower()
+    if not item:
+        return
     karma.upitem(item, reason=reason)
     karma.setwhoup(item, ievent.nick)
     ievent.reply('karma of '+ item + ' is now ' + str(karma.get(item)))
 
-rebefore.add(8, '^(.+)\+\+\s+#(.*)$', handle_karmaup, ['USER', 'KARMA', 'ANONKARMA'], \
- allowqueue=False)
+rebefore.add(8, '^(.+)\+\+\s+#(.*)$', handle_karmaup, ['USER', 'KARMA', \
+'ANONKARMA'],  allowqueue=False)
 examples.add('++', "<item>++ ['#' <reason>] .. higher karma of item with 1 \
 (use optional reason)", '1) gozerbot++ 2) gozerbot++ # top bot')
 
@@ -379,8 +612,8 @@ def handle_karmaup2(bot, ievent):
     ievent.groups += [None, ]
     handle_karmaup(bot, ievent)
 
-rebefore.add(9, '^(.+)\+\+$', handle_karmaup2, ['USER', 'ANON', 'ANONKARMA'], \
-allowqueue=False)
+rebefore.add(9, '^(.+)\+\+$', handle_karmaup2, ['USER', 'ANON', \
+'ANONKARMA'], allowqueue=False)
 
 def handle_karmadown(bot, ievent):
     """ <item>-- ['#' <reason> .. decrease karma item with reason """
@@ -388,12 +621,14 @@ def handle_karmadown(bot, ievent):
         return
     (item, reason) = ievent.groups
     item = item.strip().lower()
+    if not item:
+        return
     karma.down(item, reason=reason)
     karma.setwhodown(item, ievent.nick)
     ievent.reply('karma of ' + item + ' is now ' + str(karma.get(item)))
 
-rebefore.add(8, '^(.+)\-\-\s+#(.*)$', handle_karmadown, ['USER', 'KARMA', 'ANONKARMA'], \
-allowqueue=False)
+rebefore.add(8, '^(.+)\-\-\s+#(.*)$', handle_karmadown, ['USER', 'KARMA', \
+'ANONKARMA'], allowqueue=False)
 examples.add('--', "<item>-- ['#' <reason>] .. lower karma of item with 1 \
 (use optional reason)", '1) gozerbot-- 2) gozerbot-- # bad bot')
 
@@ -402,8 +637,8 @@ def handle_karmadown2(bot, ievent):
     ievent.groups += [None, ]
     handle_karmadown(bot, ievent)
 
-rebefore.add(9, '^(.+)\-\-$', handle_karmadown2, ['USER', 'KARMA', 'ANONKARMA'], \
-allowqueue=False)
+rebefore.add(9, '^(.+)\-\-$', handle_karmadown2, ['USER', 'KARMA', \
+'ANONKARMA'], allowqueue=False)
 
 def handle_karmawhyup(bot, ievent):
     """ karma-whyup <item> .. show why karma of item has been increased """
@@ -417,7 +652,8 @@ def handle_karmawhyup(bot, ievent):
     else:
         ievent.reply('%s has no reason for karmaup yet' % item)
 
-cmnds.add('karma-whyup', handle_karmawhyup, ['USER', 'WEB', 'ANON', 'ANONKARMA'])
+cmnds.add('karma-whyup', handle_karmawhyup, ['USER', 'WEB', 'ANON', \
+'ANONKARMA'])
 examples.add('karma-whyup', 'karma-whyup <item> .. show the reason why \
 karma of <item> was raised', 'karma-whyup gozerbot')
 aliases.data['wku'] = 'karma-whyup'
@@ -434,7 +670,8 @@ def handle_whykarmadown(bot, ievent):
     else:
         ievent.reply('%s has no reason for karmadown yet' % item)
 
-cmnds.add('karma-whydown', handle_whykarmadown, ['USER', 'WEB', 'ANON', 'ANONKARMA'])
+cmnds.add('karma-whydown', handle_whykarmadown, ['USER', 'WEB', 'ANON', \
+'ANONKARMA'])
 examples.add('karma-whydown', 'karma-whydown <item> .. show the reason why \
 karma of <item> was lowered', 'karma-whydown gozerbot')
 aliases.data['wkd'] = 'karma-whydown'
@@ -446,7 +683,7 @@ def handle_karmagood(bot, ievent):
         res = []
         for i in result:
             if i[1] > 0:
-                res.append("%s=%s " % (i[0], i[1]))
+                res.append("%s=%s" % (i[0], i[1]))
         ievent.reply('goodness: ', res, dot=True)
     else:
         ievent.reply('karma void')
@@ -462,7 +699,7 @@ def handle_karmabad(bot, ievent):
         res = []
         for i in result:
             if i[1] < 0:
-                res.append("%s=%s " % (i[0], i[1]))
+                res.append("%s=%s" % (i[0], i[1]))
         ievent.reply('badness: ', res, dot=True)
     else:
         ievent.reply('karma void')
@@ -484,12 +721,13 @@ def handle_whokarmaup(bot, ievent):
             statdict.upitem(i)
         res = []
         for i in statdict.top():
-            res.append("%s=%s " % i)
+            res.append("%s=%s" % i)
         ievent.reply("whokarmaup of %s: " % item, res, dot=True)
     else:
         ievent.reply('no whokarmaup data available for %s' % item)
 
-cmnds.add('karma-whoup', handle_whokarmaup, ['USER', 'WEB', 'ANON', 'ANONKARMA'])
+cmnds.add('karma-whoup', handle_whokarmaup, ['USER', 'WEB', 'ANON', \
+'ANONKARMA'])
 examples.add('karma-whoup', 'karma-whoup <item> .. show who raised the \
 karma of <item>', 'karma-whoup gozerbot')
 
@@ -506,12 +744,13 @@ def handle_whokarmadown(bot, ievent):
             statdict.upitem(i)
         res = []
         for i in statdict.top():
-            res.append("%s=%s " % i)
+            res.append("%s=%s" % i)
         ievent.reply("whokarmadown of %s: " % item, res, dot=True)
     else:
         ievent.reply('no whokarmadown data available for %s' % item)
 
-cmnds.add('karma-whodown', handle_whokarmadown, ['USER', 'WEB', 'ANON', 'ANONKARMA'])
+cmnds.add('karma-whodown', handle_whokarmadown, ['USER', 'WEB', 'ANON', \
+'ANONKARMA'])
 examples.add('karma-whodown', 'karma-whodown <item> .. show who lowered \
 the karma of <item>', 'karma-whodown gozerbot')
 
@@ -525,16 +764,18 @@ def handle_karmasearch(bot, ievent):
     if result:
         res = []
         for i in result:
-            res.append("%s (%s)" % i)
-        ievent.reply("karma items matching %s: ", res, dot=True)
+            res.append("%s (%s)" % (i[0], i[1]))
+        ievent.reply("karma items matching %s: " % what, res, dot=True)
     else:
         ievent.reply('no karma items matching %s found' % what)
 
-cmnds.add('karma-search', handle_karmasearch, ['USER', 'WEB', 'ANON', 'ANONKARMA'])
+cmnds.add('karma-search', handle_karmasearch, ['USER', 'WEB', 'ANON', \
+'ANONKARMA'])
 examples.add('karma-search', 'karma-search <txt> .. search karma' , \
 'karma-search gozerbot')
 
 def handle_karmawhatup(bot, ievent):
+    """ show what karma items have been upped by nick """
     try:
         nick = ievent.args[0]
     except IndexError:
@@ -555,6 +796,7 @@ examples.add('karma-whatup', 'karma-whatup <nick> .. show what karma items \
 <nick> has upped' , 'karma-whatup dunker')
 
 def handle_karmawhatdown(bot, ievent):
+    """ show what karma items have been lowered by nick """
     try:
         nick = ievent.args[0]
     except IndexError:
