@@ -7,7 +7,7 @@
 __copyright__ = 'this file is in the public domain'
 
 from gozerbot.generic import rlog, handle_exception, elapsedstring, dmy, \
-hourmin, todefenc, lockdec, strtotime
+hourmin, lockdec, strtotime
 from gozerbot.commands import cmnds
 from gozerbot.examples import examples
 from gozerbot.callbacks import callbacks, jcallbacks
@@ -188,7 +188,9 @@ ievent.txt, filename))
             self.lock.release()
 
     def getmmap(self, channel):
-        """ get an mmap of a channel """
+        """ get an mmap of a channel .. as of 0.8 this just returns the 
+            open logfile
+        """
         if channel[0] in ['#', '!', '&', '+']:
             chan = channel[1:].lower()
         else:
@@ -198,31 +200,23 @@ ievent.txt, filename))
         except KeyError:
             rlog(10, 'log', 'logging is not enabled in %s' % channel)
             return
-        #if self.maps.has_key(filename):
-        #    return self.maps[filename]
-        logfile = open(filename,'r')
-        size = self.size(channel)
-        if size == 0:
-            return None
-        logmap = mmap.mmap(logfile.fileno(), size, mmap.MAP_SHARED, \
-access=mmap.ACCESS_READ)
-        #self.maps[filename] = logmap
-        return logmap
+        logfile = open(filename, 'r')
+        #size = self.size(channel)
+        #if size == 0:  
+        #    return None
+        #logmap = mmap.mmap(logfile.fileno(), size, mmap.MAP_SHARED, \
+#access=mmap.ACCESS_READ)
+        return logfile
 
     def loop(self, logmap, nrtimes, func, withbot=False, withcmnd=False):
         """ loop over the logmap """
         logmap.seek(0)
         teller = 0
         times = 0
-        while 1:
+        for line in logmap:
             teller += 1
             if teller % 100000 == 0:
                 time.sleep(0.0001)
-            #time.sleep(0.000001)
-            line = todefenc(logmap.readline())
-            if not line:
-                break
-            # data is comma seperated
             try:
                 (botname, ttime, nick, userhost, txt) = \
 line.strip().split(',', 4)
@@ -230,10 +224,12 @@ line.strip().split(',', 4)
                 continue
             if not withbot and userhost == 'bot@bot':
                 continue
-            # skip over commands
             if not withcmnd and txt.startswith('CMND:'):
                 continue
-            res = func(botname, ttime, nick, userhost, txt)
+            try:
+                res = func(botname, ttime, nick, userhost, txt)
+            except ValueError:
+                continue
             if res:
                 times += 1
                 if nrtimes and times > nrtimes:
@@ -314,6 +310,36 @@ line.strip().split(',', 4)
         result.reverse()
         return result
 
+    def bback(self, channel, what, nrtimes):
+        """ search through the bot logs backwards """
+        if not channel:
+            return
+        res = []
+        andre = re.compile(' and ', re.I)
+        ands = re.split(andre, what)
+        nrtimes = int(nrtimes)
+        logmap = self.getmmap(channel)
+        if not logmap:
+            return
+        def dofunc(botname, ttime, nick, userhost, txt):
+            """ add line that matches """
+            if nick != 'bot':
+                return
+            for j in ands:
+                if txt.find(j.strip()) == -1:
+                    return
+            res.append((botname, ttime, nick, userhost, txt))
+            return 1
+        self.loop(logmap, None, dofunc, withbot=True)
+        result = []
+        if res:
+            if len(res) > nrtimes:
+                result = res[len(res)-nrtimes:]
+            else:
+                result = res
+        result.reverse()
+        return result
+
     def fromtime(self, channel, ftime):
         """ returns log items from a certain time """
         if not channel:
@@ -425,8 +451,7 @@ line.strip().split(',', 4)
             if userhost == uh:
                 result.append(int(float(ttime)))
         self.loop(logmap, None, dofunc)
-        if result:
-            return result[-1]
+        return result[-1]
 
     def lastspokelist(self, channel, userhost, nrtimes):
         """ return time of last line spoken by user with userhost """
@@ -441,9 +466,8 @@ line.strip().split(',', 4)
             if userhost == uh:
                 result.append(int(float(ttime)))
         self.loop(logmap, None, dofunc)
-        if result:
-            return result[len(result)-nrtimes:]
-
+        return result[len(result)-nrtimes:]
+         
     def lastnicks(self, channel, nrtimes):
         """ return the nicks of last said lines """
         if not channel:
@@ -558,16 +582,16 @@ examples.add('log-len', 'show size of log file of the channel the command \
 was given in', 'log-len')
 aliases.data['log-size'] = 'log-len'
 
-def sayresult(bot, ievent, result):
+def sayresult(bot, ievent, result, withbot=False):
     """ reply with result """
     got = 0
     if result:
         res = []
         for i in result:
             try:
-                if i[3] == 'bot@bot':
+                if not withbot and i[3] == 'bot@bot':
                     continue
-                if 'PRIVMSG' in i[4] or 'CMND' in i[4]:
+                if not withbot and ('PRIVMSG' in i[4] or 'CMND' in i[4]):
                     what = i[4].split(':', 1)[1].strip()
                 else:
                     what = i[4].strip()
@@ -581,8 +605,7 @@ hourmin(float(i[1])), i[2], what))
         ievent.reply("nothing found")
 
 def handle_logback(bot, ievent):
-    """ log-back [<nrtimes>] <txt> ['back' <bytesback>] .. search back \
-        through channel log """
+    """ log-back [<nrtimes>] <txt> .. search back through channel log """
     if not ievent.channel:
         ievent.reply("use chan <channelname> to set channel to search in")
         return
@@ -602,12 +625,38 @@ def handle_logback(bot, ievent):
     sayresult(bot, ievent, result)
 
 cmnds.add('log-back', handle_logback, ['USER', 'WEB', 'ANON'], speed=3)
-examples.add('back', "log-back [<nrtimes>] <txt> ['back' <bytesback>] .. \
-search backwards in log file of channel", '1) log-back http 2) log-back 1 \
-http')
+examples.add('log-back', "log-back [<nrlinesback>] <txt> .. search backwards in log \
+file of channel", '1) log-back http 2) log-back 1 http')
 aliases.data['b'] = 'log-back'
 aliases.data['back'] = 'log-back'
 
+def handle_logbback(bot, ievent):
+    """ log-botback [<nrtimes>] <txt> ['back' <bytesback>] .. search back \
+        through bot output in the channel log """
+    if not ievent.channel:
+        ievent.reply("use chan <channelname> to set channel to search in")
+        return
+    if ievent.channel not in logs.loglist:
+        ievent.reply('logging not enabled in %s' % ievent.channel)
+        return
+    try:
+        nrtimes = int(ievent.args[0])
+        txt = ' '.join(ievent.args[1:])
+    except ValueError:
+        txt = ievent.rest
+        nrtimes = 100000
+    except IndexError:
+        ievent.missing('<searchitem> or <nrtimes> <searchitem>')
+        return
+    result = logs.bback(ievent.channel, txt, nrtimes)
+    sayresult(bot, ievent, result, withbot=True)
+
+cmnds.add('log-bback', handle_logbback, ['USER', 'WEB', 'ANON'], speed=3)
+examples.add('bback', "log-bback [<nrtimes>] <txt> ['back' <bytesback>] .. \
+search backwards in the bot output of the channel log file", '1) log-back \
+http 2) log-back 1 http')
+aliases.data['bback'] = 'log-bback'
+
 def handle_logsearch(bot, ievent):
     """ log-search [<nrtimes>] <txt> ['back' <bytesback>] .. search the log \
         from the beginning """
@@ -634,7 +683,6 @@ examples.add('log-search', 'log-search [<nrtimes>] <txt> .. search the log \
 from the beginning', '1) log-search http 2) log-search 10 http')
 aliasdel('search')
 
-
 def handle_loglast(bot, ievent):
     """ log-last [<nr>] [<nick>] [<txt>] .. search log for last lines of
         <nick> containing <txt> """
@@ -646,7 +694,7 @@ def handle_loglast(bot, ievent):
         nrlines = int(ievent.args[0])
         del ievent.args[0]
     except (IndexError, ValueError):
-        nrlines = 3
+        nrlines = 100000
     try:
         nick, txt = ievent.args
     except ValueError:
@@ -673,37 +721,6 @@ of channel or user', '1) log-last dunker 2) log-last 5 dunker 3) log-last \
 dunker http 4) log-last 5 dunker http')
 aliases.data['last'] = 'log-last'
 
-def handle_seen(bot, ievent):
-    """ seen <nick> .. give last spoken txt of <nick> """
-    if not ievent.channel:
-        ievent.reply('you need to set a channel, use chan channelname')
-        return
-    if ievent.channel not in logs.loglist:
-        ievent.reply('logging not enabled in %s' % ievent.channel)
-        return
-    try:
-        nick = ievent.args[0]
-    except IndexError:
-        ievent.missing('<nick>')
-        return
-    result = logs.seen(ievent.channel, nick)
-    if result:
-        tdiff = time.time() - float(result[1])
-        if result[4].startswith('PRIVMSG'):
-            what = result[4][9:]
-        elif result[4].startswith('CMND'):
-            what = result[4][6:]
-        else:
-            what = result[4]
-        ievent.reply("%s said: %s .. %s ago on %s (%s)" % \
-(result[2], what, elapsedstring(tdiff), ievent.channel, result[0]))
-    else:
-        ievent.reply("no %s in %s log" % (nick, ievent.channel))
-
-cmnds.add('seen', handle_seen, ['USER', 'WEB', 'ANON'], speed=3)
-examples.add('seen', 'seen <nick> .. show last logged line of <nick>', \
-'seen dunker')
-
 def handle_logtime(bot, ievent):
     """ show log from a certain time """
     if not logs:
@@ -733,3 +750,122 @@ def handle_logtime(bot, ievent):
     
 cmnds.add('log-time', handle_logtime, ['USER', 'WEB', 'ANON'])
 examples.add('log-time', 'show log since given time', 'log-time 21:00')
+
+def handle_active(bot, ievent):
+    """ active [<minback>] .. show who has been active, default is the 
+        last 15 min 
+    """
+    if not logs:
+        ievent.reply('log plugin is not enabled')
+        return
+    if ievent.channel not in logs.loglist:
+        ievent.reply('logging is not enabled in %s' % ievent.channel)
+        return
+    try:
+        channel = ievent.args[0].lower()
+    except IndexError:
+        channel = ievent.channel
+    minback = 15
+    result = [] 
+    for i in logs.fromtime(ievent.channel, time.time() - 15*60):
+        if i[2] not in result:
+            result.append(i[2])
+    if len(result) > 1:
+        ievent.reply("active in the last %s minutes: " % minback, result)
+    elif len(result) == 1:
+        ievent.reply("%s is active" % result[0])
+    else:
+        ievent.reply("nobody active")
+
+cmnds.add('active', handle_active, ['USER', 'WEB', 'ANON'])
+examples.add('active', 'active [<minutesback>] .. show who has been active \
+in the last 15 or <minutesback> minutes', '1) active 2) active 600')
+aliases.data['a'] = 'active'
+
+def handle_line(bot, ievent):
+    """ line .. show activity of last hour """
+    if not logs:
+        ievent.reply('log plugin is not enabled')
+        return
+    if ievent.channel not in logs.loglist:
+        ievent.reply('logging is not enabled in %s' % ievent.channel)
+        return
+    now = time.time()
+    times = [0]*61
+    for i in logs.fromtime(ievent.channel, time.time() - 60*60):
+        try:
+            diff = float(now - float(i[1]))
+        except ValueError:
+            continue
+        times[int(diff/60)] += 1
+    result = ""
+    for j in range(61):
+        if times[j]:
+            result += "%s " % times[j]
+        else:
+            result += "- "
+    result += '(number of lines per minute for the last hour .. \
+most recent minute first)'
+    ievent.reply(result) 
+
+cmnds.add('line', handle_line, ['USER', 'WEB', 'ANON'])
+examples.add('line', 'show activity for the last hour', 'line')
+aliases.data['l'] = 'line'
+
+# thnx to timp for this one
+def handle_dayline(bot, ievent):
+    """ dayline .. show activity for last 24 hours """
+    if not logs:
+        ievent.reply('log plugin is not enabled')
+        return
+    if ievent.channel not in logs.loglist:
+        ievent.reply('logging is not enabled in %s' % ievent.channel)
+        return
+    now = time.time()
+    times = [0]*25   
+    for i in logs.fromtime(ievent.channel, time.time() - 24*60*60):
+        diff = int(now/1440 - float(i[1])/1440)
+        if diff < 25:
+            times[diff] += 1
+    result = ""
+    for j in range(25):
+        if times[j]:   
+            result += "%s " % times[j]
+        else:
+            result += "- "
+    result += ' (nr lines per hour for the last day .. most recent hour first)'
+    ievent.reply(result)
+
+cmnds.add('dayline', handle_dayline, ['USER', 'WEB', 'ANON'])
+examples.add('dayline', 'show nr of lines spoken in last day', 'dl')
+aliases.data['dl'] = 'dayline'
+
+def handle_mono(bot, ievent):   
+    """ mono [<nick>] .. show length of monologue """
+    if not logs:
+        ievent.reply('log plugin is not enabled')
+        return
+    if ievent.channel not in logs.loglist:
+        ievent.reply('logging is not enabled in %s' % ievent.channel)
+        return
+    teller = 0
+    skip = 0
+    try:
+        nick = ievent.args[0].lower() 
+    except IndexError:
+        nick = ievent.nick.lower()
+    for i in logs.fromtime(ievent.channel, time.time() - 60*60):
+        if i[4].startswith('CMND:'):
+            continue
+        if i[2].lower() == nick:
+            teller += 1
+        else:
+            teller = 0
+    if teller > 4:
+        ievent.reply('%s lines of monologue' % teller)
+    else:
+        ievent.reply("%s is not making a monologue" % nick)
+
+cmnds.add('mono', handle_mono, ['USER', 'ANON'])
+examples.add('mono', 'mono [<nick>] .. show nr lines of monologue', \
+'1) mono 2) mono dunker')
