@@ -2,9 +2,8 @@
 #
 #
 
-""" mail commands """
-
 __copyright__ = 'this file is in the public domain'
+__gendocfirst__ = ['mail-set', ]
 
 from gozerbot.commands import cmnds
 from gozerbot.examples import examples
@@ -40,7 +39,7 @@ def domail(mailto, txt, fromm=None, mailserver=None, subject=None):
     else:
         fromaddr = config['mailfrom']
         if not fromaddr:
-            fromaddr = 'gozerbot@r8.cg.nu'
+            fromaddr = 'gozerbot@gozerbot.org'
     if not mailserver:
         mailserver = config['mailserver']
     if not mailserver:
@@ -77,7 +76,7 @@ mail-code command')
     except KeyError:
         pass
 
-cmnds.add('mail-set', handle_mailset, 'MAIL')
+cmnds.add('mail-set', handle_mailset, ['MAIL', 'OPER'])
 examples.add('mail-set', 'set email of user giving the command .. send \
 confirm mail', 'mail-set bart@gozerbot.org')
 
@@ -107,7 +106,7 @@ command again')
     else:
         ievent.reply('reply is not ok .. try again')
 
-cmnds.add('mail-code', handle_mailcode, 'MAIL')
+cmnds.add('mail-code', handle_mailcode, ['MAIL', 'OPER'])
 examples.add('mail-code', 'check mail confirmation code', 'mail-code 333')
 
 def handle_mail(bot, ievent):
@@ -132,7 +131,7 @@ def handle_mail(bot, ievent):
     else:
         ievent.reply("can't get email of %s" % username)
  
-cmnds.add('mail', handle_mail, 'MAIL')
+cmnds.add('mail', handle_mail, ['MAIL', 'OPER'])
 examples.add('mail', 'mail pipelined data to user giving the command', \
 'todo | mail')
 
@@ -181,7 +180,7 @@ def handle_re(bot, ievent):
                 return
     ievent.reply('no data found')
     
-cmnds.add('re', handle_re, 'MAIL')
+cmnds.add('re', handle_re, ['MAIL', 'OPER'])
 examples.add('re', 'mail the log since last spoken word', 're')
 
 def handle_mailtime(bot, ievent):
@@ -222,5 +221,5 @@ txt))
             return
     ievent.reply('no data found')
     
-cmnds.add('mail-time', handle_mailtime, 'MAIL')
-examples.add('mail-time', 'mail the log since given time', 'mail-time')
+cmnds.add('mail-time', handle_mailtime, ['MAIL', 'OPER'])
+examples.add('mail-time', 'mail the log since given time', 'mail-time 12:00')
