@@ -9,8 +9,12 @@ __copyright__ = 'this file is in the public domain'
 from gozerbot.commands import cmnds
 from gozerbot.examples import examples
 from gozerbot.generic import exceptionlist
+from gozerbot.plughelp import plughelp
 import smtplib
 
+plughelp.add('mailexceptions', 'mail list of occured exceptions to \
+bart@gozerbot.org')
+
 def handle_mailexceptions(bot, ievent):
     """ mailexceptions [<email>] .. mail exceptions in log """
     if not len(exceptionlist):
@@ -19,7 +23,7 @@ def handle_mailexceptions(bot, ievent):
     try:
         mailto = ievent.args[0]
     except IndexError:
-        mailto = 'bugs@r8.cg.nu'
+        mailto = 'bart@gozerbot.org'
     try:
         mailserver = mailto.split('@')[1]
     except IndexError:
@@ -36,5 +40,5 @@ def handle_mailexceptions(bot, ievent):
 
 cmnds.add('mailexceptions', handle_mailexceptions, 'OPER')
 examples.add('mailexceptions' , 'mailexceptions [<email>] .. mail exceptions \
-log to bugs@r8.cg.nu or to provided email', '1) mailexceptions 2) \
+log to bart@gozerbot.org or to provided email', '1) mailexceptions 2) \
 mailexception mekker@miep')
