@@ -18,6 +18,8 @@ TODO:
 __copyright__ = 'this file is in the public domain'
 __author__ =  'Bas van Oostveen'
 __coauthor__ = 'Bart Thate <bart@gozerbot.org>'
+__gendocfirst__ = ['markov-enable', ]
+__gendoclast__ = ['markov-disable', ]
 
 from gozerbot.datadir import datadir
 from gozerbot.generic import rlog, geturl, striphtml
@@ -78,9 +80,10 @@ def init():
     if not cfg.get('enable'):
         return 1
     # if so register callbacks
-    callbacks.add("PRIVMSG", cb_markovtalk, cb_markovtalk_test)
-    callbacks.add('JOIN', cb_markovjoin)
-    jcallbacks.add('Message', cb_markovtalk, cb_jmarkovtalk_test)
+    callbacks.add("PRIVMSG", cb_markovtalk, cb_markovtalk_test, threaded=True)
+    callbacks.add('JOIN', cb_markovjoin, threaded=True)
+    jcallbacks.add('Message', cb_markovtalk, cb_jmarkovtalk_test, \
+threaded=True)
     # learn log or url on startup
     start_new_thread(markovtrain, (markovlearn.data,))
     return 1
@@ -276,22 +279,8 @@ def getreply(bot, ievent, text_line):
     """ get 20 replies and choose the largest one """
     if not text_line:
         return ""
-    lines = []
-    for i in bot.backlog[-5:]:
-        if i.nick.lower() == ievent.nick.lower():
-            lines.append(strip_txt(bot, i.txt))
-    sd = Statdict()
-    for i in lines[:5]:
-        for j in i.split():
-            sd.upitem(j)
-    top = [i[0] for i in sd.top(limit=3)]
     text_line = msg_to_array(text_line)
-    goforit = ""
-    for i in text_line:
-        if i in top:
-            goforit = i
-    if not goforit:
-        goforit = random.choice(text_line)
+    goforit = random.choice(text_line)
     results = []
     for i in range(20):
         line = getline(goforit)
@@ -359,7 +348,7 @@ def handle_markovlearn(bot, ievent):
 
 cmnds.add('markov-learn', handle_markovlearn, 'OPER')
 examples.add('markov-learn', 'learn a logfile or learn an url', \
-'1) markov-learn #dunkbots 2) markov-learn http://r8.cg.nu')
+'1) markov-learn #dunkbots 2) markov-learn http://gozerbot.org')
  
 def handle_markovlearnadd(bot, ievent):
     """ add log or url to be learned at startup or reload """
@@ -379,7 +368,7 @@ def handle_markovlearnadd(bot, ievent):
 cmnds.add('markov-learnadd', handle_markovlearnadd, 'OPER')
 examples.add('markov-learnadd', 'add channel or url to permanent learning .. \
 this will learn the item on startup', '1) markov-learnadd #dunkbots 2) \
-markov-learnadd http://r8.cg.nu')
+markov-learnadd http://gozerbot.org')
 
 def handle_markovlearnlist(bot, ievent):
     """ show the learnlist """
@@ -405,7 +394,7 @@ def handle_markovlearndel(bot, ievent):
 
 cmnds.add('markov-learndel', handle_markovlearndel, 'OPER')
 examples.add('markov-learndel', 'remove item from learnlist', \
-'1) markov-learndel #dunkbots 2) markov-learndel http://r8.cg.nu')
+'1) markov-learndel #dunkbots 2) markov-learndel http://gozerbot.org')
 
 def handle_markov(bot, ievent):
     """ this is the command to make the bot reply a markov response """
