@@ -23,4 +23,4 @@ def ncap(bot, ievent):
     """ recapture the nick """
     bot.donick(bot.orignick)
 
-callbacks.add('QUIT', ncap, ncaptest)
+callbacks.add('QUIT', ncap, ncaptest, threaded=True)
