@@ -2,10 +2,9 @@
 #
 #
 
-""" used in a pipeline .. reverse grep elements """
-
 __copyright__ = 'this file is in the public domain'
 
+from gozerbot.examples import examples
 from gozerbot.commands import cmnds
 from gozerbot.generic import waitforqueue
 from gozerbot.plughelp import plughelp
@@ -54,3 +53,4 @@ def handle_not(bot, ievent):
         ievent.reply(res, dot=True)
 
 cmnds.add('not', handle_not, ['USER', 'WEB', 'ANON'])
+examples.add('not', 'reverse grep used in pipelines', 'list | not todo')
