@@ -2,7 +2,11 @@
 #
 #
 
-""" op on join or op on !op command"""
+""" for op to work for a user the user must have the channel name in his/hers
+ status .. use !user-addstatus <username> #channel
+ normally the bot doesnt op nicks that join after a split to prevent floods, 
+ this can be disabled by using ops-cfg oponsplit 1
+"""
 
 __copyright__ = 'this file is in the public domain'
 
@@ -12,9 +16,13 @@ from gozerbot.examples import examples
 from gozerbot.callbacks import callbacks
 from gozerbot.users import users
 from gozerbot.plughelp import plughelp
+from gozerbot.persistconfig import PersistConfig
+import time
 
-plughelp.add('ops', 'op on command or op on join .. for op on join the user\
- needs to have the channel name in his/hers status .. see addstatus ')
+plughelp.add('ops', 'op with command or op on join')
+
+cfg = PersistConfig()
+cfg.define('oponsplit', 0)
 
 def opjoincb(bot, ievent):
     """ see if we should op a user on join """
@@ -23,22 +31,23 @@ def opjoincb(bot, ievent):
         return
     if ievent.channel in bot.state['no-op']:
         return
-    if ievent.nick.lower() not in bot.splitted:
+    time.sleep(1)
+    if cfg.get('oponsplit') or ievent.nick.lower() not in bot.splitted:
         if users.status(ievent.userhost, ievent.channel.upper()):
-            if ievent.channel not in bot.opchan:
+            if ievent.channel not in bot.state['opchan']:
                 return
             bot.doop(ievent.channel, ievent.nick)
     else:
         bot.splitted.remove(ievent.nick.lower())
 
-callbacks.add('JOIN', opjoincb)
+callbacks.add('JOIN', opjoincb, threaded=True)
 
 def handle_op1(bot, ievent):
     """ op [<nick>] .. op an user """
     if ievent.channel in bot.state['no-op']:
         ievent.reply('opping is disabled in %s' % ievent.channel)
         return
-    if ievent.channel not in bot.opchan:
+    if ievent.channel not in bot.state['opchan']:
         ievent.reply("i'm not op in %s" % ievent.channel)
         return
     try:
@@ -84,7 +93,7 @@ def handle_opsdisable(bot, ievent):
         bot.state['no-op'].append(chan)
         bot.state.save()
         ievent.reply('opping in %s disabled' % chan)
-        if chan in bot.opchan:
+        if chan in bot.state['opchan']:
             bot.delop(ievent.channel, bot.nick)
     else:
         ievent.reply('opping %s is already disabled' % chan)
@@ -118,6 +127,8 @@ def handle_opsnoop(bot, ievent):
     ievent.reply('opping is disabled in %s' % bot.state['no-op'])
     
 cmnds.add('ops-list', handle_opsnoop, 'OPER')
+examples.add('ops-list', 'list in which channels opping is disabled', \
+'ops-list')
 
 def checkmode(bot, ievent):
     """ check mode string """
@@ -144,12 +155,12 @@ def checkmode(bot, ievent):
                     bot.delop(ievent.channel, bot.nick)
                 else:
                     rlog(1, 'irc', 'opped on %s' % chan)
-                    bot.opchan.append(chan)
+                    bot.state['opchan'].append(chan)
         if i == 'o' and not plus:
             if who[teller].lower() == bot.nick.lower():
                 rlog(1, 'irc', 'deopped on %s' % chan)
                 try:
-                    bot.opchan.remove(chan)
+                    bot.state['opchan'].remove(chan)
                 except ValueError:
                     pass
         teller += 1
