@@ -1,22 +1,20 @@
-# plugs/rest.py
+# gozerplugs/plugs/rest.py
+#
 #
-# Wijnand 'tehmaze' Modderman - http://tehmaze.com
-# BSD License
-
-""" shows the rest of the output in /msg"""
 
 __author__ = "Wijnand 'tehmaze' Modderman - http://tehmaze.com"
 __license__ = 'BSD'
 
-import time
 from gozerbot.commands import cmnds
 from gozerbot.persistconfig import PersistConfig
 from gozerbot.plughelp import plughelp
+from gozerbot.examples import examples
+import time
 
 plughelp.add('rest', 'show rest of the output in /msg')
 
 cfg = PersistConfig()
-cfg.define('delay', 2.0)
+cfg.define('delay', 3.0)
 cfg.define('limit', 10)
 
 def handle_rest(bot, ievent):
@@ -68,4 +66,4 @@ def handle_rest(bot, ievent):
             bot.output(ievent.nick, "%s more line%s" % (size, s))
 
 cmnds.add('rest', handle_rest, 'USER')
-
+examples.add('rest', 'show the rest of output cache data', 'rest')
