@@ -2,19 +2,20 @@
 #
 #
 
-""" size command """
-
 __copyright__ = 'this file is in the public domain'
 
 from gozerbot.commands import cmnds
 from gozerbot.examples import examples
-from gozerbot.config import config
 from gozerbot.users import users
 from gozerbot.redispatcher import rebefore, reafter
 from gozerbot.aliases import aliases
 from gozerbot.callbacks import callbacks
 from gozerbot.plugins import plugins
 from gozerbot.fleet import fleet
+from gozerbot.plughelp import plughelp
+
+plughelp.add('size', 'the size command shows the sizes of plugins that \
+provide a size() plugin command and the sizes of some basic structures')
 
 def handle_size(bot, ievent):
     """ size .. show size of core datastructures """
