@@ -1,12 +1,10 @@
-# plugs/snarf.py
+# gozerplugs/plugs/snarf.py
 # 
-# (c) Wijnand 'tehmaze' Modderman - http://tehmaze.com
-# BSD License
-
-""" url snarfer """
+#
 
 __author__ = "Wijnand 'tehmaze' Modderman - http://tehmaze.com"
 __license__ = 'BSD'
+__gendoclast__ = ['snarf-disable', ]
 
 from gozerbot.callbacks import callbacks, jcallbacks
 from gozerbot.plughelp import plughelp
@@ -29,7 +27,7 @@ the provided url or the last url in the log')
 cfg           = Persist('snarf', {})
 pcfg          = PersistConfig()
 pcfg.define('allow', ['text/plain', 'text/html', 'application/xml'])
-re_html_title = re.compile(u'<title>(.*)</title>', re.I | re.M)
+re_html_title = re.compile(u'<title>(.*)</title>', re.I | re.M | re.S)
 re_url_match  = re.compile(u'((?:http|https)://\S+)')
 re_html_valid = {
     'result':   re.compile('(Failed validation, \
@@ -73,7 +71,7 @@ def geturl_title(url):
     if test_title:
         # try to find an encoding and standardize it to utf-8
         encoding = get_encoding(result)
-        title = test_title.group(1).decode(encoding, 'replace')
+        title = test_title.group(1).decode(encoding, 'replace').replace('\n', ' ')
         return decode_html_entities(title)
     return False
 
@@ -134,6 +132,9 @@ urlcache[bot.name].has_key(ievent.printto)):
         url = ievent.rest
     try:
         url = valid_url(url)
+    except KeyError:
+        ievent.reply("can't detect content type")
+        return
     except SnarfException, e:
         if direct:
             ievent.reply('unable to snarf: %s' % str(e))
@@ -141,7 +142,14 @@ urlcache[bot.name].has_key(ievent.printto)):
     except urllib2.HTTPError, e:
         ievent.reply('unable to snarf: %s' % str(e))
         return
-    title = geturl_title(url)
+    if not url:
+        ievent.reply('invalid url')
+        return
+    try:
+        title = geturl_title(url)
+    except urllib2.HTTPError, e:
+        ievent.reply('error: %s' % e)
+        return
     if title:
         host = urlparse.urlparse(url)[1]
         if len(host) > 20:
@@ -165,6 +173,7 @@ def handle_snarf_enable(bot, ievent):
     ievent.reply('ok')
 
 cmnds.add('snarf-enable', handle_snarf_enable, 'OPER')
+examples.add('snarf-enable', 'enable snarfing in the channel', 'snarf-enable')
 aliases.data['snarf-on'] = 'snarf-enable'
 
 def handle_snarf_disable(bot, ievent):
@@ -177,6 +186,8 @@ def handle_snarf_disable(bot, ievent):
     ievent.reply('ok')
 
 cmnds.add('snarf-disable', handle_snarf_disable, 'OPER')
+examples.add('snarf-disable', 'disable snarfing in the channel', \
+'snarf-disable')
 aliases.data['snarf-off'] = 'snarf-disable'
 
 def handle_snarf_list(bot, ievent):
@@ -194,6 +205,8 @@ def handle_snarf_list(bot, ievent):
         ievent.reply('snarfers enable on: %s' % ', '.join(snarfs))
 
 cmnds.add('snarf-list', handle_snarf_list, 'OPER')
+examples.add('snarf-list', 'show in which channels snarfing is enabled', \
+'snarf-list')
 
 def handle_validate(bot, ievent):
     """ validate provided url or last url in log """
@@ -205,7 +218,11 @@ urlcache[bot.name].has_key(ievent.printto):
         url = urlcache[bot.name][ievent.printto]
     else:
         url = ievent.rest
-    url = valid_url(url)
+    try:
+        url = valid_url(url)
+    except urllib2.HTTPError, e:
+        ievent.reply('error: %s' % e)
+        return
     if not url:
         ievent.reply('invalid or bad URL')
         return
@@ -231,7 +248,8 @@ def privmsgcb(bot, ievent):
         if not urlcache.has_key(bot.name):
             urlcache[bot.name] = {}
         urlcache[bot.name][ievent.printto] = url
-        rlog(10, bot.name, 'cached url %s on %s' % (url, ievent.printto))
+        rlog(0, 'snarf', 'cached url %s on %s (%s)' % (url, ievent.printto, \
+bot.name))
         if cfg.data.has_key(bot.name) and cfg.data[bot.name]\
 .has_key(ievent.printto) and cfg.data[bot.name][ievent.printto]:
             nevent = copy.copy(ievent)
