@@ -2,8 +2,6 @@
 #
 #
 
-""" do a timing of a command """
-
 __copyright__ = 'this file is in the public domain'
 
 from gozerbot.commands import cmnds
@@ -27,7 +25,7 @@ def handle_timer(bot, ievent):
         ievent.reply('no result for %s' % ievent.rest)
         return
     result.insert(0, "%s seconds ==>" % str(stoptime-starttime))
-    ievent.reply(result)
+    ievent.reply('timer results: ', result)
 
 cmnds.add('timer', handle_timer, ['USER', 'WEB'], allowqueue=False)
-examples.add('timer', 'time a command', 'timer b http')
+examples.add('timer', 'time a command', 'timer list')
