@@ -2,14 +2,17 @@
 #
 #
 
-""" used in a pipeline .. direct output to <nick> """
-
 __copyright__ = 'this file is in the public domain'
 
 from gozerbot.commands import cmnds
 from gozerbot.generic import getwho, waitforqueue
+from gozerbot.examples import examples
+from gozerbot.plughelp import plughelp
+
+plughelp.add('to', 'send the output to another user .. used in a pipeline')
 
 def handle_to(bot, ievent):
+    """ direct pipeline output to <nick> """
     if not ievent.inqueue:
         ievent.reply('use to in a pipeline')
         return
@@ -35,3 +38,4 @@ def handle_to(bot, ievent):
         ievent.reply('nothing to send')
 
 cmnds.add('to', handle_to, 'USER')
+examples.add('to', 'send pipeline output to another user', 'list | to dunker')
