@@ -2,7 +2,9 @@
 #
 #
 
-""" provide todo related commands """
+""" manage todo lists .. by user or by channel .. a time/data string can 
+ be provided to set time on a todo item.
+"""
 
 __copyright__ = 'this file is in the public domain'
 
@@ -14,10 +16,11 @@ from gozerbot.datadir import datadir
 from gozerbot.persist import Persist
 from gozerbot.plughelp import plughelp
 from gozerbot.aliases import aliases
+from gozerbot.config import config
 from gozerplugs.plugs.alarm import alarms
 import time, thread, os
 
-plughelp.add('todo', 'manage todo lists .. by user or by channel')
+plughelp.add('todo', 'todo lists')
 
 todolock = thread.allocate_lock()
 locked = lockdec(todolock)
@@ -27,14 +30,14 @@ class Todoitem:
     """ a todo item """
 
     def __init__(self, name, descr, ttime=None, duration=None, warnsec=None, \
-priority=None):
+priority=None, num=0):
         self.name = name
         self.time = ttime
         self.duration = duration
         self.warnsec = warnsec
         self.descr = descr
         self.priority = priority
-        self.num = 0
+        self.num = num
 
     def __str__(self):
         return "name: %s num: %d time: %s duration: %s warnsec: %s \
@@ -92,6 +95,7 @@ class Todo(Persist):
             self.data[key] = newd
 
     def size(self):
+        """ return number of todo entries """
         return len(self.data)
 
     def get(self, name):
@@ -108,7 +112,7 @@ class Todo(Persist):
         self.data[name].append(Todoitem(name, txt.strip(), ttime, \
 warnsec=0-warnsec))
         self.save()
-        return 1
+        return len(self.data[name])
 
     @locked
     def addnosave(self, name, txt, ttime):
@@ -118,27 +122,30 @@ warnsec=0-warnsec))
             self.data[name] = Todolist()
         self.data[name].append(Todoitem(name, txt, ttime))
 
+
     @locked
-    def delete(self, name, nrlist):
+    def reset(self, name):
+        name = name.lower()
+        if self.data.has_key(name):
+           self.data[name] = Todolist()
+        self.save()
+
+    @locked
+    def delete(self, name, nr):
         """ delete todo item """
         if not self.data.has_key(name):
             return 0
-        nrlist.sort()
-        nrlist.reverse()
         todoos = self.data[name]
-        teller = 0
-        for i in nrlist:
-            try:
-                if todoos[i].warnsec:
-                    alarmnr = 0 - todoos[i].warnsec
-                    if alarmnr > 0:
-                        alarms.delete(alarmnr)
-                del todoos[i]
-                teller += 1
-            except KeyError:
-                pass
+        try:
+            if todoos[nr].warnsec:
+                alarmnr = 0 - todoos[nr].warnsec
+                if alarmnr > 0:
+                    alarms.delete(alarmnr)
+            del todoos[nr]
+        except KeyError:
+            return 0
         self.save()
-        return teller
+        return 1
 
     def toolate(self, name):
         """ show if there are any time related todoos that are too late """
@@ -171,6 +178,7 @@ warnsec=0-warnsec))
 
     @locked
     def setprio(self, who, itemnr, prio):
+        """ set priority of todo item """
         try:
             todoitems = self.get(who)
             todoitems[itemnr].priority = prio
@@ -180,6 +188,7 @@ warnsec=0-warnsec))
             pass
 
     def settime(self, who, itemnr, ttime):
+        """ set time of todo item """
         try:
             todoitems = self.get(who)
             todoitems[itemnr].time = ttime
@@ -188,22 +197,162 @@ warnsec=0-warnsec))
         except (KeyError, TypeError):
             pass
 
-todo = Todo(datadir + os.sep + 'todo')
+class TodoDb(object):
 
-if not todo.data:
-    todo.data = {}
+    """ database todo interface """
+
+    def reset(self, name): 
+        result = db.execute(""" DELETE FROM todo WHERE name == %s """, name)
+        return result
+
+    def size(self):
+        """ return number of todo's """
+        result = db.execute(""" SELECT COUNT(*) FROM todo """)
+        return result[0][0]
+        
+    def get(self, name):
+        """ get todo list of <name> """
+        name = name.lower()
+        result = db.execute(""" SELECT * FROM todo WHERE name = %s ORDER BY \
+priority DESC, indx ASC """, name)
+        res = []
+        if result:
+            for i in result:
+                args = [i[1],i[5], i[2], i[3], i[4], i[6], i[0]]
+                res.append(Todoitem(*args))
+        return res
+
+    def getid(self, idnr):
+        """ get todo data of <idnr> """
+        result = db.execute(""" SELECT * FROM todo WHERE indx = %s """, idnr)
+        res = []
+        if result:
+            for i in result:
+                args = [i[1],i[5], i[2], i[3], i[4], i[6], i[0]]
+                res.append(Todoitem(*args))
+        return res
+        return result
+
+    def setprio(self, who, todonr, prio):
+        """ set priority of todonr """
+        result = db.execute(""" UPDATE todo SET priority = %s WHERE indx \
+= %s """, (prio, todonr))
+        return result
+
+    def getprio(self, todonr):
+        """ get priority of todonr """
+        result = db.execute(""" SELECT name, priority FROM todo WHERE \
+indx = %s """, todonr)
+        return result
+
+    def getwarnsec(self, todonr):
+        """ get priority of todonr """
+        result = db.execute(""" SELECT warnsec FROM todo WHERE \
+indx = %s """, todonr)
+        return result
+
+    def settime(self, who, todonr, ttime):
+        """ set time of todonr """
+        result = db.execute(""" UPDATE todo SET time = %s WHERE indx \
+= %s """, (ttime, todonr))
+        return result
+
+    def add(self, name, txt, ttime, alarmnr=None):
+        """ add a todo """
+        name = name.lower()
+        txt = txt.strip()
+        if ttime:
+            if not alarmnr:
+                result = db.execute(""" INSERT INTO todo(name, time, \
+descr) VALUES (%s, %s, %s) """, (name, ttime, txt))
+            else:
+                result = db.execute(""" INSERT INTO todo(name, time, \
+descr, warnsec) VALUES (%s, %s, %s, %s) """, (name, ttime, txt, 0-alarmnr))
+
+        else:
+            result = db.execute(""" INSERT INTO todo(name, descr) \
+VALUES (%s, %s) """, (name, txt))
+        return result
+
+    def delete(self, name, indexnr):
+        """ delete todo item """
+        name = name.lower()
+        try:
+            warnsec = self.getwarnsec(indexnr)[0][0]
+            if warnsec:
+                alarmnr = 0 - warnsec
+                if alarmnr > 0:
+                    alarms.delete(alarmnr)
+        except (IndexError, TypeError):
+            pass
+        result = db.execute(""" DELETE FROM todo WHERE name = %s AND \
+indx = %s """, (name, indexnr))
+        return result
+
+    def toolate(self, name):
+        """ show if there are any time related todoos that are too late """
+        name = name.lower()
+        now = time.time()
+        result = db.execute(""" SELECT * FROM todo WHERE name = %s AND \
+time < %s """, (name, now))
+        res = []
+        if result:
+            for i in result:
+                args = [i[1],i[5], i[2], i[3], i[4], i[6], i[0]]
+                res.append(Todoitem(*args))
+        return res
+
+    def withintime(self, name, time1, time2):
+        """ show todo list within time frame """
+        name = name.lower()
+        result = db.execute(""" SELECT * FROM todo WHERE name = %s AND \
+time > %s AND time < %s """, (name, time1, time2))
+        res = []
+        if result:
+            for i in result:
+                args = [i[1],i[5], i[2], i[3], i[4], i[6], i[0]]
+                res.append(Todoitem(*args))
+        return res
+
+    def timetodo(self, name):
+        name = name.lower()
+        now = time.time()
+        result = db.execute(""" SELECT * FROM todo WHERE time AND name = %s \
+""", name)
+        res = []
+        if result:
+            for i in result:
+                args = [i[1],i[5], i[2], i[3], i[4], i[6], i[0]]
+                res.append(Todoitem(*args))
+        return res
+
+    def reset(self, name):
+        """ reset todo items of user with <name> """
+        name = name.lower()
+        result = db.execute(""" DELETE FROM todo WHERE name = %s """, (name, ))
+        return result
+
+if config['dbenable']:
+    from gozerbot.db import db
+    todo = TodoDb()
+else:
+    todo = Todo(datadir + os.sep + 'todo')
+    if not todo.data:
+        todo.data = {}
 
 def size():
+    """ return number of todo entries """
     return todo.size()
 
 def handle_todo(bot, ievent):
-    """ todo [<item>] .. show todoos or set todo item """
+    """ todo [<item>] .. show todo's or set todo item .. a time/date can be \
+given"""
     if len(ievent.args) > 0:
         handle_todo2(bot, ievent)
         return
-    user = users.getuser(ievent.userhost)
+    name = users.getname(ievent.userhost)
     try:
-        todoos = todo.get(user.name)
+        todoos = todo.get(name)
     except KeyError:
         ievent.reply('i dont have todo info for %s' % user.name)
         return
@@ -216,16 +365,17 @@ def handle_todo2(bot, ievent):
         return
     else:
         what = ievent.rest
-    user = users.getuser(ievent.userhost)
+    name = users.getname(ievent.userhost)
     ttime = strtotime(what)
+    nr = 0
     if not ttime  == None:
         ievent.reply('time detected ' + time.ctime(ttime))
         what = striptime(what)
         alarmnr = alarms.add(bot.name, ievent.nick, ttime, what)
-        todo.add(user.name, what, ttime, alarmnr)
+        nr = todo.add(name, what, ttime, alarmnr)
     else:
-        todo.add(user.name, what, None)
-    ievent.reply('todo item added')
+        nr = todo.add(name, what, None)
+    ievent.reply('todo item %s added' % nr)
 
 cmnds.add('todo', handle_todo, 'USER')
 examples.add('todo', 'todo [<item>] .. show todo items or add a todo item', \
@@ -243,8 +393,10 @@ def handle_tododone(bot, ievent):
     except ValueError:
         ievent.reply('%s is not an integer' % i)
         return
-    user = users.getuser(ievent.userhost)
-    nrdone = todo.delete(user.name, nrs)
+    name = users.getname(ievent.userhost)
+    nrdone = 0
+    for i in nrs:
+        nrdone += todo.delete(name, i)
     if nrdone == 1:
         ievent.reply('%s item deleted' % nrdone)
     elif nrdone == 0:
@@ -258,30 +410,28 @@ todo list', '1) todo-done 1 2) todo-done 3 5 8')
 aliases.data['done'] = 'todo-done'
 
 def handle_chantodo(bot, ievent):
-    """ todo-chan [<item>] .. show todoos or set todo item for channel"""
+    """ todo-chan [<item>] .. show channel todo's or set todo item for \
+channel"""
     if ievent.rest:
         handle_chantodo2(bot, ievent)
         return
     todoos = todo.get(ievent.channel)
-    if not todoos:
-        ievent.reply('i dont have todo info for %s' % \
-ievent.channel)
-        return
     saytodo(bot, ievent, todoos)
 
 def handle_chantodo2(bot, ievent):
     """ set todo item for channel"""
     what = ievent.rest
     ttime = strtotime(what)
+    nr = 0
     if not ttime  == None:
         ievent.reply('time detected ' + time.ctime(ttime))
         result = '(%s) ' % ievent.nick + striptime(what)
         alarmnr = alarms.add(bot.name, ievent.channel, ttime, result)
-        todo.add(ievent.channel, result, ttime, alarmnr)
+        nr = todo.add(ievent.channel, result, ttime, alarmnr)
     else:
         result = '(%s) ' % ievent.nick + what
-        todo.add(ievent.channel, result, None)
-    ievent.reply('todo item added')
+        nr = todo.add(ievent.channel, result, None)
+    ievent.reply('todo item %s added' % nr)
 
 cmnds.add('todo-chan', handle_chantodo, 'USER')
 examples.add('todo-chan', 'todo-chan [<item>] .. add channel todo', \
@@ -301,7 +451,9 @@ def handle_todochandone(bot, ievent):
     except ValueError:
         ievent.reply('%s is not an integer' % i)
         return
-    nrdone = todo.delete(ievent.channel, nrs)
+    nrdone = 0
+    for i in nrs:
+        nrdone += todo.delete(ievent.channel, i)
     if nrdone == 1:
         ievent.reply('%s item deleted' % nrdone)
     elif nrdone == 0:
@@ -315,7 +467,7 @@ from channel todo list', 'todo-chandone 2')
 aliases.data['chandone'] = 'todo-chandone'
 
 def handle_settodo(bot, ievent):
-    """ todo-set <name> <txt> .. add to todo of another user """
+    """ todo-set <name> <txt> .. add a todo to another user's todo list"""
     try:
         who = ievent.args[0]
         what = ' '.join(ievent.args[1:])
@@ -329,32 +481,33 @@ def handle_settodo(bot, ievent):
     if not userhost:
         ievent.reply("can't find userhost for %s" % who)
         return
-    whouser = users.getuser(userhost)
+    whouser = users.getname(userhost)
     if not whouser:
         ievent.reply("can't find user for %s" % userhost)
         return
-    user = users.getuser(ievent.userhost)
-    if not users.permitted(userhost, user.name, 'todo'):
+    name = users.getname(ievent.userhost)
+    if not users.permitted(userhost, name, 'todo'):
         ievent.reply("%s doesn't permit todo sharing for %s " % \
-(who,user.name))
+(who, name))
         return
     what = "%s: %s" % (ievent.nick, what)
     ttime = strtotime(what)
+    nr = 0
     if not ttime  == None:
         ievent.reply('time detected ' + time.ctime(ttime))
         what = striptime(what)
         alarmnr = alarms.add(bot.name, who, ttime, what)
-        todo.add(whouser.name, what, ttime, alarmnr)
+        nr = todo.add(whouser, what, ttime, alarmnr)
     else:
-        todo.add(whouser.name, what, None)
-    ievent.reply('todo item added')
+        nr = todo.add(whouser, what, None)
+    ievent.reply('todo item %s added' % nr)
 
 cmnds.add('todo-set', handle_settodo, 'USER')
-examples.add('todo-set', 'todo-set <nick> <txt> .. >set todo item of \
+examples.add('todo-set', 'todo-set <nick> <txt> .. set todo item of \
 <nick>', 'todo-set dunker bot proggen')
 
 def handle_gettodo(bot, ievent):
-    """ todo-get <nick> .. get todo of other user """
+    """ todo-get <nick> .. get todo of another user """
     try:
         who = ievent.args[0]
     except IndexError:
@@ -364,16 +517,15 @@ def handle_gettodo(bot, ievent):
     if not userhost:
         ievent.reply("can't find userhost for %s" % who)
         return
-    whouser = users.getuser(userhost)
+    whouser = users.getname(userhost)
     if not whouser:
         ievent.reply("can't find user for %s" % userhost)
         return
-    user = users.getuser(ievent.userhost)
-    if not users.permitted(userhost, user.name, 'todo'):
-        ievent.reply("%s doesn't permit todo sharing for %s " % \
-(who,user.name))
+    name = users.getname(ievent.userhost)
+    if not users.permitted(userhost, name, 'todo'):
+        ievent.reply("%s doesn't permit todo sharing for %s " % (who, name))
         return
-    todoos = todo.get(whouser.name)
+    todoos = todo.get(whouser)
     saytodo(bot, ievent, todoos)
 
 cmnds.add('todo-get', handle_gettodo, ['USER', 'WEB'])
@@ -382,31 +534,32 @@ examples.add('todo-get', 'todo-get <nick> .. get the todo list of \
 
 def handle_todotime(bot, ievent):
     """ todo-time .. show time related todoos """
-    user = users.getuser(ievent.userhost)
-    todoos = todo.timetodo(user.name)
+    name = users.getname(ievent.userhost)
+    todoos = todo.timetodo(name)
     saytodo(bot, ievent, todoos)
 
 cmnds.add('todo-time', handle_todotime, 'USER')
-examples.add('todo-time', 'show todo items with time fields', 'todo-time')
+examples.add('todo-time', 'todo-time .. show todo items with time fields', \
+'todo-time')
 aliases.data['tt'] = 'todo-time'
 
 def handle_todoweek(bot, ievent):
     """ todo-week .. show time related todo items for this week """
-    user = users.getuser(ievent.userhost)
-    todoos = todo.withintime(user.name, today(), today()+7*24*60*60)
+    name = users.getname(ievent.userhost)
+    todoos = todo.withintime(name, today(), today()+7*24*60*60)
     saytodo(bot, ievent, todoos)
 
 cmnds.add('todo-week', handle_todoweek, 'USER')
-examples.add('todo-week', 'todo items for this week', 'todo-week')
+examples.add('todo-week', 'todo-week .. todo items for this week', 'todo-week')
 
 def handle_today(bot, ievent):
     """ todo-today .. show time related todo items for today """
-    user = users.getuser(ievent.userhost)
-    todoos = todo.withintime(user.name, today(), today()+24*60*60)
+    name = users.getname(ievent.userhost)
+    todoos = todo.withintime(name, today(), today()+24*60*60)
     saytodo(bot, ievent, todoos)
 
 cmnds.add('todo-today', handle_today, 'USER')
-examples.add('todo-today', 'todo items for today', 'todo-today')
+examples.add('todo-today', 'todo-today .. todo items for today', 'todo-today')
 aliases.data['today'] = 'todo-today'
 
 def handle_tomorrow(bot, ievent):
@@ -417,7 +570,7 @@ def handle_tomorrow(bot, ievent):
         ttime = strtotime(what)
         if ttime != None:
             if ttime < today() or ttime > today() + 24*60*60:
-                ievent.reply("can't set time on %s" % \
+                ievent.reply("%s is not tomorrow" % \
 time.ctime(ttime + 24*60*60))
                 return
             ttime += 24*60*60
@@ -432,7 +585,8 @@ time.ctime(ttime + 24*60*60))
     saytodo(bot, ievent, todoos)
 
 cmnds.add('todo-tomorrow', handle_tomorrow, 'USER')
-examples.add('todo-tomorrow', 'todo items for tomorrow', 'todo-tomorrow')
+examples.add('todo-tomorrow', 'todo-tomorrow .. todo items for tomorrow', \
+'todo-tomorrow')
 aliases.data['tomorrow'] = 'todo-tomorrow'
 
 def handle_setpriority(bot, ievent):
@@ -443,8 +597,7 @@ def handle_setpriority(bot, ievent):
     except ValueError:
         try:
             (itemnr, prio) = ievent.args
-            user = users.getuser(ievent.userhost)
-            who = user.name
+            who = users.getname(ievent.userhost)
         except ValueError:
             ievent.missing('[<channe|namel>] <itemnr> <priority>')
             return
@@ -479,8 +632,7 @@ def handle_todosettime(bot, ievent):
     except ValueError:
         try:
             (itemnr, ) = txt.split()
-            user = users.getuser(ievent.userhost)
-            who = user.name
+            who = users.getname(ievent.userhost)
         except ValueError:
             ievent.missing('[<channe|namel>] <itemnr> <timestring>')
             return
@@ -508,8 +660,7 @@ def handle_getpriority(bot, ievent):
     except ValueError:
         try:
             itemnr = ievent.args[0]
-            user = users.getuser(ievent.userhost)
-            who = user.name
+            who = users.getname(ievent.userhost)
         except IndexError:
             ievent.missing('[<channel|name>] <itemnr>')
             return
