@@ -2,20 +2,17 @@
 #
 #
 
-""" upgrade command .. determine on svn/mercurial update's output if plugs can
-    be reloaded or that the bot has to be rebooted
-"""
-
 __copyright__ = 'this file is in the public domain'
 
 from gozerbot.fleet import fleet
 from gozerbot.plugins import plugins
 from gozerbot.commands import cmnds
 from gozerbot.examples import examples
-from gozerbot.generic import reboot, gozerpopen, lockdec, handle_exception
+from gozerbot.generic import reboot_stateful, gozerpopen, lockdec, \
+handle_exception, gethgrev, getsvnrev
 from gozerbot.eventhandler import mainhandler
 from gozerbot.plughelp import plughelp
-from gozerbot.config import config, gethgrev
+from gozerbot.config import config
 from gozerbot.lockmanager import lockmanager
 from gozerbot.partyline import partyline
 import os, os.path, time
@@ -27,7 +24,12 @@ because we can reload them")
 def handle_hgupgrade(bot, ievent, silent=None):
     """ upgrade .. do a mercurial pull -u .. see if we need to reboot \
         otherwise reload plugins """
-    startrev = int(gethgrev())
+    
+    startrev = gethgrev()
+    if not startrev:
+        ievent.reply("can't fetch current revision")
+        return
+    ievent.reply('upgrading from revision %s' % str(startrev))
     url = None
     if ievent.rest:
         if not ievent.rest.startswith('http://'):
@@ -62,7 +64,7 @@ def handle_hgupgrade(bot, ievent, silent=None):
     else:
         not silent and ievent.reply(' .. '.join(res))
     rev = gethgrev()
-    ievent.reply('revision is %s' % rev)
+    ievent.reply("new revision is %s" % rev)
     args = ['hg', 'diff', '-r%s' % startrev]
     try:
         proces = gozerpopen(args)
@@ -123,7 +125,7 @@ def handle_hgupgrade(bot, ievent, silent=None):
             fleet.save()
         finally:
             time.sleep(1)
-            mainhandler.put(0, reboot)
+            mainhandler.put(0, reboot_stateful, bot, ievent, fleet, partyline)
             return
     config.load()
     if not plugs and not dbplugs:
@@ -193,7 +195,7 @@ def handle_svnupgrade(bot, ievent, silent=None):
             fleet.save()
         finally:
             time.sleep(1)
-            mainhandler.put(0, reboot)
+            mainhandler.put(0, reboot_stateful, bot, ievent, fleet, partyline)
     if not plugs and not dbplugs:
         ievent.reply('nothing to reload')
         return
