@@ -5,6 +5,7 @@
 """ user related commands """
 
 __copyright__ = 'this file is in the public domain'
+__gendoclast__ = ['user-del', ]
 
 from gozerbot.generic import getwho, stripident
 from gozerbot.commands import cmnds
@@ -15,6 +16,14 @@ from gozerbot.plughelp import plughelp
 
 plughelp.add('user', 'manage users')
 
+def handle_whoami(bot, ievent):
+    """ user-whoami .. get your username """
+    ievent.reply('%s' % users.getname(ievent.userhost))
+
+cmnds.add('user-whoami', handle_whoami, 'USER')
+examples.add('user-whoami', 'get your username', 'user-whoami')
+aliases.data['whoami'] = 'user-whoami'
+
 def handle_getuserperm(bot, ievent):
     """ user-allperm <perm> .. list all users with permission <perm> """
     try:
@@ -109,8 +118,8 @@ def handle_merge(bot, ievent):
         return  
     name, nick = ievent.args
     name = name.lower()
-    if name == 'owner' and ievent.userhost != bot.owner:
-         ievent.reply('only owneruserhost can merge with owner user')
+    if name == 'owner' and not bot.ownercheck(ievent, 'can merge with owner \
+user'):
          return 
     if users.gotperm(name, 'OPER') and not users.allowed(ievent.userhost, \
 'OPER'):
@@ -434,8 +443,8 @@ def handle_adduserhost(bot, ievent):
         return
     name, userhost = ievent.args
     name = name.lower()
-    if name == 'owner' and ievent.stripped != bot.owner:
-        ievent.reply('only owneruserhost can add userhost to owner')
+    if name == 'owner' and not bot.ownercheck(ievent, 'can add userhost to \
+owner'):
         return
     userhost = stripident(userhost)
     user = users.byname(name)
@@ -465,8 +474,8 @@ def handle_deluserhost(bot, ievent):
         return
     name, userhost = ievent.args
     name = name.lower()
-    if name == 'owner' and ievent.stripped != bot.owner:
-        ievent.reply('only owneruserhost can delete userhost from owner')
+    if name == 'owner' and not bot.ownercheck(ievent, 'can delete userhost \
+from owner'):
         return
     userhost = stripident(userhost)
     user = users.byname(name)
@@ -635,7 +644,7 @@ def handle_addpermit(bot, ievent):
     try:
         who, what = ievent.args
     except ValueError:
-        ievent.missing("<name> <what>")
+        ievent.missing("<name> <permit>")
         return
     whouser = users.byname(who)
     if not whouser:
@@ -661,9 +670,9 @@ aliasdel('allow')
 def handle_permit(bot, ievent):
     """ user-permit .. get permit list of user giving the command """
     if ievent.rest:
-        ievent.reply("use the allow command to allow somebody something .. \
-use getname <nick> to get the username of somebody .. this command shows \
-what permits you have")
+        ievent.reply("use the user-addpermit command to allow somebody \
+something .. use getname <nick> to get the username of somebody .. \
+this command shows what permits you have")
         return
     user = users.getuser(ievent.userhost)
     if user.permit:
