@@ -7,6 +7,8 @@
 __copyright__ = 'this file is in the public domain'
 __credits__ = 'thanks to John Munn (jrmunn@home.com) for his form focus \
 javascript'
+__gendocfirst__ = ['web-enable', ]
+__gendoclast__ = ['web-disable', ]
 
 from gozerbot.thr import start_new_thread
 from gozerbot.generic import rlog, handle_exception
@@ -23,7 +25,7 @@ from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
 from SocketServer import ThreadingMixIn
 from urllib import unquote_plus
 from cgi import escape
-import time, sys, select
+import time, sys, select, types
 
 plughelp.add('webserver', 'maintain the bots webserver')
 
@@ -90,7 +92,7 @@ class BotHTTPServer(ThreadingMixIn, HTTPServer):
     def __init__(self, addr, handler):
         self.stop = False
         self.addr = addr
-        self.dontshow = ['nodes', 'ping', 'dispatch']
+        self.dontshow = ['nodes', 'ping', 'dispatch', 'join']
         self.handlers = {}
         self.webmods = {}
         self.whitelist = []
@@ -117,7 +119,12 @@ class BotHTTPServer(ThreadingMixIn, HTTPServer):
         txt = request.path[1:].split('?')[0]
         if self.handlers.has_key(txt):
             func = self.handlers[txt]
-            return func(request)
+            try:
+                result = func(request)
+            except Exception, ex:
+                handle_exception()
+                result = None
+            return result
         else:
             return None
 
@@ -168,6 +175,20 @@ endtxt = """</body>
 </html>
 """
 
+def writeheaderplain(request):
+    request.send_response(200)
+    request.send_header('Content-type', 'text/plain; charset=%s' % \
+sys.getdefaultencoding())
+    request.send_header('Server', config['version'])
+    request.end_headers()
+
+def writeheaderhtml(request):
+    request.send_response(200)
+    request.send_header('Content-type', 'text/html; charset=%s' % \
+sys.getdefaultencoding())
+    request.send_header('Server', config['version'])
+    request.end_headers()
+
 def webheader(request):
     """ create web header """
     request.wfile.write('<form action="dispatch">')
@@ -179,11 +200,35 @@ def webheader(request):
 % ' '.join(cmndslist))
     request.wfile.write('use chan #channel to provide a channel for \
 channel related commands<br>example: search http chan #dunkbots<br><br>')
-    for i in cfg.get('showplugs'):
-        request.wfile.write("<a href='/%s'>%s</a>" % (i, i))
-        request.wfile.write("    ")
+    #for i in cfg.get('showplugs'):
+    #    request.wfile.write("<a href='/%s'>%s</a>" % (i, i))
+    #    request.wfile.write("    ")
     request.wfile.write('<br>')
 
+def sendresult(request, result, how=[]):
+    if not result:
+        return
+    if type(result) == types.StringType:
+        result = [result, ]
+    for i in result:
+        output = i
+        if 'escape' in how:
+            output = escape(output)
+        request.wfile.write(output)
+        if 'break' in how:
+            request.wfile.write("<br>")
+            request.wfile.write("\n")
+    return True
+
+def makeresult(request):
+    try:
+        result = request.server.do(request)
+    except:
+        handle_exception()
+        return
+    if result:
+        return result
+
 class BotHTTPRequestHandler(BaseHTTPRequestHandler):
 
     """ bots request handler class """
@@ -192,63 +237,35 @@ class BotHTTPRequestHandler(BaseHTTPRequestHandler):
         """ called on GET """
         ip = self.client_address[0]
         if not cfg.get('whitelistenable') and ip in cfg.get('blacklist'):
-            rlog(5, 'webserver', 'denied %s' % ip)
+            rlog(100, 'webserver', 'denied %s' % ip)
             self.send_error(404)
             return
         if  cfg.get('whitelistenable') and ip not in cfg.get('whitelist'):
-            rlog(5, 'webserver', 'denied %s' % ip)
+            rlog(100, 'webserver', 'denied %s' % ip)
             self.send_error(404)
             return
-        pathin = unquote_plus(self.path)
-        if pathin.endswith('&how=direct'):
-            self.send_response(200)
-            self.send_header('Content-type', 'text/plain; charset=%s' % \
-sys.getdefaultencoding())
-            self.send_header('Server', config['version'])
-            self.end_headers()
-            self.path = self.path[:-11]
-            result = self.server.do(self)
-            for i in result:
-                self.wfile.write("%s\n\n" % escape(str(i)))
-            return
-        if pathin.endswith('&how=noescape'):
-            self.send_response(200)
-            self.send_header('Content-type', 'text/plain; charset=%s' % \
-sys.getdefaultencoding())
-            self.send_header('Server', config['version'])
-            self.end_headers()
-            self.path = self.path[:-13]
-            result = self.server.do(self)
-            for i in result:
-                self.wfile.write("%s\n" % str(i))
-            return
-        if self.path == '/':
-            self.send_response(200)
-            self.send_header('Content-type', 'text/html; charset=%s' % \
-sys.getdefaultencoding())
-            self.send_header('Server', config['version'])
-            self.end_headers()
+        if self.path == '/' or self.path.startswith('/dispatch'):
+            writeheaderhtml(self)
             self.wfile.write(focustxt)
             self.wfile.write("<h1>GOZERBOT</h1>")
             webheader(self)
             self.wfile.write(endtxt)
-            return
-        result = self.server.do(self)
-        if result:
-            self.send_response(200)
-            self.send_header('Content-type', 'text/html; charset=%s' % \
-sys.getdefaultencoding())
-            self.send_header('Server', config['version'])
-            self.end_headers()
-            self.wfile.write(focustxt)
-            self.wfile.write("<h1>GOZERBOT</h1>")
-            webheader(self)
-            self.wfile.write('<h2>results</h2>')
-            for i in result:
-                self.wfile.write("%s<br>" % escape(str(i)))
-            self.wfile.write(endtxt)
+            result = makeresult(self)
+            if result:
+                self.wfile.write('<h2>results</h2>')
+                sendresult(self, result, ['break', 'escape'])
+        elif self.path.startswith('/json'):
+            result = makeresult(self)
+            self.wfile.write(result)
         else:
-            self.send_error(404)
+            result = makeresult(self)
+            if result:
+                writeheaderplain(self)
+                sendresult(self, result)
+            else:
+                writeheaderplain(self)
+                sendresult(self, 'no data')
+        self.wfile.close()
 
     #localhost - - [18/Mar/2006 14:51:53] "GET / HTTP/1.1" 200 -
     def log_request(self, code, size='-'):
