@@ -6,7 +6,7 @@
 
 __copyright__ = 'this file is in the public domain'
 
-from gozerbot.generic import waitforqueue
+from gozerbot.generic import waitforqueue, handle_exception
 from gozerbot.fleet import fleet
 from gozerbot.ircevent import Ircevent
 from gozerbot.plugins import plugins
@@ -18,7 +18,7 @@ import Queue
 def handle_dispatch(event):
     """ dispatch web request """
     input = unquote_plus(event.path)
-    bot = fleet.getmainbot()
+    bot = fleet.getfirstbot()
     ievent = Ircevent()
     try:
         what = input.split('?', 1)[1]
