@@ -2,6 +2,8 @@
 #
 #
 
+""" show all karma items """
+
 __copyright__ = 'this file is in the public domain'
 
 from gozerbot.config import config
@@ -12,11 +14,12 @@ else:
 from gozerplugs.plugs.webserver import httpd
 
 def handle_karma(event):
+    """ show all pickled karma items """
     result = []
     resultlist = []
-    for i,j in karma.karma.iteritems():
+    for i, j in karma.karma.iteritems():
         resultlist.append((i,j))
-    resultlist.sort(lambda x,y: cmp(y[1], x[1]))
+    resultlist.sort(lambda x, y: cmp(y[1], x[1]))
     for i in resultlist:
         result.append("%s = %s" % (i[0], i[1]))
     if not result:
@@ -24,6 +27,7 @@ def handle_karma(event):
     return result
 
 def handle_karmadb(event):
+    """ show all database karma items """
     result = []
     dbresult = db.execute(""" SELECT item, value FROM karma ORDER BY \
 value DESC""")
