@@ -2,13 +2,16 @@
 #
 #
 
+""" return all collective nodes """
+
 __copyright__ = 'this file is in the public domain'
 
 from gozerplugs.plugs.webserver import httpd
-from gozerplugs.plugs.collective import nodesxml
+from gozerplugs.plugs.collective import colls
 
 def handle_nodes(event):
-    result = nodesxml()
+    """ return all collective nodes """
+    result = colls.nodesxml()
     if result:
         return [result, ]
     else:
